//VArrayEvnet2HDF5
//This program converts an FPIO VArrayEvent file to an HDF5 file
//This version for Run Data File Format V:0.1.2.
//   G.H.Sembroski
//   Physics Dept.
//   Purdue Univ.
//   W.Lafayette, In 47907
//sembroski@physics.purdue.edu
//   24/03/03

//Modified:


#include <DataStream/array_event_file_input.h>
#include "VHDF5RunDataFile.h"
#include "VPattern.h"
#include "run_file_header.h"
#include <iostream>
#include <vector>
using namespace std;


int main( int argc, char **argv )
 {
  int inID=0;
  int outID=0;
  while (int c = getopt(argc, argv, "i:o:") != -1)
    {
      switch (c)
	{
	case 'i':
	  inID=optind;
	case 'o':
	  outID=optind;
	case '?':
	  cout<<"Unknown option:"<<c<<endl;
	  return 1;
	}
    }
  if(inID==0)
    {
      cout<<"Input file name not specified (option -i)!"<<endl;
      return 0;
    }
  if(outID==0)
    {
      cout<<"Output file name not specified (option -o)!"<<endl;
      return 0;
    }
  char* filename = argv[inID];  //Run File name to process  
   //The following opens the file object (called 'input') 
   //VRunFileHeader::getInstance() is just a pointer to the static 
   //VRunFileHeader header class. This is used by 'VArrayEventFileInput input'
  //(look eventually at generic_file_input.h) to read in the first record as
  //a header record into 'header' 
  //This is convienent since we want to skip this record.
  VArrayEventFileInput input(filename, VRunFileHeader::getInstance());

  //Now set up the output HDF5 file. Create and open the file:
  filename = argv[outID];
  VHDF5RunDataFile output;
  output.createFile(filename); //Creates the ouput file.
  int eventRecordNumberHDF5File=0;

  //Now we need to set up the HDF5RunDataFile info record.
  //first the base info(and stuff we don't need an input record for. 
  //This includes stuff we would need from the data base.
  //(see below)):
  VHDF5InfoHeader* iHead = new VHDF5InfoHeader();



  iHead->softwareVersion=kVHDF5SoftwareVersion.c_str();
  //The following are obtained from the data base. For now give default values.
  iHead.arrayMode=1;  //Not sure what this is. Flags to indicate trigger type?
                      // (laser, pedestal etc.?)
  iHead.expectedDuration=28.*60.;  //in seconds(same as UTC times)
  iHead.ReservedFlags=0;  //Specifies special things about this file(may be 
                          //Tagged MC data for example)

  //The rest of the iHead can be set from data in the first event record
  //except for the UTCEnd and actualDuration. These must wait until we
  //read the last record. So don't write out the infoRec until the end of this
  //program.
  bool first=true;   //Flag to tell us this is first record read and we need
  //to get stuff out of it for  the infoRec
  //Start main read in loop

  //Define stuff
  vector< VHDF5TelescopeInfo > tInfo;  //Need an array of these.     

  VArrayEvent *ae;
  VHDF5EventTriggerHeader* eTrigHead = new VHDF5EventTriggerHeader();

  try 
    {
      for (int cnt=0;;cnt++) 
	{
	  ae=input.readNext();   //Readin the next event
	  
	} 
      //NULL means we should quit(error or no more events)
      if (ae==NULL) 
	{
	  break;
	}

      //Now we are ready to set up for the HDF5 output.
      if(first)
	{
	//Fill up the rest of the info record that we can do but don't write 
	//it out yet. We Still we need actualDuration and UTCEnd and total 
	//number of telescopes in total configuration. We get that 
	//stuff after we've read all the events.
          first=false;
	  iHead->runNumber==ae->getRun();    //VArrayEvent member function.
	  iHead->UTCStart=UTCConvert(ae->getTrigger()->getGPSTime());
	                                    //???Find method to do this
	}

      //Normal VarrayEvent  processing:
      //First the VArrayTrigger part


      eTrigHead->triggerMask=ae->getTrigger()->getTriggerMask();
      unsigned char tConfig= eTrigHead->triggerMask;

      eTrigHead->eventNumber=ae->getEventNumber();
      for(unsigned int j=0;j<5;j++)
	{
	  eTrigHead->GPSTime[j]=ae->getTrigger()->getGPSTimeElement(j);
	}
      eTrigHead->statusFlags=ae->getTrigger()->getFlags();
      eTrigHead->configurationMask=ae->getTrigger()->getConfigurationMask();
      unsigned char config=eTrigHead->configurationMask;
      int nTel=0;

      for(int j=0;j<7;j++)
	{
	  if(config & 1)
	    {
	      nTel++;
	      config >> 1;
	    }
	}
      eTrigHead->numberTelescopes=nTel;

      eTrigHead->numberTelescopesData=0; //init.
      //numberTelescopesData determined from data, not counting bits in
      //trigger mask!

      //Now Go through the VEvents in this VarrayEvent 
      //We use Filip's event Iterator.
      //Use vectors since it makes variable length single dimension arrays easy

      vector< VHDF5EventDataTelescope > eTelData; 
                                       //There is one
                                       //VHDF5EventDataTelescope per telescope
                                       //for each VEvent in this VarrayEvent

      //We go to the use of vectors of vectors below instead of vectors of 
      //arrays because by using the vector of vectors we don't have to worry 
      //about the cleanup ('delete') of any arrays created with 'new'. The 
      //destructor of the vector type will cleanup things as needed.

      vector< vector< unsigned long > > eHitPattern; 
                                       //There is a maxNumChannels/32
                                       //unsigned long words vector
                                       //for each VEvent in this VarrayEvent
      vector< vector< unsigned long  > > CFDPattern; 
                                       //There is a maxNumChannels/32
                                       //unsigned long words vector
                                       //for each VEvent in this VarrayEvent
      vector< vector< VHDF5ClkTriggerData > >  eClkData;
                                       //There is an nClkTriggerBoards long 
                                       //vector of VHDF5ClkTriggerData objects
                                       //for each VEvent in this VarrayEvent
     
      vector< vector< VHDF5DataChannel > > datChannel;
                                       //There is an vector nChannel long of  
                                       //VHDF5DataChannel objects for each hit 
                                       //FADC channels in
                                       //each VEvent in this VarrayEvent.
                                       //hit channels are specified by the
                                       //bits set in eHitPattern. 

      vector< vector< vector< unsigned short> > >  samplesFADC;
                                       //There is an nFADCSamples long 
                                       //unsigned short vector for
                                       //each hit channel in 
                                       //each VEvent in this VarrayEvent
                                       //Hit channels are specified by the
                                       //bits set in eHitPattern. 
      
      unsigned long* hPattern;
      int eventCount=0;
      for (VArrayEvent::EventIterator i=ae->getEventIterator();i.hasNext();)
	{
	  VEvent *event=i.getNext();
	  eventCount++;
	  eTrigHead->numberTelescopesData=eventCount;

//****************************************************************************
      //First look to see if we find VEvents for telescopes we haven't put in 
      //tInfo vector yet.
	  int teleID= event->getNodeNumber(); 
	      //Search through tInfo to see if we have this telescope yet.
	  bool use=false;	
	  int tInfoNum=tInfo.size();
	  if(tInfoNum==0)
	    {
	      use=true;
	    }
	  else
	    {
	      use=true;
	      for(int j=0;j<tInfoNum; j++)
		{
		  if(tInfo[j].telescopeID==teleID)
		    {
		      use=false;
		      break;
		    }
		}
	    }
	  if(use)        //Put this into tInfo
	    {
	      tInfo.resize(tInfoNum+1); //Add another.
	      tLast=tInfo.size();
	      iHead.nTelescopes=tLast;
	      //Fill it up
	      tInfo[tLast].telescopeID=teleID;
	      tInfo[tLast].maxNumberChannels=event->getMaxNumChannels();
	      tInfo[tLast].nTrigger=379; //Fudge:Should come from DB;
	      tInfo[tLast].numberClockTriggerBoards=
		event->getNumClockTrigBoards();
	      tInfo[tLast].numberFADCSamples=event->getNumSamples();
	    }

//****************************************************************************

//****************************************************************************
//Now we can load up the VEvents.
//First the eTelData vector of structs.
	  eTelData.resize(eventCount);
	  unsigned int telID=event->getNodeNumber(); 
	  eTelData[eventCount].nodeID=telID;
	  eTelData[eventCount].eventType=event->getEventType();
	  eTelData[eventCount].rightAscension=
	                            ae->getTrigger()->getRightAscension(telID);
	  eTelData[eventCount].Declination=
		                    ae->getTrigger()->getDeclination(telID);
	  eTelData[eventCount].TDCCounts=
                                    ae->getTrigger()->getTDC(telID);
	  eTelData[eventCount].delay=
                                    ae->getTrigger()->getDelay(telID);
	  eTelData[eventCount].CFDScalerRate=
		                    ae->getTrigger()->getL2ScalerRate(telID);
	  
	  for (int k=0;k<3;k++)
	    {
	      eTelData[eventCount].PSTPattern[k]=
		ae->getTrigger()->getL2PatternElement(telID,k);
	    }
	  
	  for(int k=0;k<5;k++)
	    {
	      eTelData[eventCount].GPSTime[k]=event->getGPSTimeElement(k);
	    }
	  eTelData[eventCount].statusFlags=event->getFlags();
	  eTelData[eventCount].numberChannels=event->getNumChannels();

	      //Now the Patterns
	  int patternSize=(event->getMaxNumChannels()+31)/32;
      
	      //Hit Pattern
	  eHitPattern.resize(eventCount); 
	  eHitPattern[eventCount].resize(patternSize);
	  hPattern=new unsigned long(patternbSize);  //Temp for use later
	      //******************Don't forget to delete this later!


	      //CFD Pattern
	  eCFDPattern.resize(eventCount);
	  eCFDPattern[eventCount].resize(patternSize);
	                //******************Don't forget to delete this later!
	  for(j=0;j<patternSize;j++)
	    {
	      eHitPattern[eventcount][j]=event->getHitPatternWord(j);
	      hPattern[j]=event->getHitPatternWord(j);
	      eCFDPatttern[eventCount][j]=event->getTriggerPatternWord(j);
	    }
	      //Clock Trigger Board data
	  eClkData.resize(eventCount);
	  eClkData[eventCount].resize(event->getNumClockTrigBoards());
	  for(int k=0;k<event->getNumClockTrigBoards();k++)
	    {
	      for(j=0;j<sizeof(eClkData[eventCount][k].clkTriggerData);j++)
		{
		  eClkData[eventCount][k].clkTriggerData[j]=
		                      event->getClockTriggerDataElement(k,j);
		}
	    }

	      //Finally the FADC Data
	  int nChan=event->getNumChannels();
	      //First define the dat channel. 
	  datChannel.resize(eventCount);
	  datChannel.[eventCount]resize(nChan);
	     //Define the samples
	  samplesFADC.resize(eventCount);
	  samplesFADC.[eventCount]resize(nChan);
	  samplesFADC.[eventCount][nChan].resize(event->getNumSamples());

	  VPattern hitPat(event->getMaxNumChannels(),hPattern);
	  delete hPattern;  //Done with it.
	  int jChan=0;
	  for(int j=0;j<event->getMaxNumChannels();j++)
	    {
	      if(hitPat.isBitOn(j))
		{
		  //Found a hit, fill the datChannel
		  datChannel[eventCount][jChan].channelID=j;
		  datChannel[eventCount][jChan].pedestalAndHiLow=
		                          event->getPedestalAndHiLo(jChan);
		  datChannel[eventCount][jChan].charge=
		                          event->getCharge(jChan);
		      //Fill the samples
		  for(k=0;k<event->getNumSamples();k++)
		    {
		      samplesFADC.[eventCount][jChan][k]=
			event->getSample(jChan,k);
		      jChan++;
		    }
		}
	    }

	}
      //Everything is loaded for this event now. Write it out to the HDF5 file.
      eventRecordNumberHDF5File++;
      output->writeEvent( eventRecordNumberHDF5File, eTrigHead, eTelData,
			  eHitPattern, eCFDPattern, eClkData, datChannel,
			  samplesFADC);
    }
  catch (FPException &e) 
    {
      cout<<"Error: %s",e.what()<<endl;
    }

      //end of run stuff here.
  iHead->UTCSEnd=UTCConvert(ae->getTrigger()->getGPSTime());
	                                    //???Find method to do this
  iHead->actualDuration=iHead->UTCEnd-iHead->UTCStart;
	  //write out the info record.
  output->writeInfoRec(iHead,tInfo);
  output->closeFile();
  return 0;
 }
