//VEventD.h
//! Wrapper class for VEvent.
//!Upgraded to Revised datum.h 19/02/02 GHS.
#ifndef VEVENTD_H
#define VEVENTD_H
#include "words.h"
#include "datum.h"

//VPattern used by VEventD
class VPattern
{
 private:
  word32* f_Pattern;
 public:
  VPattern(){};
  ~VPattern(){};
  bool isBitOn(word32 channel);
  void setBit(word32 channel);
  void clearBit(word32 channel);
  void initPattern(word32* parray);
  void setPattern(word32* parray){f_Pattern=parray;return;};
};


/*! Wrapper class for VEvent. Using the "Delegater Pattern". That is, this 
 *  class acts as an interface to an instance of VEvent. This means we only
 *  need one instance of this class. We just change which VEvent we point to 
 *  using setVEvent() or a getVEvent().  We don't make this a derived class of
 *  VEvent( which would require us to make a new instance of this class for 
 *  every new Vevent, of which we may have a large number, (> 5000)at any one 
 *  time ). Instead this class will have a private pointer to VEvent which is 
 *  pointed to a particular VEvent by the calls to setVEvent or getVEvent.
 *  Note:This structure does not prevent the user from directly calling VEvent 
 *  member functions, thus bypassing this interface.
 */

/* The following member functions (or their results) of VEvent are NOT 
 * accesable from this class:
 * isValid  Probably I should add checks on this.
 * getData
 * clearDatum
 * getDataAfterHeader
 */

class VEventD {
 protected:
  VEvent* f_Event;    //Pointer to an instance of VEvent.
  char* f_Data;  //Pointer used when reading events from a file (see 
                      //readFromFile member function)
 private:
  VPattern f_ChannelHits;
  VPattern f_ChannelTriggers;
  VPattern f_ChannelHiLo;
 
 public:
  VEventD(){};
  ~VEventD(){};
                           //Caller must always creates Event

  void setEvent(VEvent* Event);

                            //This reads from input file an event into f_Event
  word32 getEvent(FPInput* in){return f_Event->readFrom(in);};
                                            //Write out f_Event to output file.
  word32 putEvent(FPOutput* out){return f_Event->writeTo(out);};
            // returned status of above 2 operations getEvent and setEvent: 
            // (1=ok,0=empty header,-1 error)
  word32 configureEvent(word32 telescopeId, word32 num_samples,
		     word32 num_channels)
    {return f_Event->configureEvent((word16) telescopeId, 
				     num_samples, num_channels);};
  word32 configureEvent(word32 telescopeId, word32 num_samples,
		     word32 num_channels,word32 extra)
    {return f_Event->configureEvent((word16) telescopeId, 
				     num_samples, num_channels,extra);};

//Data retrival member functions.
  word32 getEventLength(){return word32(f_Event->getLength());};
  word32 getTelescopeID(){return word32(f_Event->getNodeNum());};
  word32* getGPSTime(){return f_Event->getGPSTime();};  
  word32 getNumSamples(){return f_Event->getNumSamples();};
  word32 getNumChannels(){return f_Event->getNumChannels();};
  
  bool getStatus(word32 flagNumber);
  bool isChannelHit(word32 channel){return f_ChannelHits.isBitOn(channel);};
  bool isChannelTrigger(word32 channel){
    return f_ChannelTriggers.isBitOn(channel);};
  bool isChannelHiLo(word32 channel){return f_ChannelHiLo.isBitOn(channel);};
  word32 getChannelNum(word32 fadcIndex){
    return f_Event->getChannelNum(fadcIndex);}; 
  bool getFadcIndex(word32  channel, word32 fadcIndex);
  byte* getFadcSamples(word32 fadcIndex)
                                     {return f_Event->getSamples(fadcIndex);};
  word32 getFadcPedestal(word32 fadcIndex)
                            {return (word32)f_Event->getPedestal(fadcIndex);};
  word32 getFadcCharge(word32 fadcIndex)
                              {return (word32)f_Event->getCharge(fadcIndex);};


  //Data setting functions(to load up Vevent for output)

  void putTelescope(word32 telescopeId){
    f_Event->setNodeNum((word16)telescopeId);return;};
  void setStatus(bool flagValue,word32 flagNumber);

  void setChannelHit(word32 channel){f_ChannelHits.setBit(channel);return;};
  void setChannelTrigger(word32 channel){f_ChannelTriggers.setBit(channel);
                                                                  return;};
  void setChannelHiLo(word32 channel){f_ChannelHiLo.setBit(channel);return;};
  void clearChannelHit(word32 channel){f_ChannelHits.clearBit(channel);
  return;};
  void clearChannelTrigger(word32 channel){f_ChannelTriggers.clearBit(channel);
                                         return;};
  void clearChannelHiLo(word32 channel){f_ChannelHiLo.clearBit(channel);
                                         return;};
  void setGPSTime(word32* gpsTime){f_Event->setGPSTime(gpsTime);return;};
  void putFadcChannelNum(word32 fadcIndex,word32 channel)
                           {f_Event->setChannelNum(fadcIndex,channel);return;};
  void putFadcSamples(word32 fadcIndex, byte* samples)
                              {f_Event->setSamples(fadcIndex,samples);return;};
  void putFadcPedestal(word32 fadcIndex, word32 pedestal)
                   {f_Event->setPedestal(fadcIndex,(word16)pedestal); return;};
  void putFadcCharge(word32 fadcIndex, word32 charge)
                        {f_Event->setCharge(fadcIndex,(word16)charge);return;};

};
#endif //VEVENTD_H

