//VHDF52Ntpl.cpp
//This program reads a VHDF5 file with a PARAM dataset and creates a Whipple 
//paw ntupple file. If a KASCADE dataset also exists it adds that stuff to
//the file also.
//   G.H.Sembroski
//   Physics Dept.
//   Purdue Univ.
//   W.Lafayette, In 47907
//sembroski@physics.purdue.edu
//   19/02/04

//Modified:
// **************************************************************************

#include "VHDF5KascadeRunDataFile.h"
#include "exception.h"
#include <iostream>
#include <vector>
#include <getopt.h>
using namespace std;

int main( int argc, char **argv )
 {
  bool inID=false;
  bool outID=false;
  char* iFilename="";
  char* oFilename="";
  int c;   //c MUST be declared outside the while
           //(gcc compiler 'feature' 3.0.4)
  while ((c = getopt(argc, argv, "i:o:h")) != -1)
//Note for Myself:In the getopt call, in the allowed options list, an option 
//followed by a ':' is required to be followed by an option argument. An 
//option not folowed by a ':' has no following option argumnet. Thus in 
//"i:to:" above, options -i and -o must have arguments, option -t must not!

    // followed by a : means it expects an argument to follow
    {
      switch (c)
	{
	case 'i':
	  cout<<"Input file (option -i): "<<optarg<<endl;
          iFilename=optarg;
          inID=true;
	  break;                 //This break just gets us out of the switch
                                 //not the while!
	case 'o':
	  cout<<"Output file (option -o): "<<optarg<<endl;
	  oFilename=optarg;
          outID=true;
	  break;                 //This break just gets us out of the switch
                                 //not the while!
	case 'h':
	  cout<<"VHDF52Ntpl options:"<<endl;
	  cout<<"-i filename :Input VHDF5 file name:required"<<endl;
	  cout<<"-o filename :Output PAW Ntupple file name:required"<<endl;
          cout<<"-h          :Generate this listing of options"<<endl;
	  break;                 //This break just gets us out of the switch
                                 //not the while!
	case '?':
	  cout<<"Unknown option:"<<c<<endl;
	  return 1;
	}
    }
  if((!inID))
    {
      cout<<
	"Fatal--Input VHDF5 file name not specified (option -i)!"<<
	endl;
      return 0;
    }

  if(!outID)
    {
      cout<<"Fatal--Output Paw Ntupple file name not specified (option -o)!"<<
	endl;
      return 0;
    }
    
 
  //Now set up the input VHDF5 file. Open the file:
  try
    {
      VHDF5KascadeRunDataFile input(); 
      input.openFile(iFilename); //Opens the input file.
	                         //Sets numberFADCSamples to whatever file 
	                         //has.
  //The following opens the file object (called 'output') 
  //For us the VRunFileHeader object is setup with defaults. It is written
  //out as the first record by 'VArrayEventFileOutput.
  //(look eventually at generic_file_output.h)
       	// set up array event input
      VRunFileHeader run_header;
      VArrayEventFileOutput output(oFilename,&run_header); 
                                              //Writes out run header record.
 
      //Now we need to read in the HDF5RunDataFile info record.
      //Define stuff
      VHDF5InfoHeader* iHead = new VHDF5InfoHeader();
      vector< VHDF5TelescopeInfo > tInfo;          //Need an array of these.

      input.readInfoRec(iHead,tInfo);            //read in  the info record.
      //printout some of this stuff.
      if(verbal)
	{
	  input.printInfoRec(iHead,tInfo,vLevel);
	}
   //to get stuff out of it for  the infoRec
      VHDF5EventVA Event;  //This class inherits from VHDF5Event

  //Start main read in loop
  //Start the event read in loop. First get the number of events in this file
      int nEvents= input.getNumberEventsInFile();
      if(verbal)
	{
	  cout<<"Number of Events in input VHDF5 file:"<<nEvents<<endl;
	}
      //bool first=true;
      for (int recNumber=0;recNumber<nEvents;recNumber++) 
	{
	  AutoreleasePool arp;   //Make cleanup of ae stuff automatic. A Filip 
	                         //DAST thing
	  //note:recordNumber starts at 0(0 is first event in file)
	  //define the output VArrayEvent
	  VArrayEvent *ae = RCAutorelease(new VArrayEvent());
	  ae->setRun((long)iHead->runNumber);

	  input.readEvent(Event,recNumber);
	  if(verbal)
	    {
	      Event.printEvent(vLevel);
	    }
	  Event.extract(ae,iHead,tInfo);//moves a copy of  Event into ae
	      //This VArrayEvent Complete, write it out.
	  output.writeNext(ae);
	}     //End of loop of VHDF5 event records in HDF55 file. 
      input.closeFile();   //close input HDF5 file. Output closed by destructor
    }  //End of main try loop
  catch (FPException &e) 
    {
      cout<<"Error: %s"<<e.what()<<endl;
    }
  catch(VException &e)
    {
      cerr<<e;
      return 1;
    }
  catch(...)
    {
      cout<<"Fatal--Unknow exception found."<<endl;
      return 1;
    }
  return 0;
 }

