/*
 * VTaggedEvent.h -- Extension of VEvent for MonteCarlo generated events
 * Extends VEvent to include Tag data at the end.
 * by Glenn Sembroski 19/02/02
 */

#ifndef V_TAGGED_EVENT_H
#define V_TAGGED_EVENT_H


#include "datum.h"

struct TagData           //Define this so that we get the size of the tags.
{
  float tep;
  word32 idFile;
  word32 type;
  word32 iTheta;
  word32 iPhi;
  float xm;
  float ym;
  float showerMax;
};

class VTaggedEvent: public VEvent {
 private:
  TagData  f_Tags;     //To insure we keep TagData up-to-date when we add more 
                       //tags use f_Tags structure to temporarily keep the 
                       //tags before we enter them into the datum record.
 
 public:
  VTaggedEvent();
  ~VTaggedEvent();
  word32 getTagSize(){return sizeof(f_Tags);}

  void getTags(word32 numSamples, word32 numChannels);
  float getShowerEnergyTev(){return f_Tags.tep;};
  word32 getShowerId(){return f_Tags.idFile;};
  word32 getShowerType(){return f_Tags.type;};
  word32 getMountITheta(){return f_Tags.iTheta;};
  word32 getMountIPhi(){return f_Tags.iPhi;};
  float getMountLocationX(){return f_Tags.xm;};
  float getMountLocationY(){return f_Tags.ym};
  float getShowerMax(){return f_Tags.showerMax;};


  void setShowerEnergyTev(float energy){f_Tags.tep=energy; return;};
  void setShowerId(word32 fileId){f_Tags.idFile=fileId; return;};
  void setShowerType(word32 itype){f_Tags.type=itype; return;};
  void setMountITheta(word32 theta){f_Tags.iTheta=theta; return;};
  void setMountIPhi(word32 phi){f_Tags.iPhi=phi;return;};
  void setMountLocation(float x, float y){f_Tags.xm=x; f_Tags.ym=y; return;};
  void setShowerMax(float shwrMax){f_Tags.showerMax=shwrMax;return;};
  void putTags(word32 numSamples, word32 numChannels);
};
#endif

