MODULE HDFNTP
!Modified:
!	27/07/01 GHS-MPK 
!		Add sdev_xy and alph for 2-d analysis.
	type par_s
		sequence		!Necessay because we want to put this
					!into a common block for hdf
  		real*8  :: gpstime
  		real*8  :: osctime
  		real*8  :: livetime
		real*4  :: length
		real*4  :: width
		real*4  :: miss
		real*4  :: distance
		real*4  :: azwidth
		real*4  :: frac(3)
		integer :: size
		integer :: location(3)
		real*4  :: maximum(3)
		real*4  :: asymm
		real*4  :: xc
		real*4  :: yc
		real*4  :: xo
		real*4  :: yo
                real*4  :: sdev_xy
                real*4  :: alph
                real*4  :: weight
                real*4  :: tep           !Monte-Carlo shower energy in TeV
                integer :: idfile        !Monte-Carlo shower id number
                real*4  :: muon_ratio    !Ratio of Mu PES in image to total
             end type par_s

CONTAINS

    SUBROUTINE NTUPLE_CWN_HILLAS(nt,ntold)
!******************************************************************************
!	This routine enteres data from ARRAY HNTP into ntuple nt.
!******************************************************************************
!	This is the version that makes a column wide ntuple.
!	HNTP is put into a common block in calling routine.

!	Modified:

!	10/4/96 GHS
!
!		Increase max number of records to maximum zera allows
!		which is 64 k (default=32000)
!		Requires 'Q' option in HROPEN and setting of RZ variable
!		IQUEST(10) in RZ common are QUEST.
!		If this value is too small one gets the error:
!		      'RZOUT . Request exceeds quota'	
!		May also be able to increase record size to solve this kind
!		of problem.
!		PAW FAQ says:
!PAW FAQ !!!!!!!!!!!!!!!!!!!!!!!!
!cIt may happen with the current versions of HBOOK/PAW that large ntuples
!cgive problems if the following process is not followed. To create large Ntuple
!cfiles, you can : 
!c
!cCALL HROPEN with a large value of LRECL (ex LRECL=4096 instead of
!c1024) and you can get files 4 times bigger.
!c
!cWhen you do that, do not forget in PAW (in Hi/File) to specify the LRECL
!cparameter.
!c
!cIf changing LRECL is not sufficient, you can also specify the option Q in
!cCHOPT and in IQUEST(10) (COMMON/QUEST/IQUEST(100)) give the
!cmaximum number of records (current default is 32000 records in CERNLIB
!c94A)
!c
!cExample: 
!c
!c    COMMON/QUEST/IQUEST(100)
!c    IQUEST(10)=64000
!c    CALL HROPEN(lun,......,'NQ',4096,istat)
!c
!cwill create a file with a maximum of 64000 records of 4096 words.
!cTo minimize the number of keys in the HBOOK/RZ directory, you can also
!cincrease the buffer size for your ntuple.
!cIf it is a CWN created by HBNT/HBNAME
!cSet the buffer size for each column via CALL HBSET('BSIZE',nwords) A
!cconvenient value for nwords is the record size itself (eg,4096 words)
!cPAW FAQ !!!!!!!!!!!!!!!!!!!!!!!!

!	12/2/98 GHS	Convert to F90

!	24/6/98 GHS V:1:1:5:1:2:1.2
!		Modify NTUPLE_CWN_HILLAS to limit size of ntuple files
!		to 2000000 events(1000000 blocks). Each new file' name ends
!		with *.*_v1,*.*_v2 etc.  Do this to avoid some PAW problems
!		with really big ntupples. Also makes it easier to put on tape.

!	25/6/98 GHS V:1:1:5:1:2:1.2
!		Put in a test so that when we make a new ntupple file we don't
!		split up a shower doing it. That caused problems in
!		KAS_ARRAY_TRIGGER

!	22/1/99 GHs
!		Change source of output file name to command argument p1
!		Change file extension to *_ntp

!	06/3/00 GHS
!		Use output file name of ntupple.out. Most useful approache for
!		Linux system.

	use structures
	IMPLICIT NONE

	integer :: nt,ntold,itfirst=0

	real,dimension(h_ntp_size) ::  ntp_h
	type(h_ntp) :: hntp
	common/ntp/hntp
	equivalence (ntp_h,hntp)

	integer,dimension(100) :: iquest
	common/quest/iquest

	real,dimension(2000000) :: paw
	common /pawc/paw	!for ntuple creation

	integer :: lrec=4096,lsize,ifirst,isymst,istat,icycle,i,nchar
	integer :: nt_count,nv,idold
	integer,parameter :: nt_count_max=2000000
	character(len=80) :: namefile
	character(len=2) :: nv_char


	if(itfirst.eq.0) then
!	Open the ouput file.(These are all hbook commands.)

		iquest(10)=64*1024-1	!Set max number of records in ntupple
					!file. Default is 32000 blocks.
		nv=1
		write(nv_char,1000)nv
1000	format(i1)
		namefile="ntupple.out"//nv_char
		call hropen(4,'NTUPLE',namefile,'NQ',lrec,istat)
					!Open unit 4, with arbitrary 
					!HBOOK name:'ntuple'
					!file name from 'p2'
					!File is 'N'ew file
					!1024=lrec.Block size(recomended)
		if(istat.ne.0)then
			print*,' istat:',istat
			print*,' namefile:',namefile
			stop 'hropen problem'
		else
			print*,' Hillas NTUPPLE opened OK! File version#v',nv
			itfirst=1
			idold=0
		endif
	endif

	if(nt.ne.ntold)then
		if(ntold.gt.-1000000)then
			call hrout(ntold,icycle,' ')
			call hcdir('//PAWC',' ')
			call hdelet(ntold)
		endif
!		Define the variables.
!		For now h_ntp is all real, so its easy, but if we need to
!		go to Integers or logicals we can do that here also.
!		Also we could compress it here also with a little work, but
!		not now.(see hbook manual pg.24-27)

		call hcdir('//NTUPLE',' ')
		call hbnt(nt,'PES',' ')
		do i=1,h_ntp_size
			call hbname(nt,'H_NTP',ntp_h(i), &
					 &   tags_h_ntp(i)//':R')
					!ntuple id is nt
					!Block Name is 'H_NTP'
					!Tags_h_ntp=names of the varables
					!assumed to all be real, no compression
		enddo
		ntold=nt
		nt_count=0
	endif
	
	if(nt_count>=nt_count_max)then		!time to close this file and
						!open a new one.
 		if(idold==0)then		!But wait until we finish this
						!shower
			idold=hntp%id
		elseif(hntp%id/=idold)then	!Test for the new one.
		       	call hrout(ntold,icycle,' ')
		       	call hcdir('//PAWC',' ')
		       	call hdelet(ntold)
			call hrend('NTUPLE')
			close(unit=4)

			nv=nv+1
			if(nv<10)then
				write(nv_char,1000)nv
			else
				write(nv_char,1001)nv
1001	format(i2)
			endif
			namefile="ntupple.out"//nv_char
			call hropen(4,'NTUPLE',namefile,'NQ',lrec,istat)
					!Open unit 4, with arbitrary 
					!HBOOK name:'ntuple'
					!file name from 'p2'
					!File is 'N'ew file
					!1024=lrec.Block size(recomended)
			if(istat.ne.0)then
				print*,' istat:',istat
				print*,' namefile:',namefile
				stop 'hropen problem'
			else
			print*,' Hillas NTUPPLE opened OK! File version#v',nv
				itfirst=1
			endif
				!define new file
			call hcdir('//NTUPLE',' ')
			call hbnt(nt,'PES',' ')
			do i=1,h_ntp_size
				call hbname(nt,'H_NTP',ntp_h(i), &
					 &   tags_h_ntp(i)//':R')
			enddo
			idold=0
			nt_count=0
		endif
	endif



		!	Enter this image
	call hfnt(nt)
	nt_count=nt_count+1
	return
    END SUBROUTINE NTUPLE_CWN_HILLAS
!******************************************************************************

END MODULE HDFNTP

    PROGRAM HDF2NTP

!******************************************************************************
!	Converts the Hilass parameter HDF file generated by CPARMS by the
!	Purdue Data Analysis (PDA) sytem into a simular Ntupple file for
!	use with PAW.
!******************************************************************************

!Modified:

!       GHS 01/06/01
!              Add stuff for Monte-carlo runs. read in tags.

	use structures
	use hdfntp

	IMPLICIT NONE

	type(par_s) :: hhdf		!HDF hillas paramter structure.
	type(h_ntp) :: hntp
	common/ntp/hntp
	common/h_hdf/hhdf

	integer,parameter :: n_lwords_per_rec=26

	real,dimension(n_lwords_per_rec) ::   databuf
	equivalence  (databuf,hhdf)

	integer :: nt=1,ntold=-10000000,icycle,nchar
	real :: alpha_temp

!  	character(len=20),dimension(16) : : vparinfo_name = (/&
!		& 'VERSION','NADC','UTC_START','IDATE','ITIME',&
!		& 'SOURCE_NAME','SOURCE_RA','SOURCE_DEC',      &
!		& 'GAINLABEL','N2ID','PADLABEL','NTUBESOFF',   &
!		& 'TUBESOFF','PICTURE','BOUNDARY','EFACTOR'/)

  	character(len=20),dimension(1) :: vparinfo_name = (/&
             & 'SOURCE_NAME'/)


!	character(len=132) ::  vparinfo_name_str = &
!		& 'VERSION,NADC,UTC_START,IDATE,ITIME,SOURCE_NAME,&
!		&SOURCE_RA,SOURCE_DEC,GAINLABEL,N2ID,PADLABEL,NTUBESOFF,&
!		&TUBESOFF,PICTURE,BOUNDARY,EFACTOR'

	character(len=132) ::  vparinfo_name_str = &
		& 'SOURCE_NAME'

!	character(len=132),dimension(17) :: vpar_name = (/&
!	      & 'GPSTIME','OSCTIME','LIVETIME','LENGTH','WIDTH','MISS',&
!	      & 'DISTANCE','AZWIDTH','FRAC','SIZE','LOC','MAX',        &
!	      & 'ASYMM','XC','YC','XO','YO'/)

	character(len=132) :: vpar_name_str = &
		& 'GPSTIME,OSCTIME,LIVETIME,LENGTH,WIDTH,MISS,DISTANCE,&
		&AZWIDTH,FRAC,SIZE,LOC,MAX,ASYMM,XC,YC,XO,YO,sdev_xy,alph'

	character(len=132) :: vpar_mc_name_str = &
		& 'GPSTIME,OSCTIME,LIVETIME,LENGTH,WIDTH,MISS,DISTANCE,&
		&AZWIDTH,FRAC,SIZE,LOC,MAX,ASYMM,XC,YC,XO,YO,sdev_xy,alph, &
                &weight,tep,idfile,muon_ratio'

      integer,parameter  :: DFACC_READ=1, FULL_INTERLACE=0
      character(len=80) ::source_name
      logical :: mc_run
      logical :: first=.true.
!     Function declaration
      integer :: hopen, hclose
      integer :: vfstart, vsffnd, vsfatch, vsfsfld, vsfrd, vsqfnelt, &
 		&        vsfdtch, vfend 

!C**** Variable declaration *******************************************
      integer :: status,file_id,vdata_id,vdata_ref
      integer :: vdata_head_id,vdata_head_ref

	integer :: n_records,i


!C**** End of variable declaration ************************************
	call hlimit(2000000)	!Defines maximum total size of PAWC

				!Get input hdf file name.
				!Open HDF file for reading
        file_id = hopen('hdf.in', DFACC_READ, 0)
				!Initialize the VS interface.
	status = vfstart(file_id) 

!***************************************************************************
!   GHS 01/06/01
!    First we want to determine if this is a monte-carlo generated run.
!    Read the source_name from the header
!***************************************************************************
	!Get the reference number of the vdata.
	vdata_head_ref = vsffnd(file_id, '10M Parameter Info')
                       		!Attach to the vdata for reading if it is 
				!found,otherwise exit the program.
        if (vdata_head_ref .eq. 0) stop 'Failed to Attach to Vdata of Run info'
        vdata_head_id = vsfatch(file_id, vdata_head_ref, 'r') 
					!Specify the fields that will be read. 
        status = vsfsfld(vdata_head_id, vparinfo_name_str)
                    !read HDF record into hhdf.
        i = vsfrd(vdata_head_id, source_name, 1,FULL_INTERLACE)
        if(i /= 1) then
           stop 'Failure to read in runinfo record from HDF4 file'
        endif
        if(index(source_name,"MONTE-CARLO")/=0)then
           mc_run=.true.
           print*,' Processing a Monte-Carlo generated run.'
        else
           mc_run=.false.
        endif
        status = vsfdtch(vdata_head_id)  !Detach from this vdata
!***************************************************************************

	!Get the reference number of the vdata
        vdata_ref = vsffnd(file_id, '10M Parameters')

                       		!Attach to the vdata for reading if it is 
				!found,otherwise exit the program.
      if (vdata_ref .eq. 0) stop 'Failed to Attach to Vdata'
      vdata_id = vsfatch(file_id, vdata_ref, 'r') 

!***************************************************************************
! GHS 01/06/01: For MOnte-Carlo runs add tags to input list.
!***************************************************************************
					!Specify the fields that will be read. 
      if(mc_run)then
         status = vsfsfld(vdata_id, vpar_mc_name_str)
      else
         status = vsfsfld(vdata_id, vpar_name_str)
      endif


!***************************************************************************
!Main loop to read hdf records and write ntupple records.
!***************************************************************************
				!hntp is the MC version of the ntupple. Leave
				!some things set blank.
	hntp%tep=0. 		!Shower energy
        hntp%type=0            !Primary type
        hntp%id=0              !Shower id
        hntp%x_offset=0        !Shower core offset position
        hntp%y_offset=0
        hntp%x=0               !X(e-w) position
        hntp%y=0               !Y(n-s) position
        hntp%theta=0           !Zenith angle of mount
        hntp%phi=0             !Azmutial angle
        hntp%bright=0
        hntp%boundry=0
        hntp%w_hits_in=0
        hntp%w_hits_mid=0
        hntp%neighbor=0
        hntp%emission_alt=0
        hntp%em_alt_sig=0
        hntp%muon_ratio=0
        hntp%alph=0            !Varible used to get slope.
        hntp%sdevxy=0          !Varible used to get slope.
        hntp%psi=0
        hntp%asym=0
        hntp%emaltmpln=0       !Distance along pramary track from
	hntp%max_pe=0.
        hntp%clstr_thrsh=0.			!vector Arith.
	hntp%in_clstr_thrsh=0.                  !vector Arith.
        hntp%max_p=0.				!vector Arith.
        hntp%i_mult=0.
        hntp%fp_mult=0.
        hntp%xmpln=0.
        hntp%ympln=0.
        hntp%max_pe_in=0.
        print*,' hi there'
	do
				!read HDF record into hhdf.
		i = vsfrd(vdata_id, hhdf, 1,FULL_INTERLACE)
		if(i /= 1) exit	!done!
				!load various parts of this record into hntp
				!hntp is the MC version of the ntupple. Leave
				!some things set blank above.
                hntp%azwidth=hhdf%azwidth
                hntp%width=hhdf%width
                hntp%length=hhdf%length
                hntp%dist=hhdf%distance
                hntp%miss=hhdf%miss
!********************************************************************
!27/07/01 MPK-GHS
		hntp%sdevxy=hhdf%sdev_xy
		hntp%alph=hhdf%alph
!********************************************************************


                if(hntp%dist>0) then
                       	alpha_temp=(abs(hntp.miss/hntp.dist))
			if(alpha_temp>=1.0)then
	                       hntp%alpha=90.
			else
	                       hntp%alpha=asind(alpha_temp)
			endif
                else
                       hntp%alpha=-1.
                endif
                hntp%size=hhdf%size




                hntp%xmean=hhdf%xc           !X of Center of mass of Image
                hntp%ymean=hhdf%yc           !Y of Center of mass of Image
 
                hntp%asym=hhdf%asymm

		hntp%max_p(9)=hhdf%maximum(1)	
		hntp%max_p(10)=hhdf%maximum(2)
		hntp%max_p(11)=hhdf%maximum(3)

				!Below are compromizes for things from data 
				!which don't have an obvious place in hntp.
                hntp%max_pe_in(1)=hhdf%location(1)  !PMT # of biggest ADC
                hntp%max_pe_in(2)=hhdf%location(2)  !PMT # of 2nd biggest ADC
                hntp%max_pe_in(3)=hhdf%location(3)  !PMT # of 3rd biggest ADC
!                if(first)then
!                   print*,' xo,yo:',hhdf%xo,hhdf%yo,hhdf%alph
!                   first=.false.
!                endif
                hntp%xmpln=hhdf%xo       !2d determined source location
                hntp%ympln=hhdf%yo

                hntp%median_time=aint(hhdf%gpstime)	!Whole GPS seconds
						!Fraction of second of GPS time
                hntp%fp_median_time=hhdf%gpstime-hntp%median_time 

                hntp%time_cfd=aint(hhdf%osctime)        !Whole OSC clock seconds
						!Fraction of second of OCS time
                hntp%fp_time_cfd=hhdf%osctime-hntp%time_cfd

                hntp%i_threshold=aint(hhdf%livetime)  !Whole livetime seconds
						!Fraction of second of livetime
                hntp%fp_threshold=hhdf%livetime-hntp%i_threshold
				
!***************************************************************************
! GHS 01/06/01   Add tags if this is Monte-Carlo generated run.
!                  Except for "weight'. It can get regenerated from "tep" and
!                  by use of sum_init.kumac
!***************************************************************************
                hntp%tep=hhdf%tep               !Shower energy
                hntp%id=hhdf%idfile         !Shower ID
                hntp%muon_ratio=hhdf%muon_ratio !Muon pes ratio
!***************************************************************************

!Write hntp to ntupople(initalizes on first 
				!call.
	    	call ntuple_cwn_hillas(nt,ntold)
	enddo

!     Terminate access to the vdata and to the VS interface, and
!     close the HDF file.
      status = vsfdtch(vdata_id)
      status = vfend(file_id)
      status = hclose(file_id)


!	Close up ntupple
	nt=1
	call hcdir('//NTUPLE',' ')
	call hrout(nt,icycle,' ')
	call hldir(' ',' ')
	call hrend('NTUPLE')
    END PROGRAM HDF2NTP

