//kasArrayTrigger.cpp
//Program to seach though a VHDF5 single telescope event file (created by 
//kasaomega) and find VERITAS Array Triggers.
//i.e. VHDF5 File must have KASCADE and PARAMS datasets.
//   G.H.Sembroski
//   Physics Dept.
//   Purdue Univ.
//   W.Lafayette, In 47907
//sembroski@physics.purdue.edu
//   26/03/03
// *********************************************************************
//Our algorithum is a little complex so we will try to go through it simply.

//1:First we load into a deque array(from C++ STL) enough events from 
//the KASCADE dataset (which has VHDF5Kascade types, tag info) in the input 
// file from the same shower to span our first search.

//Events (events that trigger single telescopes) in the VHDF5 file 
//are ordered first by shower energy (tep), within that by shower 
//id (idfile), within that by ny, within that by nx, whithin that by ithphi.

//So we start with the first shower in the file and start loading
//VTrig records (has nx,ny,ithphi etc) into the deque array(called 
// triggerStack)
//For gamma-ray events there is only 1 ithphi per nx,ny pair (ithphi=1).

//2.We then take the first event in from triggerStack and let it represent a 
//trigger in a particular telescope of the array (say the central one). We 
//then test to see how many of the other telescopes in the array would have 
//triggers in them. We do this by determining the nx,ny of each of the other 
//telescopes in the specified array by seaching to see if we have triggers 
//with those nx,ny coordinates (and ithphi for hadrons). 

//If we find enough trigger telescopes we declare an array trigger
//and construct a complete array VHDF5Event and write it out to our ouput VHDF5
//file. This can be converted to a raw VArrayEvent file using the 
//VHDF52VArrayEvent conversion program.

//Note: We will add the array KASCADE and array PARAM data to this output file
//      in the future. (We have yet to define how we construct these array 
//      datasets)

//We then try the search again now using our first event in a different 
//position in the array. This is repeated until the event has been tried in 
//all positions of the array. We then proceed to the next event and do this 
//until we have looked at each event in each postion in the array and looked 
//for array triggers. We then go on to the next shower.

//Details on how the array trigger search is made for gammas rays (simpilest)
//follows.

//3:As we load the 1 dimensional deque array with gamma ray events we also 
//load a 2 dimentional nx,ny grid array with pointers to the elements in the 
//deque array. Obviously some (many) locations in the grid array will have no
//pointers.  We will actually make our array trigger search within the grid
//array itself and just use the deque array so we only have to read the  
//VTrig data once from the input file.

//4:We do our seach by defining a particular area (nx,ny 
//location) in the shower as our 'current reference area'(CRA). The grid array
//spans enough nx,ny areas to cover an telescope array where the CRA is in 
//any positon in the telescope array.

//5.For a particular position of the CRA (say in the central telescope) 
//we have a list  of the nx,ny offsets to the other telescopes. With those 
//offsets we can find the positions in the grid array where the other 
//telescopes would be and we can check in the grid array to see if we have 
//other triggers in the other telescopes of the array. We have such offset 
//lists for each possible position of the CRA in the telescope array..

//6:As we keep testing new areas as the CRA we will eventually have to remap
//the areas in the grid array. Also we will be able to drop events from the 
//begining of the deque array and will have add new ones to the end as we read
//in new events.

//7.Modification for this scheme for hadronic showers:
//a)Since all the events with the same nx,ny values but different ithphi
// values in the input file are sequential, we only need to put the index for 
//the first event found in the input file for a particular nx,ny area into 
//the grid array. All other events with the same nx,ny but different ithphi
//values will follow immediatly in sequence in the deque array after that
// event.
//b)During the grid seach of a CRA in a particular popsition in the telescope
//array we check all the occuring ithphi that the events in the CRA give
//in all the other areas.

//8.We keep track of which nx,ny,ithphi evnets we checked in each position
//in the array so that we don't duplicate a check.(uses flags in the VTrig 
//objects).

//Or as stated in the original kas_array_trigger.for:
// 	Impliment the 'GRID' search method for triggers.  This method
// 	is added on top of the current stack method.  The stack is 
// 	still needed to handle the itheta iphi records.	The GRID
// 	method consists of making an array called GRID that represents
// 	all nx ny positions on the ground.  As VTrig records are read into
// 	the stack their stack index (first index for a set with 
// 	multiple itheta iphi records) is placed at the apropiate
// 	position in the GRID array.  An array trigger search can then 
// 	just use offsets to positions in the GRID array to find the
// 	apppropiate stack indices.  This eliminates the stack
// 	searches.  To handle the problem of GRID not being big enough
// 	to contain all triggered areas the grid is relative and when
// 	an area is found that is outsid the grid GRID can be zeroed
// 	and refilled with a new relative origen that holds what needs
// 	to be covered.  This refilling won't hapopen often and the 
// 	whole method should still be much faster especially for
// 	hadronic showers with their large number of itheta aiphi
// 	records in the stack.

//A note on directions:(from kascade.for):
// kascade.for:11/21/90 G.H.S. V:1:0.3
// 	Since we have chosen dn positive to be down going and we have
// 	chosen the +x axis to be east this makes the +Y axis South.
// 	Only thing to fix was the direction of the earths magnetic
// 	field which should have a component in the North or
// 	-Y direction.

// ****************************************************************************
//	WARNING: WE are only setup for NS arrays in this code!!!!
// ****************************************************************************
//A note on triangle array(frim kaslite.for):
// kaslite.for:21/5/97 GHS V:1:1:4.2
// 	Add the Vetritas 'VT' option. This specifies a veritas array
// 	with a triangular grid spacing.
// 	TRiangle array achieved by having 1:NX
// 	as before. 2:NX even then NY as before. 3:NX odd then NY center
// 	shifted 1/2 YSEG up....
// 	If we let xseg=10m(or 12m)=sqrt(yseg**2-(yseg/2)**2). => 
// 	yseg=sqrt(4/3)*xseg=11.547m.    Do it this way so 
// 	both xseg and yseg are .ge. 10m.(or 12m, ie mirror size)
//    NS: TRiangle array achieved by having 
//    1:nx as before. 
//    2:Nx even then ny as before
//    3:Nx odd then Ny center shifted 1/2 
//    yseg up.To insure correct 
//    triangle spacing and minimum dimension
//    of 10m use 10m x 11.547m

//    EW: Interchange x,y

//    02/05/03 Change to NS:xseg:12m x yseg:13.856m spacing (Mirror grew)
//A note:We could have put yseg at 12m and xseg at 10.39 and due to the
//       triangular spacing we would have had closed packed arrays. But then
//       the rectangular areas would have intruded on each others telescopes.
//       Using xseg=12 and yseg=13.865 just makes it much easier to figure out
//       which telescope a photon belongs to.

//Yet Another note:When we go to place the telescope array on the grid we have
//       2 choices: once the central telescope is chosen we could place the
//       next telescope directly north any multiple of Yseg steps, or instead
//       we could place it east in multiples of 2*xseg (for NS aray)
//       We choose East in this code.

//kassrtmrg:    Areas are sorted with ny varying fastest and then nx.
//from original kas_array_trigger.for:
//	The input file is sorted by nx then ny then itheta then iphi.
//from original kas_array_trigger:
//		This places alternate rows of nx a half step up.
//		if we pick xseg=yseg*cos(60 deg) then we get a triangular
//		grid point that hexagonal spaced arryas fit on perfectly.

// *******************************************************
// Trigger: For now we use a simple multiplicity. No timing checks
// *******************************************************


// **************************************************************
// Don't forget peds,params and kascade tags  should also be copied over to 
// output file in someway (arrays now for each event) This can wait for a bit.
// **************************************************************


#include <getopt.h>

using namespace std;

#include "kasArrayTriggerClasses.h"
#include "Random.h"

RandomNumbers ranp("kasArrayTrigger.ran"); //VVV random number generator.
                                    //This definintion should happen only once
                                    //in a program.
                                    //All other classes(in other files) get 
                                    //ranp with extern declartions of 
                                    //ranp(looks like):
                                    //  extern RandomNumbers ranp;
double Rexp(float rate)
{
  /*
    This function randomly picks from an exponatial distribution using
    rate as the scale factor.
    Rexp is distributed as exp(-q/rate)
  */
	
  
  //       This produces the required random distribution.
	double r = ranp.Exponential()/rate;
	return r;
}

int main( int argc, char **argv )
{
  ArraySpecs VeritasArray;  //Defines  VERITAS Array object which holds lots of
                            //particulars about this version of the array.
                            //Doesn't fill anything in. Some of that is done 
                            //below from reading command line input.
    // Command Line input:*************************************
  char* iFilename="";
  char* oFilename="";
  bool inID=false;
  bool outID=false;
  int arraySpacing;
  bool iSpace=false;
  int config;
  bool iConfig=false;
  int multiplcity;
  bool iMult=false;
  int n;
  int c;   //c MUST be declared outside the while
           //(gcc compiler 'feature' 3.0.4)
  while ((c = getopt(argc, argv, "i:o:s:c:m:h")) != -1)
//Note for Myself:In the getopt call, in the allowed options list, an option 
//followed by a ':' is required to be followed by an option argument. An 
//option not folowed by a ':' has no following option argumnet. Thus in 
//"i:to:" above, options -i and -o must have arguments, option -f must not!

    // followed by a : means it expects an argument to follow
    {
      switch (c)
	{
	case 'i':
	  cout<<"Input file name(option -i): "<<optarg<<endl;
          iFilename=optarg;
          inID=true;
	  break;                 //This break just gets us out of the switch
                                 //not the while!	
	case 'o':
	  cout<<"Output file name(option -o): "<<optarg<<endl;
	  oFilename=optarg;
          outID=true;
	  break;                 //This break just gets us out of the switch
                                 //not the while!
	case 's':
	  cout<<"Array Spacing (in units of xseg. Must be even,>1 (option -s): "<<optarg<<endl;
	  arraySpacing=atoi(optarg);
	  n=arraySpacing/2;  //We want odd round down;
	  if(n*2!=arraySpacing || arraySpacing<2)
	    {
	      cout<<"Fatal--Bad Telescope spacing.Must be >1 and divisable by 2"<<endl;
	      return 1;
	    }
	  VeritasArray.setArraySpacing(arraySpacing);
          iSpace=true;
	  break;                 //This break just gets us out of the switch
                                 //not the while!
	case 'm':
	  cout<<"Trigger Multiplcity;(simple trigger)(option -m): "<<optarg<<endl;
	  multiplcity=atoi(optarg);
	  if(multiplcity==0)
	    {
	      cout<<"Fatal--Bad Multiplcity. Must be >0"<<endl;
	      return 1;
	    }
	                      //Too large a multiplicity will be checked later.
	  VeritasArray.setTriggerMultiplicity(multiplcity);
          iMult=true;
	  break;                 //This break just gets us out of the switch
                                 //not the while!
	  
	case 'c':
	  cout<<"Array Configuration(input is assumed decimal)(option -c): "<<optarg<<endl;
	  config=atoi(optarg);
	  cout<<"Set bits in configuration indicate telescopes in trigger:configuration in octal (easier to decode bits): "<<oct<<config<<dec<<endl;
	  if((config & 0177)==0)
	    {
	      cout<<"Fatal--Bad Configuration given"<<endl;
	      cout<<"Configuration must have some or all of first 7 bits set!"<<endl;
	      return 1;
	    }
	  VeritasArray.setConfiguration((unsigned short)config);
          iConfig=true;
	  break;                 //This break just gets us out of the switch
                                 //not the while!
	case 'h':
	  cout<<"kasArrayTrigger Command line options:"<<endl;
	  cout<<"-i Input file name"<<endl;
	  cout<<"-o Output file name"<<endl;
	  cout<<"-s Array Spacing (in units of xseg. Must be even,>1)"<<endl;
	  cout<<"-m Trigger Multiplcity (simple trigger)"<<endl;
	  cout<<"-c Array Configuration(input is assumed decimal)Set bits indicate telescopes in trigger"<<endl;
	  break;                 //This break just gets us out of the switch
                                 //not the while!
	case '?':
	  cout<<"Unknown option:"<<c<<endl;
	  return 1;
	}
    }
  if(!inID)
    {
      cout<<
	"Input VHDF5 file name not specified (option -i)!"<<endl;
      return 0;
    }

  if(!outID)
    {
      cout<<"Output VHDF5 file name not specified (option -o)!"<<endl;
      return 0;
    }
  if(!iSpace)
    {
      cout<<"Must specify Array Spacing (in units of xseg)(option -s)"<<endl;
      cout<<"Must be >1 and even (integer)"<<endl;
      return 0;
    }
  if(!iConfig)
    {
      cout<<"Must specify Array Configuration(option -s)"<<endl;
      cout<<"Given in decimal. Set bits indicate telescopes in trigger."<<endl;
      return 0;
    }
  if(!iMult)
    {
      cout<<"Must specify a Trigger Multiplcity(>0)(option -m)"<<endl;
      return 0;
    }

//Now set up the input VHDF5 file. 
  try
    {
      VHDF5VTrig* input = new VHDF5VTrig(&VeritasArray);
                         //This is a class that inherits from 
                         //VHDF5KascadeRunDataFile
                         //Has additional stuff to extract the VTrig info
                         //also sets various flags when shower is done.
                         //Note: VeritasArray was instantiated just after 
                         //'main' statement above.
      input->openFile(iFilename);
// ******************************************************
      //Note: In the VHDF5VTrig class we have overidden the dummy virtual
      //member function VHDF5RunDataFile::postOpenFile (VHDF5VTrig inherits 
      //from VHDF5KascadeRunDataFile which inherits form VHDF5RunDataFile). 
      //VHDF5RunDataFile::openFile calls postOpenFile just before it 
      //returns. We use VHDF5VTrig::postOpenFile to init a bunch of stuff.
// ******************************************************
                        //After opening the VHDF5 file this then calls 
                        //VHDF5VTrig::postOpenFile which does the following:
                        //0: Checks for existance of KASCADE dataset. If it 
                        //   doesn't exists we thow an exception.

                        //1:Gets number of events in VHDF5 file
                        //2: Initalizes eof and end-of-shower flags to false
                        //3:Sets file index to 0
                        //4:Gets energy and shower id of first shower in file.
                        //5:Puts xseg and yseg (kept as atributes of KASCADE 
                        //  dataset)of ground grid into VeritasArray
// *********************************************************************

      float spacing=VeritasArray.getXseg()*VeritasArray.getArraySpacing();
      cout<<"The Spacing in meters for the array is:"<<spacing<<" m."<<endl;
      //From attributes of KASCADE dataset;
      cout<<"xSeg and ySeg of the array is:"<<VeritasArray.getXseg()<<","<<
	                                        VeritasArray.getYseg()<<endl;
    

      cout<<"Original size of input file:"<<input->getNumberOfEvents()<<
                                                                         endl;

// *********************************************************************
//Create Ouput file:
      VHDF5KascadeRunDataFile output;
      output.createFile(oFilename); 
      int eventNumber=0;  //counts event in output file.



// *********************************************************************
//Setup Array offsets: 
      bool even=true;
      ArrayOffsets evenOffsets(VeritasArray.getConfiguration(),
                                         VeritasArray.getArraySpacing(),even);
      even=false;
      ArrayOffsets oddOffsets(VeritasArray.getConfiguration(),
                                         VeritasArray.getArraySpacing(),even);

      VeritasArray.setNumberOfTelescopes(evenOffsets.getNumberOfTelescopes());
      if(VeritasArray.getTriggerMultiplicity() >
	                     VeritasArray.getNumberOfTelescopes())
	{
	  cout<<"Fatal--Bad Multiplcity. Must be < number of telescopes from configuration."<<endl;
	  return 1;
	}


//  ************************************************************************
//Set up and write out the Info rec to the ouput VHDF5 file:
      //Now we need to read in the VHDF5RunDataFile info record.
      //Define stuff
      VHDF5InfoHeader* iHead = new VHDF5InfoHeader();
      vector< VHDF5TelescopeInfo > tInfo;          //Need an array of these.
      input->readInfoRec(iHead,tInfo);            //read in  the info record.

      //Now we need to modify this stuff for the ouput file.
      iHead->nTelescopes=VeritasArray.getNumberOfTelescopes();
      
      //The input file had one tInfo element. Expand that.
      vector< VHDF5TelescopeInfo > arrayInfo;      //Need an array of these.
      for(int i=0;i<VeritasArray.getNumberOfTelescopes();i++)
	{
	  arrayInfo.push_back(tInfo[0]); //copy in base
	  arrayInfo[i].telescopeID=
	    (unsigned int)evenOffsets.telescopeNode(evenOffsets.getConfig(i));
	}
      output.writeInfoRec(iHead,arrayInfo);
      
      //Get UTC time
      double UTCTime=iHead->UTCStart;


//  **********************************************************
// Stack definintion
      //define the stack       //Also saves input reference. and the Array ref.
      VStack vStack(input);    //input is a pointer.
      
      //read in first event and load it into stack
      //This event does exists since it was read in once before when we
      //instantialed the input file.
      VTrig vTrig;
      if(input->getVTrig(0,&vTrig))
	{
	  //Very first event in file.
	  vStack.push_back(vTrig);
	  input->setIndexNextVTrig(1);
	}
      else
	{
	  return 1;  //Should never get here (empty file (or no KASCADE 
	             //dataset will throw exception when we open the file.. 
	             //We may want to change that  to jusrt returning.
	}
      
      //Define the grid
      VGrid vGrid(input, &vStack, &VeritasArray); 
                        //Save input,vStack,Veritas Array ref in grid object
                        //set in vGrid size of array configuration. 
                        //Used in decide how much to fill in thegrid.
      vGrid.setNXMin(0);
      vGrid.setNXMax(max<int>(evenOffsets.getNXMax(),oddOffsets.getNXMax()));
      vGrid.setNYMin(min<int>(evenOffsets.getNYMin(),oddOffsets.getNYMin()));
      vGrid.setNYMax(max<int>(evenOffsets.getNYMax(),oddOffsets.getNYMax()));
      
      //To find where grid starts get nx ny of first event (its in the stack)
      
      int nx=(vStack.getBack()).getNX();
      int ny=(vStack.getBack()).getNY();

      //WE need now to map the grid to the nx,ny of the events. This grid
      //needs to be big enough so that we don't have to remap very often
      //We would have to remap if we read an event whose nx,ny would put it 
      //off the grid.
      //the grid starts at 0,0. our first event (since things are ordered)
      //has the minimum nx(most negative) we will ever see. 
      //So we will never get another
      //event with its nx less than the first event's nx. This means we can
      //put this event at nx=0 in the grid.
      //Now its ny can be anything, so we can't use it to make this decision
      //lets guess that the best place to set the grid is centered on the ny 
      //axis. Ie We assign the y index: kMaxGrid/2 to correspond to ny=0
      //So chose for the relative position of the grid such that the grid 
      //point 0,0 corresponds to nx,-kMaxGrid/2
      //Last point in grid will be nx+kMaxGridSize-1,kMaxGridSize-1

      //Fill the grid for this  area
      //This will cause events to be read into the stack the stack gets 
      //trimmed, and
      //their iterater postions of the start of the itheta,iphi groups in the
      // stack placed in the grid until the first trigger area is covered.
 
      vGrid.fillGrid(nx, -kMaxGridSize/2);
     
      //Now check that out first event is actually contained within the grid.
      
      if(!vGrid.contained(nx,ny))
	{
	  //Remap the grid.(does a fillGrid, fillStack and trimStack
	  //if it needs to)
	  vGrid.reMap(nx,ny);
	}

      //Loop though the areas in the shower.
      while(true)  //Forever loop
	{
	//We let the elements in the grid drive our selection of reference 
	//area.
	  int nxAreaRef;
	  int nyAreaRef;
	  deque<VTrig>::iterator pos;
	  for(pos=vStack.getBegin(); pos!= vStack.getEnd();pos++)
	    //This loop continues until we have to remap the grid.
	    //Get the specifics of the reference event this trigger
	    {
	      nxAreaRef=pos->getNX();
	      nyAreaRef=pos->getNY();
	      //Make sure this area and all the other areas we may need to 
	      //test it are contianed in the grid
	      if(!vGrid.contained(nxAreaRef,nyAreaRef))
		{
		  //Remap the grid.(does a fillGrid and fillStack trimStack
		  //if it needs to)
		  vGrid.reMap(nxAreaRef,nyAreaRef);
		  break; //Leave this for loop(not the while loop). 
		  //We need to restart the for loop after the Remap since
		  // the remap screws up all the pos iterators
		  //The remap does a trim and when its done this present event
		  //should be the first in the stack.
		}
	  //Now search the events to see if this particular event is part of 
	  //a trigger in any postiion in the array
	      for(int i=0;i<VeritasArray.getNumberOfTelescopes();i++)
		{
		  //Find relative position of this telecope in array
		  unsigned short configNow=evenOffsets.getConfig(i);
		  
		  //assign our even odd array
		  ArrayOffsets* offSets;
		  if(VeritasArray.centerEven(nxAreaRef,configNow))
		    { 
		      offSets=&evenOffsets;
		    }
		  else	
		    {
		      offSets=&oddOffsets;
		    }
		  //Now check to see if we have a trigger
		  //Also set config bits so that we know which areas have been 
		  //checked in which array positions.
		  deque<VTrig>::iterator 
		    posTrigger[VeritasArray.getNumberOfTelescopes()];
		  unsigned char triggerMask[kMaxNumberOfTelescopes];
		  int triggerMult = vGrid.triggerMultiplicity(i,pos,offSets,
						   posTrigger,triggerMask);
		  //Do we have an array trigger?
		  //Some day we may put timing etc testing in here.
		  if(triggerMult >= VeritasArray.getTriggerMultiplicity())
		    {

//Build Ouput VHDF5 VArrayEvent ***************************
		      VHDF5Event VHDF5aeIn;
		      VHDF5Event VHDF5aeOut;

		      //Read in first event to use a a base. Get index in file.
		      
		      int fileIndex=posTrigger[0]->getFileIndex();
		      
		      //Use Fisrt event as base event. Gets header and
		      //telescope 1 event data
		      
		      input->readEvent(VHDF5aeOut,fileIndex);

		      //Add in the remaining telescopes to create a single
		      //'Array' event

		       //Now add in the remaining telescope event data.
		      //Notice we skip index 0, that was the first one.
		      if(triggerMult>1)
			{
			  for(int i=1;i<triggerMult;i++)
			    {
			     //File index of event data
			      fileIndex=posTrigger[i]->getFileIndex();	   
			      input->readEvent(VHDF5aeIn,fileIndex);
			      unsigned char nodeID=
				offSets->telescopeNode(triggerMask[i]);
			      VHDF5aeOut.pushTelescope(VHDF5aeIn,nodeID);
			    }
			}

// ******************
//For Later: Also fix TDCCounts,tenMhzClock,vetoedClock in each 
//           telescope
// ******************
		      //Load trigger mask
		      unsigned char tMask=0;
		      for(int j=0;j<triggerMult;j++)
			{
			  tMask=(tMask | triggerMask[j]);
			}
		      VHDF5aeOut.setTriggerMask(tMask);
		      
		      eventNumber++;
		      VHDF5aeOut.setEventNumber(eventNumber);
		      

		      //Determine a time for this event;
		      UTCTime=UTCTime+(float)Rexp(.03);
		 VUTCGPS::UTC2GPS(UTCTime,VHDF5aeOut.getTrigGPSTimePointer());
		      for(int j=0;j<triggerMult;j++)
			{  //Copy in the UTC timne to each telescope data
	    VUTCGPS::UTC2GPS(UTCTime,VHDF5aeOut.getTelescopeGPSTimePointer(j));
			} 
    
		      VHDF5aeOut.setConfigurationMask(
					 VeritasArray.getConfiguration());
		      VHDF5aeOut.setNumberTelescopes(
			                 VeritasArray.getNumberOfTelescopes());
		      VHDF5aeOut.setNumberTelescopesData(triggerMult);

		      //Done. Write it out.
		      
	              output.writeEvent(VHDF5aeOut);
		    }//end Trigger block
		} //End CRA position loop
	    } //End event loop
	      //See why we left the event loop.
	  if(pos== vStack.getEnd())
	    { 
		  //We processed last event in shower.
	      cout<<"For Energy:"<<input->getTepNow()<<" shower ID:"<<
		       input->getShowerIDNow()<<"Found "<<eventNumber<<
		                                         " triggers"<<endl;
	      // See is this is end of file
	      if(input->isEOFReached())
		{
		  input->closeFile();
		  output.flushEventWriteBuffer(); //do this to get final 
		                                      //event totals
		  int numberOutput=output.getNumberEventsInFile();
		  cout<<"Final size of ouput EVENT dataset:"<<numberOutput
                                                                 <<endl;
		  output.closeFile();
		  return 0;
		}
		  //Do whatever needed to do to start up new shower
	    }
	  
	  //************Start processing events from stack again.*******
	
	}//end Forever WHILE loop
    }//End of try block
  catch(VException &e)
    {
      cerr<<e;
      return 1;
    }
  catch(...)
    {
      cout<<"Fatal--Unknown exception found."<<endl;
      return 1;
    }
  return 0;
}

