#ifndef KAS_ARRA_TRIGGER_CLASSES_H
#define KAS_ARRA_TRIGGER_CLASSES_H
//Classes used in kasArrayTrigger
// ***************************************************************
//Modified:
//
// 29/10/04 GHS Bring code backup to speed.

#include <iostream>
#include <string>
#include <vector>
#include <cmath>
#include <algorithm>
#include <deque>

using namespace std;

#include "exception.h"
#include "VHDF5KascadeRunDataFile.h"


const int kMaxNumberOfTelescopes=7;
const int kMaxGridSize=300; //Dimension of the grid is [kMaxGrid][kMacGrid]

// ***************************************************************

//Define the Exceptions

class   kasArrayTriggerDuplicateAreas:public VException
{
 public:
  kasArrayTriggerDuplicateAreas(const char* msg=NULL)
    {
      setStrings("kasArrayTriggerDuplicateAreas",msg);
    }
};
 
class   kasArrayTriggerNoKASCADEDataset:public VException
{
 public:
  kasArrayTriggerNoKASCADEDataset(const char* msg=NULL)
    {
      setStrings("kasArrayTriggerNoKASCADEDataset",msg);
    }
};
 
class   kasArrayTriggerBadTelescopeID:public VException
{
 public:
  kasArrayTriggerBadTelescopeID(const char* msg=NULL)
    {
      setStrings("kasArrayTriggerBadTelescopeID",msg);
    }
};

class   kasArrayTriggerIllegalAreaRequested:public VException
{
 public:
  kasArrayTriggerIllegalAreaRequested(const char* msg=NULL)
    {
      setStrings("kasArrayTriggerIllegalAreaRequested",msg);
    }
};


// ***************************************************************

class ArraySpecs
// ***************************************************************
// Has all parameters and specifications of the array used in searching.
// ***************************************************************
{
 private:
  float xSeg;             //array spacinf dimentions
  float ySeg;
  bool evenGrid;          //true for an even (n-s) triangular array.
  unsigned short configuration; //Bit flags for configuration this array
  int arraySpacing;
  int numberOfTelescopes; //Gotten from counting bits set in configuration.
  int triggerMultiplcity;
 public:
  ArraySpecs(){};
  ~ArraySpecs(){};
  int findNX(float x);    //convert x and y positions of array on ground
  int findNY(float y,int nx);    //indicies.Needs nx to tell odd or even
  void setXseg(float x){xSeg=x;return;};
  void setYseg(float y){ySeg=y;return;};
  float getXseg(){return ySeg;};
  float getYseg(){return ySeg;};
  void setConfiguration(unsigned short config){configuration=config;return;};
  unsigned short getConfiguration(){return configuration;};
  void setArraySpacing(int spacing){arraySpacing=spacing;return;};
  unsigned int getArraySpacing(){return arraySpacing;};
  void setEvenGrid(bool e){evenGrid=e;return;};
  bool centerEven(int nx,unsigned short config);
  void  setTriggerMultiplicity(int m){triggerMultiplcity=m;};
  int getTriggerMultiplicity(){return triggerMultiplcity;};
  void setNumberOfTelescopes(int number){numberOfTelescopes=number; return;};
  int getNumberOfTelescopes(){return numberOfTelescopes;};
};



// ***************************************************************

class VTrig
// ***************************************************************
// Triggered telescope object. Has all stuff neede to define it for
// array trigger search.
// ***************************************************************
{
 private:
  int nx;        //Coords to specifiy trigger position and direction.
  int ny;
  int ithphi;
  float trigTime; //Trigger time of event.
  int fileIndex;  //Index in VHDF5 file to retrieve original event.
  unsigned char tested; //Bit Flags to indicate if we have tested  this event 
                         //in a particular position within the array
 public:
  VTrig(){};
  ~VTrig(){};
  int getNX(){return nx;};
  int getNY(){return ny;};
  int getIThphi(){return ithphi;};
  int getFileIndex(){return fileIndex;};
  void setNX(int NX){nx=NX;return;};
  void setNY(int NY){ny=NY;return;};
  void setIThphi(int iThphi){ithphi=iThphi;return;};
  void setTrigTime(float trigtime){trigTime=trigtime;return;};
  void setFileIndex(int index){fileIndex=index;return;};
  void clearTested(){tested=0;return;};
  unsigned char getTested(){return tested;};
  void setTested(unsigned char test){tested=test;return;};
};


class VHDF5VTrig:public VHDF5KascadeRunDataFile
  //This is a class to read from the VHDF5 file made by KASCADE (has the
  //KASCADE dataset in it) for use in find array triggers. 
  //It inherits from VHDF5KascadeRunDataFile which inherits from 
  //VHDF5RunDataFile
  //we add a method to read from the kascade dataset and from that data 
  //fill in a VTrig object. Ther VTrig objects are used to find the array
  //triggers
{
 private:
  VHDF5Kascade* Kascade;
  float tepNow;           
  int showerIDNow;
  int numberOfEvents;
  bool endOfFileReached;
  bool endOfShowerReached;
  int indexNextVTrig;
  ArraySpecs* VArray;
 protected:
  void postOpenFile();    //This overides the virtual dummy postOpenFile in
                          //VHDF5RunDataFile (which VHDF5KascadeRunDataFile 
                          //inherits from)
                          //VHDF5RunDataFile::openFile calls this after it has
                          //successfully opend the file. See VHDF5RunDataFile
                          //comments for 08/07/03 for explination why we do it
                          //this way.
 public:
  VHDF5VTrig(ArraySpecs* VArr);
  bool getVTrig(int index, VTrig* vTrig);
  int getNumberOfEvents(){return numberOfEvents;};
  bool isEOFReached(){return endOfFileReached;};
  void setIndexNextVTrig(int index){indexNextVTrig=index;return;};
  int getIndexNextVTrig(){return indexNextVTrig;};
  float getTepNow(){return tepNow;};
  int getShowerIDNow(){return showerIDNow;};
};
// ***************************************************************

class ArrayOffsets
{
 private:
  int offset[4][kMaxNumberOfTelescopes];  //3 for x,y,z: 1 for config
  int offsetLoc[4][kMaxNumberOfTelescopes][kMaxNumberOfTelescopes];
  int nyMin;
  int nxMax;
  int nyMax;
  int numberOfTelescopes;
  void reOrder(); //Reordes offset arry with nx(descending) and within that 
                  //ny(descending)array  
 public:
  ArrayOffsets( unsigned int configuration, int nSpacing, bool even);
  ~ArrayOffsets(){};
  int getNXMax(){return nxMax;};
  int getNYMax(){return nyMax;};
  int getNYMin(){return nyMin;};
  int getNumberOfTelescopes(){return numberOfTelescopes;};
  unsigned char getConfig(int telescope)
                          {return (unsigned char)offsetLoc[3][telescope][0];};
  int getNXOffset(int i, int index){return offsetLoc[0][i][index];};
  int getNYOffset(int i, int index){return offsetLoc[1][i][index];};
  unsigned int telescopeNode(unsigned char config);
};
// *******************************************************************

class VStack
// *******************************************************************
//This is the Stack class, which we use to search for triggers.
// *******************************************************************
//This is a deque that we store the trigger events form the input file
{
 private:
  VHDF5VTrig* input;
  deque<VTrig> areaStack;
 public:
  VStack(VHDF5VTrig* in){input=in;};
  ~VStack(){};
  VTrig getBack(){return areaStack.back();};
  void push_back(VTrig vTrig){areaStack.push_back(vTrig);};
  void fillStack(int nxLast, int nyLast); //Fills stack until nxLast,nyLast 
                                          //exceeded
                                 //or shower ends or file ends.
  void trimStack(unsigned short configuration);
  deque<VTrig>::iterator getBegin(){return areaStack.begin();};
  deque<VTrig>::iterator getEnd(){return areaStack.end();};
};

// *******************************************************************

class VGrid
// *******************************************************************
//This is the grid class, which we use to search for triggers.
// *******************************************************************
//From original kas_array_trigger.for:
// 	Impliment the 'GRID' search method for triggers.  This method
// 	is added on top of the current stack method.  The stack is 
// 	still needed to handle the itheta iphi records.	The GRID
// 	method consists of making an array called GRID that represents
// 	all nx ny positions on the ground.  As VTrig records are read into
// 	the stack their stack location (position of first trigger for a set 
//      with multiple ithphi records) is placed at the apropiate
// 	position in the GRID array.  An array trigger search can then 
// 	just use offsets to positions in the GRID array to find the
// 	apppropiate stack positions.  This eliminates the stack
// 	searches.  To handle the problem of GRID not being big enough
// 	to contain all triggered areas the grid is relative and when
// 	an area is found that is outsid the grid GRID can be zeroed
// 	and refilled with a new relative origen that holds what needs
// 	to be covered.  This refilling won't hapopen often and the 
// 	whole method should still be much faster especially for
// 	hadronic showers with their large number of ithphi
// 	records in the stack.
// *******************************************************************

//We are  useing a deque for our stack.
//Our grid will be an array holding deque position iterators into the stack.

{
 private:
  VHDF5VTrig* input; //set by constructer
  VStack* areaStack;
  ArraySpecs* VArray;
  VTrig nullArea;
  deque<VTrig> NullArea;
  deque<VTrig>::iterator posNull;

  deque<VTrig>::iterator grid[kMaxGridSize][kMaxGridSize];
                                   //An array of iterators into the areaStack.

  int nxMin;
  int nxMax;
  int nyMin;
  int nyMax;
  int iThphiNow;
  int nxRef;
  int nyRef;


 public:
  VGrid(VHDF5VTrig* in, VStack* stack, ArraySpecs* VArr);
  ~VGrid(){};
  void setNXMin(int NXMin){nxMin=NXMin; return;};
  void setNXMax(int NXMax){nxMax=NXMax;return;};
  void setNYMin(int NYMin){nyMin=NYMin;return;};
  void setNYMax(int NYMax){nyMax=NYMax;return;};
  void fillGrid(int nx, int ny);
  void reMap(int nx, int ny);
  int getIThphiNow(){return iThphiNow;};
  void setIThphiNow(int ithphi){iThphiNow=ithphi; return;};
  void setNXRef(int nx){nxRef=nx;return;};
  void setNYRef(int ny){nyRef=ny;return;};
  bool contained(int nx, int ny);
  int triggerMultiplicity(int offsetIndex, deque<VTrig>::iterator pos,
			 ArrayOffsets* offSets, 
			 deque<VTrig>::iterator* posTrigger, 
			 unsigned char* triggerMask);
  deque<VTrig>::iterator posArea(int nx, int ny, int ithphi);
};
 
#endif //KAS_ARRA_TRIGGER_CLASSES_H

