//kasSumMhdf5.cpp
//	This program reads in the MHDF5 files for the whipple
//	Camera and appends the records back out to a summary file.
//      All files are in MHDF5 format.
//     This is a conversion to C++ of kassum_hdf5.f90 but using the 
//     MHDF5SimDataFile class to do the work. Should run faster.
//   G.H.Sembroski
//   Physics Dept.
//   Purdue Univ.
//   W.Lafayette, In 47907
//sembroski@physics.purdue.edu
//   01/08/03
//Modified:

//03/05/04 GHS Add ability to cut on itheta and iphi. Add -t and -p options.
//             They must be used together. Ignore itheta iphi is these options
//             not used (default).
//             Add the help option (-h)
//22/06/04 GHS Add a SELECT option (-w). This causes cuts to events to the 
//             ouput file depending on spectrum weights.

#include "MHDF5SimDataFile.h"
//#include <unistd.h>
#include "Random.h"
#include <getopt.h>
#include <iostream>
#include <iomanip>
#include <vector>
#include <string>

using namespace std;
RandomNumbers RanP("kasSumMHDF5.ran"); //VVV random number generator.
                                    //This definintion should happen only once
                                    //in a program.
                                    //All other classes(in other files) get 
                                    //ranp with extern declartions of 
                                    //ranp(looks like):
                                    //  extern RandomNumbers ranp;

extern "C" void VeritasEventWeight(float* tep, float* type, float* weight,
				   bool* quiet);
extern "C" void WhippleEventWeight(float* tep, float* type, float* weight,
				   bool* quiet);


int main( int argc, char **argv )
 {
  char* iFilename="";
  char* oFilename="";
  string operation;
  bool inID=false;
  bool outID=false;
  bool operationFlag=false;
  bool quiet=false;
  bool selectByWeight=false;

  short theta_only=-1;
  short phi_only=-1;

  int c;   //c MUST be declared outside the while
           //(gcc compiler 'feature' 3.0.4)
  while ((c = getopt(argc, argv, "i:o:a:t:p:qhw")) != -1)
//Note for Myself:In the getopt call, in the allowed options list, an option 
//followed by a ':' is required to be followed by an option argument. An 
//option not folowed by a ':' has no following option argumnet. Thus in 
//"i:to:" above, options -i and -o must have arguments, option -f must not!

    // followed by a : means it expects an argument to follow
    {
      switch (c)
	{
	case 'w':
          selectByWeight=true;
	  if(!quiet)cout<<"Select ouput events by spectrum weights(option -w)"
			<<endl;
	  break;                 //This break just gets us out of the switch
	case 'q':
          quiet=true;
	  break;                 //This break just gets us out of the switch
	case 'i':
	  if(!quiet)cout<<"Input file (option -i): "<<optarg<<endl;
          iFilename=optarg;
          inID=true;
	  break;                 //This break just gets us out of the switch
                                 //not the while!	
	case 'o':
	   if(!quiet)cout<<"Output file (option -o): "<<optarg<<endl;
	  oFilename=optarg;
          outID=true;
	  break;                 //This break just gets us out of the switch
                                 //not the while!
	case 'a':
	   if(!quiet)cout<<"Operation to preform (option -a): "<<optarg<<endl;
	  operation=optarg;
          operationFlag=true;
	  break;                 //This break just gets us out of the switch

	case 't':
	  if(!quiet)cout<<"Keeping only events with itheta= "<<optarg<<
		      "(option -t)"<<optarg<<endl;
	  theta_only=(short)atoi(optarg);
 	  break;                 //This break just gets us out of the switch
	case 'p':
	  if(!quiet)cout<<"Keeping only events with iphi= "<<optarg<<
		      "(option -p)"<<optarg<<endl;
	  phi_only=(short)atoi(optarg);
 	  break;                 //This break just gets us out of the switch
	case 'h':
	  cout<<" *****************************************************"<<endl;
	  cout<<" * Command line input options:"<<endl;
	  cout<<" *****************************************************"<<endl;
	  cout<<" * -i Specify Input HDF5 M file name "<<endl;
	  cout<<" * -o Specify Output HDF5 M file name "<<endl;
	  cout<<" * -q Quiet (no informational output)"<<endl;
	  cout<<" * -a Indicate type of copy (NEW or APPEND)"<<endl;
	  cout<<" * -t Specify itheta of kept events"<<endl;
	  cout<<" * -p Specify iphi of kept events"<<endl;
	  cout<<"      Bothe -t and -p options must be used together"<<endl;
	  cout<<"     (unless -t specifies an itheta of 1(iphi ignored)"<<endl;
	  cout<<" * -w Select ouput events by spectrum weights"<<endl;
	  cout<<" * -h Print this options summary"<<endl;
	  cout<<" *****************************************************"<<endl;
	  return 0;
 	case '?':
	  cout<<"Unknown option:"<<c<<endl;
	  return 1;
	}
    }

  if(!inID)
    {
      cout<<
	"Input VHDF5 file name not specified (option -i)!"<<
	endl;
      return 0;
    }

  if(!outID)
    {
      cout<<"Output VArrayEvent file name not specified (option -o)!"<<
	endl;
      return 0;
    }
  if(!operationFlag)
    {
      cout<<"Operation to preforme (NEW or APPEND) not specified (option -a)!"<<
	endl;
      return 0;
    }

  if(operation=="NEW")
    {
       cout<<"File "<<iFilename<<endl;
       cout<<" will be coppied to a NEW ouput file "<<endl;
       cout<<oFilename<<endl;
    }
  else if(operation=="APPEND")
    {
       if(!quiet)cout<<"File "<<iFilename<<" will be APPENDed to ouput file "<<
                                                               oFilename<<endl;
    }
  if( theta_only!=1)
    {
      if( ( (theta_only!=-1) && (phi_only==-1) ) ||
	  ( (theta_only==-1) && (phi_only!=-1) ) )
	{
	  cout<<"Fatal--Options -t and -p must be used together"<<endl;
	  return 0;
	}
    }

  //Test that the seed isn't screwed up.
  double ranP=RanP.Uniform();
  if(ranP==1.0)
    {
      cout<<" kasSumMHDF5.ran random seed file is bad. Replace it!"<<endl;
      return 0;
    }

  //Now set up the input MHDF5 file. Open the input and output files:
  try
    {
      MHDF5MHead* iHead = new MHDF5MHead();

      MHDF5EventData* MHDF5ev = new MHDF5EventData();
      MHDF5SimDataFile input;         //Open input file and get MHEAD record
      input.openFile(iFilename); 
      input.readMHeadRec(iHead);
      string options=iHead->options;
      int nPixels=0;
      if(options.find("W490")!=string::npos)
	{
	  nPixels=490;
	  
	}
      else if(options.find("V499")!=string::npos)
	{
	  nPixels=499;
         }
      else
	{
	  cout<<" Unable to decode number of pixels in camera"<<endl;
	  return 1;
	}
      float  TeV=(iHead->tep);
      int GeV=(int)(TeV*1000.);
      int ID=iHead->idfile;
      //      float x=iHead->x_offset;
      //float y=iHead->y_offset;
      float type=(float)iHead->itype;

      MHDF5SimDataFile output;       //Open ouput file
      int numberOutput;            //Place to start writing events in ouput
      if(operation=="NEW")
	{
	  if(!quiet)cout<<"Creating new Ouput file:"<<oFilename<<endl;
	  output.createFile(oFilename);
          output.writeMHeadRec(iHead);
	  numberOutput=0;
	}
      else
	{
	  //Appending to existing ouput file.
	  output.openFile(oFilename);
	  numberOutput=output.getNumberEventsInFile();
	  output.setNextWriteIndex(numberOutput);
	}

      bool MRECExistsIn=input.existsMRECDataset();
      if(!MRECExistsIn)
	{
	  //No MREC dataset in Input file. 
	  //Just quit nicely after printing a line. 
	  //Just an empty file;
	  return 0;
	}
      
      int numberInput=input.getNumberEventsInFile();
      int nextIndexToWriteTo=0;
      for(int i=0;i<numberInput;i++)
	{  
	  //Events read one a time, write as a buffer. Thus the screwaround 
	  //with event numbver. events start event number 0
	  //EVENT dataset first;
	  input.readEvent(MHDF5ev,i);

	  // ****************************************************************
	  // Check if we are selecting by weight
	  // ****************************************************************
	  if(selectByWeight)
	    {
	      float EnergyGev=MHDF5ev->dataHead.tep;//Already in GeV
	      float weight;
	      bool quiet=true;
	      //bool quiet=false;
	      if(nPixels==490)
		{
		  WhippleEventWeight(&EnergyGev,&type,&weight,&quiet);
		  //cout<<"Weight: "<<weight<<endl;
		}
	      else
		{
		  VeritasEventWeight(&EnergyGev,&type,&weight,&quiet);
		}
	      // Test to see if we keep this event
	      ranP=RanP.Uniform();
	      //cout<<"ranP: "<<ranP<<endl;
	      if(ranP>weight)
		{
		  continue;  //Drop this event
		}
	    }
		  
	  if(theta_only == -1)

	    {     //No itheta,iphi selection. Keep everything
	      nextIndexToWriteTo = output.getNextIndexToWriteTo();
	      if(!selectByWeight && nextIndexToWriteTo!=numberOutput+i)
		{
		  cout<<"FATAL-Expected ouput index does not match actual"
		      <<endl;
		  cout<<"nextIndexToWriteTo:"<<nextIndexToWriteTo<<endl;
		  cout<<"numberOutput:"<<numberOutput<<endl;
		  cout<<"i:"<<i<<endl;
		  return 1;
		}
	      output.writeEvent(MHDF5ev);  //No index since we are buffering
	    }

	  else

	    {  //theta and phi event selection mandated (-t -p options)
	      short itheta=MHDF5ev->rec.itheta;
	      short iphi=MHDF5ev->rec.iphi;
	      if(itheta==theta_only)
		{
		  if( (itheta==1) ||
		      ( (itheta==theta_only) && (iphi==phi_only)) )
		    {
		      output.writeEvent(MHDF5ev);//No index since we are 
		      //buffering
		    }
		}
	    }
	}
      
      input.closeFile();
      output.flushEventWriteBuffer(); //do this to get final event totals
      int numberOutputEnd=output.getNumberEventsInFile();
      int net=numberOutputEnd-numberOutput;

      cout<<setw(8)<<numberOutput<<" to"
	  <<setw(8)<<numberOutputEnd-1<<"#:"<<setw(6)<<net<<" from "
	  <<setw(6)<<numberInput
	  <<" GeV:"<<setw(4)<<GeV<<" id:"<<setw(4)<<ID<<" th,ph"<<setw(2)
	  <<theta_only<<setw(2)<<phi_only<<" specCut:"<<selectByWeight<<endl;
      output.closeFile();
    }  

  catch(VException &e)
    {
      cerr<<e;
      return 1;
    }
  catch(...)
    {
      cout<<"Fatal--Unknow exception found."<<endl;
      return 1;
    }
  return 0;
 }


	  
 
