	Program KAS_ARRAY_TRIGGER
c	Version:V:1:1:5:1:1:1:1.0
c	This routine_ is used to find array triggers for an a VERiTAS
!	triangular array.

!****************************************************************************
!	WARNING: WE are only setup for NS arrays in this code!!!!
!****************************************************************************

!	ny=int((y/yseg)+.5)   
!	For ny even: nx=int((x/xseg)+.5)
!	For ny  odd: nx=int(x/xseg)
!		This places alternate rows of ny a half step over(to the right)
!		if we pick yseg=xseg*cos(60 deg) then we get a triangular
!		grid point that hexagonal spaced arryas fit on perfectly.
!  NOTE!!!!!:::For the triangular array we will need to relative arrays for
!		the telescopes. One for when the center is on an even
!		column('NS') and one for when its on an odd column('NS')(Rows
!		for 'EW') This is do to the symetry of the shifts of columns.
!		if were on an even colum the next colum over(and all the odd
!		coulmns) are shifted UP relative to it. If we are on an odd
!		column the next one over (and all even columns) are shifted
!		DOWN relative to it.

c	Presently only one type of input method is used.
c	veritas:
c		Uses the h_ntp ntupple file as input for telescope triggers.
c		Output is a V_ntp ntupple file

c	All parameters for the run are read from .inp file.

!	To add new input variables from h_ntp:
!		1:NTREAD_CW_HILLAS:
!			a:extend var() definitions(use tag names).
!			b:dd new variables of M_V loading(use ntp names).
!		2:KASTRUCTURES.INC
!			a:add to M_V structure(unse h_ntp names).
!		3:VERITAS_PROC:
!			a:add loading of variables into m.pheight for
!			  beg-of-shower
!	To add new output variable to v_ntp:
!		1:KASTRUCTURES.INC
!			a:Add new svariable to V_NTP structure.
!		2:VERITAS_PROC:
!			a:Add load of variables into v_ntp.

c	Files:
c	unit=1	Input parameter file.
c	unit=4  'H_ntp' input file
c	unit=8	Mfile that was used to create h_ntp file. We just need the 
c		header(mostly just for xseg and yseg)
c	unit=9	'V_ntp' output file.

c	Written by:
c	Mary P. Kertzman
c	Dept. of Physics and Astronomy
c	DePauw University
c	Greencastle, In. 
c	E-Mail: "kertzman@maat.physics.depauw.edu"

c	G.H.Sembroski
c	Physics Dept
c	Purdue University
c	W.Lafayette, In 47907
c	E-Mail: "SEMBROSKI@physics.purdue.edu"


c	Modified:

c	4/2/92 G.H.S. V:1:0:1:0:0:0.6
c		Impliment the 'GRID' search method for triggers.  This method
c		is added on top of the current stack method.  The stack is 
c		still needed to handle the itheta iphi records.	The GRID
c		method consists of making an array called GRID that represents
c		all nx ny positions on the ground.  As M records are read into
c		the stack their stack index (first index for a set with 
c		multiple itheta iphi records) is placed at the apropiate
c		position in the GRID array.  An array trigger search can then 
c		just use offsets to positions in the GRID array to find the
c		apppropiate stack indices.  This eliminates the stack
c		searches.  To handle the problem of GRID not being big enough
c		to contain all triggered areas the grid is relative and when
c		an area is found that is outsid the grid GRID can be zeroed
c		and refilled with a new relative origen that holds what needs
c		to be covered.  This refilling won't hapopen often and the 
c		whole method should still be much faster especially for
c		hadronic showers with their large number of itheta aiphi
c		records in the stack.

c	2/4/92 G.H.S. V:1:0:1:0:0:0.6
c		ASGAT:Include a PE_THRESHOLD level in the .inp inut file. This 
c		will cut on area triggers in the same way as the pesum used
c		in KASTRIG. PE_THRESHOLD should be .ge. PESUM. This allows 
c		us to vary the mirror threshold without having to rerun
c		KASTRIG. Later we could use individual mirror cuts.
c		Cut on PE_THRESHOLD is done in ARRAY_SCAN

c	16/7/96 GHS V:1:0:2:1:1:1:1.0
c		Add VERITAS processing. Use h_ntp Zebra ntupple file for input.
c		Allow for use of summary file. Output will be another Zebra
c		ntupple file to be used as input to PAW
c		Absorb the fileopen subroutine into the individual *_ini

c	12/8/96 GHS V:1:0:2:1:1:1:1.1
c		Add code to determine PARALLAXWIDTH in VERITAS_PROC.

c	21/8/96 GHS V:1:0:2:1:1:1:1.1
c		Configure for 9 telescope array, 3 in centre.

c	06/10/97 GHS V:1:0:2:1:1:1:1.2
c		Configure for 7 telescope array at 1300m

c	10/4/98 GHS V:1:1:5:1:1:1:1.0
c		Convert for wuse with triangular grid spacins for nx,ny
c		See note above.

c	26/6/98 GHS V:1:1:5:1:1:1:1.1
c		Due to a bug in kaslite the NS or EW option was truncated out
c		of pe_type. This caused KASASOMEGA to assume EW(when we really 
!		Have been using NS. So force NS!
 
!	06/7/98 GHSV:1:1:5:1:1:1:1.2
!		Debug: This will be a generalized debug for use with paw to 
!		convince ourselves that this program really does find array 
!		triggers. Mainly we write out 3 files.  One(unit=40) is the 
!		position of the telecopes in the array. The second (unit=41) is
!		the positions of the single telescopes that are hit for a 
!		shower. The last is the positon of the central telescope for 
!		an array trigger. These printouts are activated by setting 
!		DEBUG_TRIGGER=.true. in the KAS_ARRAY_TRIGGER.INC file

!	07/7/98 GHS  1:1:5:1:1:1:1.3
!		Fix the even/odd problem with veritasloc by relplacing it
!		with two arrays: VERITASLOC_EVEN,VERITASLOC_ODD
!		See the note above about evem/odd columns

	include 'kas_array_trigger.inc'
	character*80 ptitle

	character*80 version
	data version/'V:1:1:5:1:1:1:1.3'/	!Present version.

c	Version number.
c	The version number has one digit for each program in the KASCADE system.
c	seperated by ':'.   KAS_array_Trigger has 2 digits seperated by ".".
c	Digits for previous programs are for compatibility version to 
c	KAS_array_Trigger.  New compatibility version require later programs
c	to be modified due to changing output data formats, or require rerunning
c	of the data base due to corrections to important mistakes.
c	Digit 2:	KASCADE   version
c	      3:	KASLITE   version
c	      4:	KASSRTMRG version
c	      5:	KASTRIG   version
c	      6:	KASAOMEGA version	
c	      7:	KAS_ARRAY_TRIGGER	  version.sub version


	write(6,1205)version
1205	format(' KAS_ARRAY_TRIGGER***Version: ',a)

	call datain		!Get input parameters.

c	Initalize various things according to detector type.

	if(index(options,'VT').ne.0)then
		print*,' VERITAS Triangular Array.'
		call veritas_ini !Loads up for first search.
		call array_vecini !setp direction vectors	
	else
		print*,' KAS_ARRAY_TRIGGER_FATAL--Bad detector type:',
	1 detector
		stop
	endif

c	The M-file is sorted by nx then ny then itheta then iphi.

	if(index(options,'VT').ne.0)then
		call veritas_proc
	endif
	call exit	!No stop message.
	end




	subroutine arrayorder(array,isize,outarray,xseg,yseg,iref)
c	This routine orders an array after converting it to integer.
c	OUTARRAY is then made relative to ARRAY(x,iref)
c	Uses xseg and yseg from header.
c	Note that array is dimensioned(3,isize) since it also has
c	z information(used by dynamic tilt?)
c	Written 10/18/90 G.H.S.

c	Modified:

c	1/3/91 G.H.S.v:1:0:1:0:0:0.6
c		Convert to the multiple trigger search technique.  This 
c		technique requires that we have a relative array for each
c		of the possible 'mandatory' postions.  Then a particular
c		area can be tried at each array position. IREF specifies
c		whats to be 0,0.

	integer isize,iref
	real array(3,isize),xseg,yseg
	integer outarray(2,isize)

	do i=1,isize
c	convert to nx and ny
		nx=(array(1,i)/xseg)+100000
		ny=(array(2,i)/yseg)+100000
		nx=nx-100000
		ny=ny-100000
c	order these:by nx(descending) and within that ny(descending)
		if(i.ne.1)then
			do j=1,i-1
				if(nx.lt.outarray(1,j))goto 100
				if(nx.eq.outarray(1,j))then
					if(ny.lt.outarray(2,j))goto 100
					if(ny.eq.outarray(2,j))then
	print*,' ARRAYORDER-- Duplicate Areas:',j,' and',i
					endif
				endif
	!Swap them.
				nxtemp=outarray(1,j)
				nytemp=outarray(2,j)
				outarray(1,j)=nx
				outarray(2,j)=ny
				nx=nxtemp
				ny=nytemp
100			enddo
			outarray(1,i)=nx
			outarray(2,i)=ny
		else
			outarray(1,1)=nx
			outarray(2,1)=ny
	endif
	enddo
c	Now make everthing relative to element IREF
	ixref=outarray(1,iref)
	iyref=outarray(2,iref)
	do i=1,isize
		outarray(1,i)=outarray(1,i)-ixref
		outarray(2,i)=outarray(2,i)-iyref
	enddo
	return
	end


	subroutine array_scan(nx1,ny1,itheta1,iphi1,location,
	1 array_size,trigger,iarray_top,nhits)

c	This routine scans through the array to collect those 'M' file records 
c	that hit the array. The nx,ny,(itheta,iphi) of the 
c	'mandatory' detector(relative location 0,0) are given by 
c	nx1,ny1,itheta1,iphi1.
c	This is a generic scan with the location of all
c	the array elements(on the ground in relative nx and ny coords)
c	stored in LOCATION. ARRAY_SIZE has the number of array elements.
c	This routine just scans back through the stack records, starting at 
c	IARRAY_TOP looking for the
c	correct arraingments of areas(and itheta's, iphi's).  When it finds a
c	match that area is loaded into TRIGGER. 
c	 This routine_ does not read new
c	records into the stack. It assumes all applicable records are
c	already loaded into the stack(All cuts must be done externally).
c	For gammas itheta1 and iphi1 should default to 1.

c	Written: 10/29/90 G.H.S.

c	Modified:

c	4/12/91 G.H.S. v:1:0:1:0:0:0.3
c		Speed up things by calculating nhits in ARRAY_SCAN.
c		Also set up a new variable MAX_MISS, which ARRAY_SCAN
c		tests on. If more then MAX_MISS areas are not found ARRAY_SCAN
c		quits. This speeds up things by a factor of 3.But see 2/2/92

c	2/2/92 G.H.S. v:1:0:1:0:0:0.6
c		Flag for ASGAT the positions an m record has been used for by
c		setting its pehight word for that position to -1. Use the
c		new routine_ _REC to put such a modified record back
c		into the stack.

c	4/2/92 G.H.S. v:1:0:1:0:0:0.6
c		Modify this to use the GRID search method.

c	2/4/92 G.H.S. V:1:0:1:0:0:0.6
c		ASGAT:Include the use of a PE_THRESHOLD level.. This 
c		will cut on area triggers in the same way as the pesum used
c		in KASTRIG. PE_THRESHOLD should be .ge. PESUM. This allows 
c		us to vary the mirror threshold without having to rerun
c		KASTRIG. Later we could use individual mirror cuts.

	include 'kas_array_trigger.inc'
	integer nx1,ny1,nhits
	integer itheta1,iphi1,array_size,iarray_top,
	1 location(2,array_size)

	record /m/m
	record /trigger/trigger
	logical out_of_range

c	Use the GRID to indicate first if the requested nx,ny exists and
c	second get where its group(itheta,iphis) start in the stack.
c	nx and ny in the correct relative positions.  These relative positions
c	are stored in LOCATION. There are ARRAY_size of them. LOCATION, 
c	and RECORD_LENGTH are set during the initalization part of 
c	this program.

c	Zero the trigger array
	do j=1,array_size
		do k=1,record_length
			trigger.m(j).recbuf(k)=0.
		enddo
	enddo
	nhits=0		!Keep track of found matches.
	nmiss=0		!Count missed areas.

	do j=1,array_size	!Iterate over the area
		mx=nx1+location(1,j)	!Area we are looking for.
		my=ny1+location(2,j)
c	See if its in the stack by looking in GRID
c	ASSUME its position is contined in grid
		ngx=mx-ngxref
		ngy=my-ngyref
		i=grid(ngx,ngy)		!Get stack index if it exists
		if(i.eq.0)goto 102	!Try next array position

!debug
!		print*,' mx,my:',mx,my
!enddebug
c	We found the area, see if the correct itheta,iphi exists.
c	For this we will have to search back through the stack.
103		call getmrec(m,i,out_of_range)
		if(out_of_range)goto 100	!Stack is empty. no match quit.
	!Check nx,ny
		if(m.nx.ne.mx.or.m.ny.ne.my)goto 102
						!no itheta,iphi match
	!see if we match itheta,iphi.
		if(m.itheta.lt.itheta1)then
			goto 102
		elseif(m.itheta.eq.itheta1)then
			if(m.iphi.lt.iphi1)then
				goto 102
			elseif(m.iphi.eq.iphi1)then
						!A match. Load it.
				do k=1,record_length
					trigger.m(j).recbuf(k)=
	1 m.recbuf(k)
			!I think this gets Pheight for this position into
			!TRIGGER even though we set it to -1 below!
				enddo
!We could make a cut on PE_THRESHOLD for counting this 
!area towards the multiplicity sum. Keep the record in trigger even if it
!doesent cross pe_threshold.
!set flag in pehight to -1 for this postion and
!replace the record into the stack.

!If we want a new threshold check!
!				if(detector.eq.'A ')then
!			if(m.pheight(j).ge.pe_threshold)then
!				nhits=nhits+1
!			endif
!				endif

				m.pheight(j)=-1
				call replace_rec(m,i)
				nhits=nhits+1 !Count matches.
				goto 102	!go on to next area.
			endif
		endif
!If no match and we don't need a new area get a new record.
		if(i.eq.ibottom)goto 100	!Make sure we havn't hit bottom.
		i=i-1
		goto 103	!try again
102	if(location(1,j).eq.0.and.location(2,j).eq.0)goto 100  !Quit when we
						!hit the current ref area.
	enddo			
100	return
	end
 

		
	subroutine array_vecini
!******************************************************************************
!	Make up an x and y array of locatinons in the mirror plane of the
!central det4ector of the projected position of each telescope. X-axis is as
!defined in KASTRIGGER,: It is the intersection of the mirror plane and the
!ground plane.       This is for VERITAS.
!******************************************************************************

!	All directions of the mount are given in the upward(-Z) direction 
!	convention.  Mount dn is neg.
!	Use 2 indices. First is for the azmutial angle THETA. It steps in 
!	mhead.step_size
!	deg from 0 deg(nominal direction of the mount) to 15 deg.
!	The second index is the PHI angle which steps in 
!	2*pi/((n-1)*mhead.phi_steps) radians
!	from(N= whatever Theta were at). For THETA=0 PHI doesn't matter.
!	Remember all pe directions are relative to original mount.

!	written: 10/22/90 G.H.S.

!	Modified:

!	24/5/94 GHS V:1:1:2:1:1.1
!		Add mhead.phi_steps to kasstructures.inc. phi_steps is base of
!		phi divisions for an itheta circle. total steps is 
!		(itheta-1)*mhead.phi_steps. Normally its been 6 but we want 
!		different value for w10m do to its hex structure immaging array.
!		Use mhead.phi_steps=5 for w10m.
!	26/1/98 GHS V:1:1:5:1:1.1"
!		Convert from f77 to f90.

!	13/5/98	GHS V:1:1:5:1:2.0
!		Convert from VECINI 
!		Set up for VERITAS.
!		1:Mount direction supplied here now. Unlike the past we now
!		  do not find the relative values of pe directions and timings
!		  to the mount plane in KASLITE. Instead the PES file has the
!		  directions and timings for when the pe's hit the ground.
!	          It is in this program where we read in the mount direction
!		  (DATAIN) and preform the relative pe directions and adjusted
!		  timing calculations. Put dl,dm,dn into mhead in place of where
!		  it used to come in in KASLITE.
!		In this version we find the absolute(not relative) unit vectors
!		of the mount directions. We also determine the x and y unit
!		vectors in each mirrror plane. Do this now for speed.
!		Bt doing things this way we can use whipple_tilt in a fairly
!		unmodified form(we do change definifnos of x,y axis in mirror
!		plane).
!		  Do this to increase flexability in mount direction choice.

!	25/6/98 GHS
!		Convert from MOUNT_VECINI in KASTRIGGER.
!		Add location projection stuff.

	implicit none

	include 'kas_array_trigger.inc'
	real  theta,phi,delta_phi,vlength
	integer  nsteps,i,j,k
	integer nyc,nxc
	real*8 pi
	parameter (pi=3.141592654)
!	Do theta=0 first. only phi=0.
!	dl,dm,dn Read from input .par file.
!	Its is around this vector that our sky pixels are centered.
	dlm(1,1)=mhead.pe_head.dl
	dmm(1,1)=mhead.pe_head.dm
	dnm(1,1)=-sqrt(1.-dlm(1,1)**2-dmm(1,1)**2)
		!Remember:Mount points UP!(dn <0)



!	Determine unit x,y unit vectors in mirror plane.
!	Assure that in the mirror plane the new y is in the same plane as the 
!	z axis. This allows for easy rotation of the
!	axis plane when we want to find image directions later in VERITAS
!	So new x is perp to mount normal and perp to Z axis. New y is perp to 
!	new x and perp to mount normal
!	=> X=M x Z 	X = x unit vector;M=muount unit vector Z=z unit vector
!	This gets us x goint to right in mirror plane as we look down on it.	
!	M=(dl,dm,dn),Z=(0,0,1)
!	M x Z= (dm,-dl,0) = x   But we still have to normalize it
	vlength=sqrt(dmm(1,1)**2+dlm(1,1)**2)
	xdlm(1,1)=dmm(1,1)/vlength
	xdmm(1,1)=-dlm(1,1)/vlength
	xdnm(1,1)=0

!	Y unitvector from X x M=(xdm*dn-xdn*dm,xdn*dl-xdl*dn,xdl*dm-xdm*dl)
	ydlm(1,1)=xdmm(1,1)*dnm(1,1)-xdnm(1,1)*dmm(1,1)
	ydmm(1,1)=xdnm(1,1)*dlm(1,1)-xdlm(1,1)*dnm(1,1)
	ydnm(1,1)=xdlm(1,1)*dmm(1,1)-xdmm(1,1)*dlm(1,1)
	vlength=sqrt(ydlm(1,1)**2+ydmm(1,1)**2+ydnm(1,1)**2)
	ydlm(1,1)=ydlm(1,1)/vlength
	ydmm(1,1)=ydmm(1,1)/vlength
	ydnm(1,1)=ydnm(1,1)/vlength

!	x_mplane(itheta,iphi,i),y_mplane(itheta,iphi,i) are projected 
!	coordinartes of i'th mount location in the central mirror
!	plane(center_index) specified by direction itheta,iphi.

 	do i=1,v_size
!convert to triangular array stuff here.
!	NOTE: since the origen is by definition an EVEN column we use that
		nxc=veritasloc_even(1,i,center_index)
		nyc=veritasloc_even(2,i,center_index)
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
!Force NS
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
!       	    	if(index(mhead.pe_head.petype,'NS').eq.0)then
!						!'EW' First
!			y_gplane(i)=nyc*mhead.pe_head.yseg
!			if(mod(nyc,2).eq.0)then		!even
!				x_gplane(i)=nxc*mhead.pe_head.xseg
!			else
!				x_gplane(i)=(nxc+.5)*mhead.pe_head.xseg
!			endif
!		else
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
						!'NS' Now
			x_gplane(i)=nxc*mhead.pe_head.xseg
			if(mod(nxc,2).eq.0)then		!even
				y_gplane(i)=nyc*mhead.pe_head.yseg
			else
				y_gplane(i)=(nyc+.5)*mhead.pe_head.yseg
			endif
!		endif
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX



				!Project the ground x,y coords to the initial
				! mount mirror
				!plane. This is just dot porduct of x and y
				!unit mirror plane unit vectors with the 
				!ground plane position vector.
				!(remember z of ground plane is 0)
				x_mplane(1,1,i)=
	1 x_gplane(i)*xdlm(1,1) + y_gplane(i)*xdmm(1,1)
				y_mplane(1,1,i)=
	1 x_gplane(i)*ydlm(1,1) + y_gplane(i)*ydmm(1,1)


	enddo
				!we can use these values pelow(I think)

!	Now do the rest(if needed).
	if(hadron)then
		do i=2,mhead.itheta_max
			theta=(i-1)*mhead.step_size*pi/180.
			nsteps=(i-1)*mhead.phi_steps
			delta_phi=360.0/nsteps
			phi=0.
			do j=1,nsteps
				phi=delta_phi*(j-1)*pi/180.
				!repoint mount unit vecotor by theta,phi
			        call vector_redirect(theta,phi,
	1 dlm(1,1),dmm(1,1),dnm(1,1),dlm(i,j),dmm(i,j),dnm(i,j))
			!Now x,y unit vectors in mirror plane
				vlength=sqrt(dmm(i,j)**2+dlm(i,j)**2)
				xdlm(i,j)=dmm(i,j)/vlength
				xdmm(i,j)=-dlm(i,j)/vlength
				xdnm(i,j)=0
				ydlm(i,j)=
	1 xdmm(i,j)*dnm(i,j)-xdnm(i,j)*dmm(i,j)
				ydmm(i,j)=
	1 xdnm(i,j)*dlm(i,j)-xdlm(i,j)*dnm(i,j)
				ydnm(i,j)=
	1 xdlm(i,j)*dmm(i,j)-xdmm(i,j)*dlm(i,j)
			    	vlength=  
	1 sqrt(ydlm(i,j)**2+ydmm(i,j)**2+ydnm(i,j)**2)
				ydlm(i,j)=ydlm(i,j)/vlength
				ydmm(i,j)=ydmm(i,j)/vlength
				ydnm(i,j)=ydnm(i,j)/vlength
			enddo
				!Now projected mount locations.
			do k=1,v_size
				!Project the ground x,y coords to the mirror
				!plane. This is just dot porduct of x and y
				!unit mirror plane unit vectors with the 
				!ground plane position vector.

				x_mplane(i,j,k)=
	1 x_gplane(k)*xdlm(i,j) + y_gplane(k)*xdmm(i,j)

				y_mplane(i,j,k)=
	1 x_gplane(k)*ydlm(i,j) + y_gplane(k)*ydmm(i,j)
			enddo
		enddo
	endif
	return
	END
!******************************************************************************



	subroutine datain
c        Read in the input parameters from the parameter file.

c     Modified:

        include 'kas_array_trigger.inc'

        call vaxdatini        !Open input file and do the date and time.

	read(1,1000)hadron		!Enable proton proccessing flag.
1000	format(l3)

	read(1,1001)hmult		!Required Array trigger multiplicity. 
1001	format(i3)     

	read(1,1002)swindow		!Array trigger window width.(SL)
1002	format(f10.2) 

	read(1,1003)options		!'VA':VERITAS Array
1003	format(a)

	read(1,1002)pe_threshold 	!New mirror threshold.
					!For now, same for all mirros

	write(6,1017)hadron
1017	format('      HADRON:Enable proton processing(E=true=Yes)   =',
	1 l3)
	write(6,1018)hmult
1018	format('                        HMULT:Array Trigger level   =',
	1 i3)
	write(6,1019)swindow
1019	format('                   SWINDOW:ArraySlide window(nsec)  =',
	1 f10.2)

	write(6,1020)options
1020	format('                                 OPTIONS:VA=VERITAS  =',
	1 a)

	write(6,1021)pe_threshold
1021	format('          PE_THRESHOLD:Individual Mirror threshold  =',
	1 f10.2)
	return
	end



	subroutine dynamic_tilt(itheta,iphi,array_pos,delta_t)
c	Find Dynamical timing corection for tilted mounts.
c	Tits specified by itheta,iphi.

c	Written: 1/3/91 G.H.S. V:1:0:1:0:0.0


c	Modified:
	include 'kas_array_trigger.inc'
	integer*4 itheta,iphi
	real array_pos(3),delta_t
c	Note:ARRAY_POS is dimensioned. ARRAY_POS(3) is Z height.
c	     We arn't using it yet but we probably should be!!!!!

	parameter  (c_light=2.99792e+8)     !Speed of light in vacume.m/sec

c	Set up the mount constants.
	tdlm=dlm(itheta,iphi)
	tdmm=dmm(itheta,iphi)
	xg=array_pos(1)
	yg=array_pos(2)
				!local ground coords of Mount relative
		 		!to central mount.
c        The length of the track from hobs to the tilted mount is:
c        dist=((DLm,DMm,DNm) . (xg,yg,zg))
c        (DLm,DMm,DNm) is normal to mount plane pointed at sky.
c	Calculate perendicular distance from xg,yg,zg=0
c	to plane of mount.
        dist=(tdlm*xg+tdmm*yg)        !zg*dn=0.

c        Calculate timing correction
c        DIST sign is improtant.
	delta_t=(dist/c_light)*1.e9	!Convert c_light to
							!m/nsec
	return
	end


	subroutine fill_stack(nxrequired,nyrequired)
c	Fill the stack until it surpasses nxrequired and nyrequired.

c	written:11/1/90 G.H.S.

c	Modified:

c	11/12/90 G.H.S.
c		Check to see if top of stack is already beyond
c		requiuired limits.

c	4/2/92 G.H.S. V:1:0:1:0:0:0.6
c		For use with GRID scheme every time a new area is
c		added to the stack insert the lead records stack index into 
c		GRID.  Check always that it will fit in the grid. Remapping
c		the grid is the responsibility of the calling routine.

	include 'kas_array_trigger.inc'
	record /m/m
	logical out_of_range
		
	i=itop
	call getmrec(m,i,out_of_range)
	if(out_of_range)then
		print*,' KASMULT_FATAL-FILLTRIG-- Out Of Range'
		stop
	endif

c	Check to see that this 'M' record spans the array.
102	if(m.nx.lt.nxrequired)goto 101
	if(m.nx.eq.nxrequired)then
		if(m.ny.le.nyrequired)then
			goto 101	!Fudge here. get all 
		else
			return
		endif
							!ithetas and iphi's.
	else
		return
	endif
c	always put 'M' record into stack uless were at the end of a shower.
101	nt=nth
	ntold=nt
	call ntread_cw_hillas(nt,ntold,m)

c	Just plain quit if there arn't any more.
	if(v_eof.or.v_beg_of_shower)then
		return
	endif

c	Put it into the stack
	call putmrec(m)

c	Detrermine if record is within GRID limits.
	nxg=m.nx-ngxref
	if(nxg.le.max_grid.and.nxg.ge.0)then
		nyg=m.ny-ngyref
		if(nyg.le.max_grid.and.nyg.ge.0)then
			grid(nxg,nyg)=itop	!save stack index.
					!Because of our back wards search method
					!for itheta,iphi we want the highest
					!itheta iphi index in the stack.
		endif
	endif
	goto 102	!Test this one.
	end



	subroutine getmrec(m,i,out_of_range)
c	This routine_ retrieves records from an array used as a circular stack.
c	Index ITOP points to the last record entered into the stack.
c	Index IBOTTOM points to the oldest record of interest in the stack
c	The stack is stored in an array which is treated as a circular register.
c	Index i is used to indicate which record we are to extract.
c	If i is out of range of itop and ibottom out_of_range is set to .true.

c	Modified:

c	1/12/90 G.H.S.
c		Requests for i=max_stack+1 are requests for i=1
c		wrap it around.

	include 'kas_array_trigger.inc'
	record /m/m
	logical out_of_range

	out_of_range=.false.
c	determine if we are done.
c	Only in this program is i 'wrapped around'  A request for record
c	i=0 is valid. It means record i=max_stack.
c	A request for i=max_stack+1 is also valid. Its a request
c	for event 1.
	if(i.gt.max_stack+1.or.i.lt.0)then
			out_of_range=.true.
			return
	endif
!Note we are changing an input parameter here!!!!
	if(i.eq.0)i=max_stack
	if(i.eq.max_stack+1)i=1
c	Check validity of request.
	if(itop.ge.ibottom)then		!Itop has not wrapped around.
		if(i.lt.ibottom.or.i.gt.itop)then	!Invalid request.
			out_of_range=.true.

			return
		endif

	elseif(i.lt.ibottom.and.i.gt.itop)then
			out_of_range=.true.
			return
	endif


c	Retreive the record.
	do j=1,record_length
		m.recbuf(j)=mrecord(j,i)
	enddo
	return
	end



	Subroutine ntread_cw_hillas(nt,ntold,m_v)
c	This routine reads certain specified variables from the h_ntp
c	ntupple NT into structure H_NTP.
c	H_NTP is put into a common block in KAS_ARRAY_TRIGGER.INC
c	Pertinent values from h_ntp are loaded into the m_v record.
c	Modified:

c	17/7/96 GHS V:1:0:2:1:1:1:1.0
c		This is a version modified from ntuple_cw_hillas as used
c		in KASAOMEGA.

c	12/8/96 GHS V:1:0:2:1:1:1:1.1
c		Add XMEAN and YMEAN of image to m_v record. Used in 
c		PARALLAXWIDTH determination. Also add DIST to m_v.
c		Use to indicate if image had enough pixels after CLEANUP
c		to determineimage parameters

c	20/10/97 GHS
c		Add ALPH and SDEVXY to m_v record. Use in Source position 
c		calculation.
	common/quest/iquest(100)
	integer nt,ntold,itfirst/0/
	
	
	include 'kas_array_trigger.inc'
	record/m_v/m_v
	integer lrec/0/,lsize,ifirst,isymst,istat,irow,ivar,nevents

	character*80 filename,var(h_ntp_size)
	real tep_old,id_old
	common /pawc/paw(2000000)	!for ntuple creation
	
	if(itfirst.eq.0) then
c	Open the input file.(These are all hbook commands.)

		call hlimit(2000000)	!Defines maximum total size of PAWC
		isymst=lib$get_symbol('nt_h_file',filename,nchar)
		if(.not.isymst)stop ' nt_h_file is not defined'
		iquest(10)=64*1024-1	!Set max number of records in ntupple
					!file. Default is 32000 blocks.
		call hropen(4,'HTUPLE',filename,'Q',lrec,istat)
					!Open unit 4, with arbitrary 
					!HBOOK name:'htuple'
					!file name from 'p2'
					!File is 'Q' which means 'existing'
					!and up tp 65000
					!0=lrec.Block size(use existing)
		if(istat.ne.0)then
			print*,' istat:',istat
			print*,' filename:',filename
			stop 'hropen problem'
		else
			print*,' HTUPLE opened OK!'
			itfirst=1
		endif
	endif
	if(nt.ne.ntold)then
		if(ntold.gt.-1000000)then
			call hcdir('//PAWC',' ')
			call hdelet(ntold)
		endif

c	Read in the ntupple
		call hcdir('//HTUPLE',' ')
		call hrin(nt,99999,0)

c		Define the variables.
c		For now h_ntp is all real, so its easy, but if we need to
c		go to Integers or logicals we can do that here also.
c		Also we could compress it here also with a little work, but
c		not now.(see hbook manual pg.24-27,pg.33-47)

					!Clear address for this ntuple
		call hbname(nt,' ',0,'$CLEAR')
					!Set address for ntupple varibles.		call hbnt(nt,'PES',' ')
					!Use address of first word of H_ntp.
		call hbname(nt,'h_ntp',h_ntp.recbuf(1),'$set')

		call hnoent(nt,nevents)
		print*,' Number of events in input ntupple:',nevents
c	Set up an array with the names of only those variables we need from
c	each event. This means only those will be read. Make for good 
c	efficiency. These are the tag values.
                var(1)='tep'
		var(2)='id'
		var(3)='x'
		var(4)='y'
		var(5)='theta'
		var(6)='phi'
		var(7)='xmean'
		var(8)='ymean'
		var(9)='d'
		var(10)='alph'
		var(11)='sdevxy'
		var(12)='size'
		var(13)='xm'
		var(14)='ym'
		var(15)='emaltm'
		var(16)='x_off'
		var(17)='y_off'

		ivar=17
	!Add more as we find we need them.
c	Specifie the variables to load by reading first row(first event).
		irow=1
		call hgntv(nt,var,ivar,irow,ierr)
		if(ierr.ne.0)then
		print*,' ntread_cw_hillas--Error in hgntv:ierr:',ierr
			!This may be end of file.
			v_eof=.true.
			return
		endif
		tep_old=h_ntp.tep
		id_old=h_ntp.id
		ntold=nt
		v_eof=.false.
		record_length=mv_size	!This is a global variable used to
					!copy m_v and m records into and out of
					!the stack.
	else
	
c	Read next event.
		if(irow.gt.nevents)then
		print*,'NTREAD_CW_HILLAS:All events read:Setting EOF'
		v_eof=.true.
		return
		endif


		call hcdir('//HTUPLE',' ')
		call hgntf(nt,irow,ierr)
		if(ierr.ne.0)then
		print*,' ntread_cw_hillas--Error in hgntf:ierr:',ierr
			!This may be end of file.
			v_eof=.true.
			return
		else
			v_eof=.false.
		endif
c	Check to see were not in a new shower.
		if((tep_old.ne.h_ntp.tep).or.(id_old.ne.h_ntp.id))then
			v_beg_of_shower=.true.
			tep_old=h_ntp.tep
			id_old=h_ntp.id
			v_eof=.false.
		else
			v_beg_of_shower=.false.
		endif
	endif

c	Load up m structure.
!Notice: This is correct even for triangular array. Gives correct nx,ny.
!GHS 10/4/98
!But worry about round down for negative values.
	vnx=h_ntp.x/mhead.pe_head.xseg+100000
	vny=h_ntp.y/mhead.pe_head.yseg+100000
	m_v.nx=vnx
	m_v.ny=vny
	m_v.nx=m_v.nx-100000
	m_v.ny=m_v.ny-100000

	m_v.itheta=h_ntp.theta
	m_v.iphi=h_ntp.phi
	do i=1,v_size
		m_v.pheight(i)=1.		!Set all pulse heights to 1
	enddo				!This array used to flag which area
					!has been used in which position in
					!the array. Checked agains pe_threshold
					!which we we set to 1 for now.
	m_v.xmean=h_ntp.xmean		!USE h_NTO Variable names here. not
	m_v.ymean=h_ntp.ymean		!TAGS
	m_v.dist=h_ntp.dist
	m_v.alph=h_ntp.alph
	m_v.sdevxy=h_ntp.sdevxy
	m_v.size=h_ntp.size
	m_v.xmpln=h_ntp.xmpln
	m_v.ympln=h_ntp.ympln
	m_v.emaltmpln=h_ntp.emaltmpln
	m_v.x_offset=h_ntp.x_offset
	m_v.y_offset=h_ntp.y_offset

c	ADD more stuff here as we need it.
	irow=irow+1	!Bump event pointer

	return
	end
              




	Subroutine ntwrite_cw_veritas(nt,ntold)
c	This routine enteres data from structure V_NTP into ntuple nt.
c	This is the version that makes a column wide ntuple.
c	V_NTP is put into a common block in KAS_ARRAY_TRIGGER.INC

c	Modified:

c	17/7/96 GHS V:1:0:2:1:1:1:1.0
c		This is a version modified from ntuple_cw_hillas as used
c		in KASAOMEGA. This one writes the VERITAS array trigger
c		ntupple

c	10/4/96 GHS
c
c		Increase max number of records to maximum zera allows
c		which is 64 k (default=32000)
c		Requires 'Q' option in HROPEN and setting of RZ variable
c		IQUEST(10) in RZ common are QUEST.
c		If this value is too small one gets the error:
c		      'RZOUT . Request exceeds quota'	
c		May also be able to increase record size to solve this kind
c		of problem.

	common/quest/iquest(100)
	integer nt,ntold,itfirst/0/
	
	
	include 'kas_array_trigger.inc'
	integer lrec/4096/,lsize,ifirst,isymst,istat,icycle

	character*80 filename

	common /pawc/paw(2000000)	!for ntuple creation

	if(itfirst.eq.0) then
c	Open the ouput file.(These are all hbook commands.)

!		call hlimit(2000000)	!Defines maximum total size of PAWC
		isymst=lib$get_symbol('nt_v_file',filename,nchar)
		if(.not.isymst)stop ' nt_v_file is not defined'
		iquest(10)=64*1024-1	!Set max number of records in ntupple
					!file. Default is 32000 blocks.
		call hropen(9,'VTUPLE',filename,'NQ',lrec,istat)
					!Open unit 4, with arbitrary 
					!HBOOK name:'vtuple'
					!file name from 'p2'
					!File is 'N'ew file
					!File is 'Q' to allow non defualt max
					!zebra reccords
					!1024=lrec.Block size(recomended)
		if(istat.ne.0)then
			print*,' istat:',istat
			print*,' filename:',filename
			stop 'hropen problem'
		else
			print*,' VTUPLE opened OK!'
			itfirst=1
		endif
		call hbset('BSIZE',4096,istat)
	endif
	if(nt.ne.ntold)then
					!Remove previous ntupple if defined.
		if(ntold.gt.-1000000)then
			call hrout(ntold,icycle,' ')
			call hcdir('//PAWC',' ')
			call hdelet(ntold)
		endif

		call hcdir('//VTUPLE',' ')

c		Define the variables.
c		For now v_ntp is all real, so its easy, but if we need to
c		go to Integers or logicals we can do that here also.
c		Also we could compress it here also with a little work, but
c		not now.(see hbook manual pg.24-27)

		call hbnt(nt,'PES',' ')
		do i=1,v_ntp_size
			call hbname(nt,'V_NTP',v_ntp.recbuf(i),
	1 tags_v_ntp(i)//':R')
					!ntuple id is nt
					!Block (common) Name is 'V_NTP'
					!Tags_v_ntp=names of the varables
					!assumed to all be real, no compression
		enddo
		ntold=nt
	endif
	
c	Enter this image
	call hcdir('//VTUPLE',' ')
	call hfnt(nt)
	return
	end
!******************************************************************************

	subroutine putmrec(m)
c	This routine stores M file records in an array as a circular stack.
c	Index ITOP points to the last record entered into the stack.
c	Index IBOTTOM points to the oldest record of interest in the stack
c	The stack is stored in an array which is treated as a circular register.
c	for the very first call to this routine ITOP and IBOTTOM start=0.
c	After this routine ITOP becomes =1. The user is required to set 
c	ibottom to 1.

	include 'kas_array_trigger.inc'
	record /m/m

c	First see if the new record will cause the top to wrap around.
c	Leave 1 line gap
	itop=itop+1
	if(itop.gt.max_stack)then
		if(ibottom.eq.1)then
			write(6,1000)m.nx,m.ny,m.itheta,m.iphi,
	1 itop,ibottom
1000	format(' KASMULT_FATAL--PUTMREC--Stack wrap around:nx,ny,
	1 itheta,iphi,itop,ibottom:',/,' *****',6i5)
			stop   'FATAL-Should never get here!!'
		else
			itop=1
		endif
	elseif(itop.eq.ibottom)then
		write(6,1000)m.nx,m.ny,m.itheta,m.iphi,itop,ibottom
		stop   'FATAL-Should never get here!!'
	endif

c	Store the record on the stack
	do j=1,record_length
		mrecord(j,itop)=m.recbuf(j)
	enddo

!debug_trigger
	if(debug_trigger)then
						!'NS' Now
		xdeb=m.nx*mhead.pe_head.xseg
		if(mod(m.nx,2).eq.0)then		!even
			ydeb=m.ny*mhead.pe_head.yseg
		else
			ydeb=(m.ny+.5)*mhead.pe_head.yseg
		endif
		write(41,4341)xdeb,ydeb,m.itheta,m.iphi,m.nx,m.ny
4341	format(' ',2(e14.7,','),4(i5,','))
	endif

	return
	end


	subroutine remap_grid(nx,ny)
c	This routine_ is for use with the GRID array trigger search
c	method. Its purpose to to remap the m records into the GRID
c	array.  NX and NY are assumed to be the new current reference
c	area and the GRID will be remapped in such a way for this
c	areas array search limits will fit.

c	Written 4/2/92 G.H.S.

c	Modified:

	include 'kas_array_trigger.inc'
	record /m/m	
	integer nx,ny
	logical out_of_range

c	Zero the grid
	do i=0,max_grid
		do j=0,max_grid
			grid(i,j)=0
		enddo
	enddo

c	Determine what the new reference point of the GRID should
c	be. That is what nx,ny should grid(0,0) represent?
	ngxref=nx	!Never need earlier then current refs nx.
	ngyref=ny+nymin
	write(6,1300)nx,ny,ngxref,ngyref
1300	format(' Remapping GRID:nx,ny,ngxref,ngyref:',4i5)

c	Now refill the GRID from the stack

	i=itop
100	call getmrec(m,i,out_of_range)
	if(out_of_range)goto 101

c	Detrermine if the record is within GRID limits.
	ngx=m.nx-ngxref
	if(ngx.le.max_grid.and.ngx.ge.0)then
		ngy=m.ny-ngyref
		if(ngy.le.max_grid.and.ngy.ge.0)then
			if(grid(ngx,ngy).eq.0)then
				grid(ngx,ngy)=i	!save stack index.
					!Because of our back wards search method
					!for itheta,iphi we want the highest
					!itheta iphi index in the stack.
			endif
		endif
	endif
	if(i.eq.ibottom)goto 101
	i=i-1
	goto 100	!And another

101	return
	end


	subroutine replace_rec(m,i)
c	This routine replaces modified records to the m file stack.
c	Index i is used to indicate which record we are to modify.

c	Written:	2/2/92 G.H.S.
c	Modified:

	include 'kas_array_trigger.inc'
	record /m/m

c	Note that no attempt is made to check on the validity of i.
c		It better be ok!
c	Replace the record.
	do j=1,record_length
		mrecord(j,i)=m.recbuf(j)
	enddo
	return
	end



	subroutine slide(times,mirror,window,ntubes,mult)      
c	Gives multiplicity in sliding window.
c	Sort the times:

C	Modified:

	real*4 times(ntubes),goodtime(26)

	integer mirror(ntubes)
	integer list(26)
	nhits=0
	do i=1,ntubes
		if(mirror(i).ne.0)then
			nhits=nhits+1
			goodtime(nhits)=times(i)
		endif
	!Order the times.
		if(nhits.ne.1)then		!Skip first one
			do k=1,nhits-1
				if(goodtime(nhits).lt.goodtime(k))then
					ttemp=goodtime(nhits)
					goodtime(nhits)=goodtime(k)
					goodtime(k)=ttemp
				endif
			enddo
		endif
	enddo

C  Find the window with the maximum multiplicity
c	Times are all ordered in goodtime
	imin=0
	imax=0
	mult=0
	do i=1,nhits
		endtim=goodtime(i)+window	!End of window		   
		tmult=0				!Init multiplicity for this 
						!window
		do j=i+1,nhits
			if(goodtime(j).gt.endtim)goto 111
		enddo

		!Comes here when last time is included in window. bounce things
		!to make following code work
		j=nhits+1
111		if(j-i.gt.imax-imin+1)then
			IMIN=i			!Keep starting time index
			IMAX=j-1		!endtime index
			mult=imax-imin+1
		endif
	enddo
	return
	end


	subroutine trim_stack(nxlast,nylast)
c	Trim off the bottom of the stack of areas before
c	nxlast,nylast. Don't trim itheta,iphi.(speed reasons)

c	written:11/8/90 G.H.S.

c	Modified:

	include 'kas_array_trigger.inc'
	record /m/m
	logical out_of_range

c	look at the bottom record
100	if(ibottom.eq.itop)return
	call getmrec(m,ibottom,out_of_range)
	if(out_of_range)then
		Print*,' KASMULT_FATAL-TRIM_STACK--Stack empty-a no-no'
		STOP
	endif
c	Check to see that this 'M' record can be dropped.
	if(m.nx.lt.nxlast)then
		ibottom=ibottom+1
		goto 100
	elseif(m.nx.eq.nxlast)then
		if(m.ny.lt.nylast)then
			ibottom=ibottom+1
			goto 100
		endif
	endif
	return
	end
!******************************************************************************


        subroutine vaxdatini
c           Open input data file.
c	Modified:
c	10/30/91 G.H.S.  V:1:0:1:0:0:0.4
c		For machine independence replace VMS for$date, for$time calls
c		with DATE and TIME.

        include 'kas_array_trigger.inc'
        dimension adate(3),hour(2)

c      GET TIME AND DATE OF THIS RUN
       call date(adate)
       call time(hour)
       WRITE(6,1000)adate,hour
1000  format(' ',15x,3a4,5x,2a4,
	1 '--------KAS_ARRAY_TRIGGER (VAXDATINI)------')

        OPEN(1,ACCESS='SEQUENTIAL',STATUS='OLD',READONLY,shared)
        return
        end
!******************************************************************************

	SUBROUTINE VECTOR_REDIRECT(theta,phi,dl,dm,dn,rdl,rdm,rdn)
!******************************************************************************
!	Take a unit vector dl,dm,dn and redirect it by angle theta in
!	azmituthal direction phi(theta,phi in radians) from direction of
!	dl,dm,dn and put resultant unit vector into rdl,rdm,rdn.
!******************************************************************************

!	Written by:

!	G.H.Sembroski
!	Physics Dept
!	Purdue University
!	W.Lafayette, In 47907
!	E-Mail: "sembroski@physics.purdue.edu"
!	14/5/98

!	Modified:

!	This is an adaptation of the GEOM subroutine used in KASCADE and other
!	places.
	IMPLICIT NONE
	real  theta,phi,dl,dm,dn,rdl,rdm,rdn
       real*8 a(9)
	real*8 b(3),r9(3)
	real  f,vlength

       f=sqrt(dl*dl+dm*dm)       !f is sin of verticle angle.
       a(1)=dl*dn/f
       a(2)=-dm/f
       a(3)=dl
       a(4)=dm*dn/f
       a(5)=dl/f
       a(6)=dm
       a(7)=-f
       a(8)=0.
       a(9)=dn
!             | a(1) a(2) a(3) |  | b(1) |  |r9(1) |
!             | a(4) a(5) a(6) | x| b(2) |= |r9(2) |
!             | a(7) a(8) a(9) |  | b(3) |  |r9(3) |
!             vector b is new direction in frame of
!             track
!             vector a(3);a(6);a(9)  is track in lab
!             frame  -v
!             vector a(2);a(5);.. is vector product
!             vxz normalized (mod=1)

       b(1)=cos(phi)*sin(theta)
       b(2)=sin(phi)*sin(theta)
       b(3)=cos(theta)

! standard routine for multiplying of matrices
       call mtxmlt(a,b,r9)

! new direction cosines
!      Force unit length to the vector.
!     10/23/86
       vlength=sqrt(r9(1)**2+r9(2)**2+r9(3)**2)
       rdl=r9(1)/vlength
       rdm=r9(2)/vlength
       rdn=r9(3)/vlength
       return
       end
!******************************************************************************


	subroutine veritas_ini
c	This routine sets up everthing we need to prepare to process for
c	the VERITAS  array. The  input and ouput ntupple files are inited upon
c	first call to their repective routines.
c	We do however have to read in the m_file header assigned to unit 8.
c	First we load the array location array VERITASLOC with
c	the relative offsets to all the telescopes from the leading telescope.
c	We start the search with the first event.  We don't have to load the
c	stack up!

c	Written: 10/30/90 G.H.S.

c	Modified:

c	16/7/96 GHS V:1:0:2:1:1:1:1.0
c		Covert to VERITAS from argintine searchlite code.

c	21/8/96 GHS V:1:0:2:1:1:1:1.1
c		Configure for 9 telescope array, 3 in centre.

c	06/10/97 GHS V:1:0:2:1:1:1:1.2
c		Configure for 7 telescope array. Change name from Athena
c		to VERITAS.

c	10/4/98 GHS V:1:1:5:1:1:1:1.0
c		Convert for use with triangular grid spacins for nx,ny
c		See note at beginning of program

!	24/6/98 GHS V:1:1:5:1:1:1:1.1 
!		Convert to North-South array.

!	07/7/98 GHS  1:1:5:1:1:1:1.3
!		Fix the even/odd problem with veritasloc by relplacing it
!		with two arrays: VERITASLOC_EVEN,VERITASLOC_ODD
!		See the note at beginning of program about evem/odd columns

	include 'kas_array_trigger.inc'
	record/m/m
	integer i,j
	integer nt,ntold/-10000000/
	real dn,dni
	integer  pt
	character*12 ptype(9)           !PMT/Grid type
	character*16 nspchr
	integer nchar

	data ptype/' Visable    ',' Old Luny   ',' Solar Blind',
	1 ' CEASAR     ',' ASGAT      ',' Part-on-gnd','Whipple     ',
	1 ' VERITAS    ','VERITAS-TRI '/

c*********************************************************************
c	The input and ouput ntupple files are open in their respective
c	read and write routines.
c*********************************************************************
c*********************************************************************
c	Read the mfile header(mainly to get xseg and yseg)
c*********************************************************************
c	Open the 'M' file which has been assigned to unit 8.
        OPEN(8,ACCESS='sequential',STATUS='old',FORM='UNFORMATTED')

c	Structure and size from KASSTRUCTURES.INC
c	Read in the mfile header.
	read(8)(mhead.recbuf(i),i=1,mhead_size)
	close(8)
!debug
!        OPEN(27,ACCESS='sequential',STATUS='NEW',FORM='UNFORMATTED')
!	write(27)(mhead.recbuf(i),i=1,mhead_size)
!	close(27)
!enddebug

	dni=sqrt(1.-mhead.pe_head.segment_head.dli**2-   
	1 mhead.pe_head.segment_head.dmi**2)
		!Mount is always looking up.
	dn=-sqrt(1-mhead.pe_head.dl**2-mhead.pe_head.dm**2)
	area=mhead.pe_head.xseg*mhead.pe_head.yseg

	if(index(mhead.pe_head.petype,'VI').ne.0)then
		pt=1
	elseif(index(mhead.pe_head.petype,'OL').ne.0)then
		pt=2
	elseif(index(mhead.pe_head.petype,'SB').ne.0)then
		pt=3
	elseif(index(mhead.pe_head.petype,'C').ne.0)then
		pt=4
	elseif(index(mhead.pe_head.petype,'AS').ne.0)then
		pt=5
	elseif(index(mhead.pe_head.petype,'P').ne.0)then
		pt=6
	elseif(index(mhead.pe_head.petype,'W').ne.0)then
		pt=7
	elseif(index(mhead.pe_head.petype,'VA').ne.0)then
		pt=8
	elseif(index(mhead.pe_head.petype,'VT').ne.0)then
		pt=9
	else
		write(6,1000)mhead.pe_head.petype
1000	format(' KASAOMEGA_FATAL--Illegal photon type:petype=',a)
		stop
	endif

	write(6,1001)ptype(pt),						
	1 namtyp(mhead.pe_head.segment_head.itype),                    
	1 mhead.pe_head.segment_head.tep,                              
	1 mhead.pe_head.segment_head.dli,                              
	1 mhead.pe_head.segment_head.dmi,dni,                          
	1 mhead.pe_head.segment_head.depth,                            
	1 mhead.pe_head.dl,mhead.pe_head.dm,dn,                        
	1 mhead.pe_head.hobs,                                          
	1 mhead.pe_head.segment_head.magnet_field,                     
	1 mhead.pe_head.segment_head.etr,                              
	1 mhead.pe_head.xseg,mhead.pe_head.yseg,                      
	1 mhead.pe_head.segment_head.idfile,                           
	1 mhead.pe_head.x_offset,mhead.pe_head.y_offset,              
	1 mhead.pe_head.segment_head.version,                          
	1 mhead.pe_head.version,mhead.version,version
1001	format( ' Shower paramters:',/,         			
	1 '                Detector type: ',a,/,                        
	1 '                 Primary type: ',a8,/,                       
	1 '               Primary Energy: ',f10.5,' TeV',/,
	1 '        Primary  DLI DMI DNI : ', 3e14.7,/,                  
	1 '              Injection depth: ',f10.2,/,                    
	1 '               Mount dl,dm,dn: '3e14.7,/,                    
	1 '            Detector altitude: ',f10.1,/,                    
	1 ' Magnetic Field specification: ',a,/,                        
	1 '                Thershold MeV: ' f8.2,/,                     
	1 '                    xseg,yseg: ',f8.2,' x',f8.2,/,           
	1 '                       IDFILE: ',i5,/,                       
	1 '   Shower Offset Impact Point: ','x=',f6.2,' y=',f6.2,/,     
	1 '         From KASCADE Version: ',a,/,                        
	1 '         From KASLITE Version: ',a,/,                        
	1 '      From KASTRIGGER Version: ',a,/,                        
	1 '      This KASAOMEGA  Version: ',a)

       WRITE(6,1002)mhead.pesum,mhead.hmult,mhead.noise,
	1  mhead.rate_noise, 
	1  mhead.reflectivity,mhead.concentration,             
   	1  (mhead.options), 
   	1  mhead.step_size,mhead.phi_steps,mhead.itheta_max,    
   	1  mhead.disc_width,mhead.adc_gate
1002	format( ' Original Trigger parameters:',/,    
	1  ' Minimum pe threshold in a pixel(includes noise)  = ',f10.4,/, 
 	1  ' Minimum Trigger Multiplicity (541 inner,mid:+1)  = ',f10.4,/, 
 	1  ' MHEAD.NOISE:Enable generation of noise in pixels = ',l5,/,    
 	1  '      MHEAD.RATE_NOISE:Noise rate: pes/ns/deg**2  = ',f10.4,/, 
 	1  '            MHEAD.REFLECTIVITY:Global efficiency  = ',f10.4,/, 
 	1  '   MHEAD.CONCENTRATION:Efficiency of light cones  = ',f10.4,/, 
 	1  ' KASTRIGGER OPTIONS  = ',a,/,            
 	1  '            Whipple 10m:MHEAD.STEP_SIZE for HADRONS = ',f10.2,
	1 /, 
 	1  '                        MHEAD.PHI_STEPS for HADRONS = ',i10,/,   
 	1  '                       MHEAD.ITHETA_MAX for HADRONS = ',i10,/,   
 	1  '          Effective Discriminater window width (ns) = ',f10.2,
	1 /,
	1  '                       Original ADC gate width (ns) = ',f10.4)



	print*,' Input Ntuple is #:',nth
	print*,' Output Ntuple is #:',ntv

c	Positions of counters on ground. First starts at origen.
c	(x,y,z in same units as xseg,yseg(meters,miles,whatever!)

!Since array order divides again by xseg or yseg we just need to get the
!integer valuse straight.  
!Notice that we have set xseg and yseg to 1.0 here as we must also then do
!for the call to arrayorder!

			!NSPACE is Chosen multiple of xseg for telescope 
			!spacing.
			!Must be even!
			!Determine diagonal coords.
	isymst=lib$get_symbol('nspacing',nspchr,nchar)
	if(.not.isymst)stop ' nspacing is not defined'
	read(nspchr,1200)nspace
1200	format(i5)

	print*,' Telescope spacing excactly:',nspace*mhead.pe_head.xseg

	nsx=nspace/2	!always so.

!*********************************************************************
!	EVEN COLUMN
!*********************************************************************
!First do for Center on even colum:	See note at beginiing of program.
	if(mod(nsx,2).eq.0)then
		nsy=(nsx/2)*3
		nsym=-nsy
	else
		nsy=1+((nsx-1)/2)*3
		nsym=-nsy-1
	endif
!It may take awhile to figure why this definition of the spacing gives a
!perfect hexagaon arraingment but it does. Trust me.

!This version is for the VERITAS sub arrays.
	print*,' VERITAS Sub Array ',options,' for ',v_size,' Telescopes'
	center_index=(v_size/2)+1      	!center index (in a common block)
	print*,' Central telescope index is ',center_index



	j=0
c:1:EAST
	if(index(options,'1').ne.0)then
		j=j+1                 
		veritas_array_even(1,j)=nspace
		veritas_array_even(2,j)=0
		veritas_array_even(3,j)=0
	endif
c:2:SOUTH EAST
	if(index(options,'2').ne.0)then
		j=j+1
		veritas_array_even(1,j)=nsx
		veritas_array_even(2,j)=nsym
		veritas_array_even(3,j)=0
	endif
c:3:NORTH EAST
	if(index(options,'3').ne.0)then
		j=j+1
		veritas_array_even(1,j)=nsx
		veritas_array_even(2,j)=nsy
		veritas_array_even(3,j)=0
	endif
c:4:center,center
	if(index(options,'4').ne.0)then
		j=j+1
		veritas_array_even(1,j)=0  !x
		veritas_array_even(2,j)=0  !y
		veritas_array_even(3,j)=0  !z
	endif
c:5: SOUTH WEST
	if(index(options,'5').ne.0)then
		j=j+1
		veritas_array_even(1,j)=-nsx
		veritas_array_even(2,j)=nsym
		veritas_array_even(3,j)=0
	endif
c:6:NORTH WEST
	if(index(options,'6').ne.0)then
		j=j+1
		veritas_array_even(1,j)=-nsx
		veritas_array_even(2,j)=nsy
		veritas_array_even(3,j)=0
	endif
c:7: WEST
	if(index(options,'7').ne.0)then
		j=j+1
		veritas_array_even(1,j)=-nspace
		veritas_array_even(2,j)=0
		veritas_array_even(3,j)=0
	endif
	if(j.ne.v_size)then
		write(6,1003)v_size,j
1003	format(' v_size(kasstructures.inc):',i3,
	1 ' and sub array size:',i3,' dont match')
		stop
	endif
!                         
c	order these:by nx(descending) and within that ny(descending)
!Notice in this call to arrayorder we have set xseg and yseg to 1.0
!Note this is for the central coulmum being even.
 	do i=1,v_size
		call arrayorder(veritas_array_even(1,1),v_size,
	1 veritasloc_even(1,1,i),1.0,1.0,i)
	enddo

!*********************************************************************
!	ODD COLUMN
!*********************************************************************
!Do for Center on odd colum:	See note at beginiing of program.
	nsx=nspace/2	!always so.
	if(mod(nsx,2).eq.0)then		!This requires no change
		nsy=(nsx/2)*3
		nsym=-nsy
	else
		nsym=-(1+((nsx-1)/2)*3)	!This is where the diference shows up.
		nsy=-nsym+1		!for even vs odd.
	endif
!It may take awhile to figure why this definition of the spacing gives a
!perfect hexagaon arraingment but it does. Trust me.

	j=0
c:1:EAST
	if(index(options,'1').ne.0)then
		j=j+1                 
		veritas_array_odd(1,j)=nspace
		veritas_array_odd(2,j)=0
		veritas_array_odd(3,j)=0
	endif
c:2:SOUTH EAST
	if(index(options,'2').ne.0)then
		j=j+1
		veritas_array_odd(1,j)=nsx
		veritas_array_odd(2,j)=nsym
		veritas_array_odd(3,j)=0
	endif
c:3:NORTH EAST
	if(index(options,'3').ne.0)then
		j=j+1
		veritas_array_odd(1,j)=nsx
		veritas_array_odd(2,j)=nsy
		veritas_array_odd(3,j)=0
	endif
c:4:center,center
	if(index(options,'4').ne.0)then
		j=j+1
		veritas_array_odd(1,j)=0  !x
		veritas_array_odd(2,j)=0  !y
		veritas_array_odd(3,j)=0  !z
	endif
c:5: SOUTH WEST
	if(index(options,'5').ne.0)then
		j=j+1
		veritas_array_odd(1,j)=-nsx
		veritas_array_odd(2,j)=nsym
		veritas_array_odd(3,j)=0
	endif
c:6:NORTH WEST
	if(index(options,'6').ne.0)then
		j=j+1
		veritas_array_odd(1,j)=-nsx
		veritas_array_odd(2,j)=nsy
		veritas_array_odd(3,j)=0
	endif
c:7: WEST
	if(index(options,'7').ne.0)then
		j=j+1
		veritas_array_odd(1,j)=-nspace
		veritas_array_odd(2,j)=0
		veritas_array_odd(3,j)=0
	endif

c	order these:by nx(descending) and within that ny(descending)
!Notice in this call to arrayorder we have set xseg and yseg to 1.0
!Note this is for the central coulmum being even.
 	do i=1,v_size
		call arrayorder(veritas_array_odd(1,1),v_size,
	1 veritasloc_odd(1,1,i),1.0,1.0,i)
	enddo



!Debug: This will be a generalized debug for use with paw to convince ourselves
!that this program really does find array triggers. Mainly we write out 3 files
!One(unit=40) is the position of the telecopes in the array. The second 
!(unit=41) is  the positions of the single telescopes that are hit for a 
!shower. The last is the positon of the central telescope for an array trigger.
!These printouts are activated by setting DEBUG_TRIGGER=.true.
!Use paw to look at these files. Kumac to do this looks like:
!***********************************************************************
!***********************************************************************
!nt/cr 1 ' ' 2 ' ' ! x y
!nt/read 1 for040.dat
!nt/cr 2 ' ' 6 ' ' ! x y t p nx ny
!nt/read 2 for041.dat
!nt/cr 3 ' ' 9 ' ' ! x y t p nx ny mnx mny i
!nt/read 3 for042.dat
!null -450 -250 -100 100
!igset mscf 1.0
!set mtyp 21
!nt/pl 2.y%x ! ! ! ! s
!set mtyp 24
!igset mscf 2.5
!nt/pl 3.y%x ! ! ! ! s
!***********************************************************************
!***********************************************************************
	if(debug_trigger)then
		do i=1,v_size
			nx=veritasloc_even(1,i,center_index)
			ny=veritasloc_even(2,i,center_index)
						!'NS' Now
			xdeb=nx*mhead.pe_head.xseg
			if(mod(nx,2).eq.0)then		!even
				ydeb=ny*mhead.pe_head.yseg
			else
				ydeb=(ny+.5)*mhead.pe_head.yseg
			endif
			write(40,4340)xdeb,ydeb
4340	format(' ',2(e14.7,','))
		enddo
	endif
					
c	For GRID search:Find max and mins needed to insure our GRID
c	covers our present search.  Since we never have to look more
c	negative in nx then the nx of our current ref:
c		Note:I don't see this right now but assume its true for nxmin.
c		Note: h_ntp  sorted    by increasing values by nx then ny.
c		      nx varies slowest. 


	nxmin=0

c	And find the others.
	nxmax=-1000000
	nymax=-1000000
	nymin=1000000
c	The other limits we can get by searching veritasloc.
c	There may be some overkill in these limits but that won't
c	matter much.

!***********************************************************************
!EVEN/ODD :Get extremes by checking both arrays.
!***********************************************************************

	do i=1,v_size
		do k=1,v_size
	if(veritasloc_even(1,k,i).gt.nxmax)nxmax=veritasloc_even(1,k,i)	
	if(veritasloc_even(2,k,i).gt.nymax)nymax=veritasloc_even(2,k,i)	
	if(veritasloc_even(2,k,i).lt.nymin)nymin=veritasloc_even(2,k,i)	
		enddo
	enddo
	do i=1,v_size
		do k=1,v_size
	if(veritasloc_odd(1,k,i).gt.nxmax)nxmax=veritasloc_odd(1,k,i)	
	if(veritasloc_odd(2,k,i).gt.nymax)nymax=veritasloc_odd(2,k,i)	
	if(veritasloc_odd(2,k,i).lt.nymin)nymin=veritasloc_odd(2,k,i)	
		enddo
	enddo
!***********************************************************************


c	Now we need to load the first event into the stack.

c	Set up for the begining of a shower.
	itop=0			!Init the stack!
	ibottom=0

c	Open the input ntupple and read the first event
	nt=nth		!hillas h_ntp
	ntold=-1000000

c	Data from h_ntp is entered by ntread_cw_hillas into common block
c	hNTP. Pertinent info is loaded into the m structure. Begining and end
c	of showers is tracked.
	call ntread_cw_hillas(nt,ntold,m)
	if(v_eof)then
		print*,' Veritas_ini:Fatal:Nothing in H_ntp'
		stop
	endif


c	Put it into the stack
	call putmrec(m)		!This incriments itop to 1.


c	Assumption is made that any cuts to be made are done in KASAOMEGA
c	before h_ntp is created.(Or in paw; makeing a 'new' h_ntp from an 'old'
c	one).

	ibottom=1	!Init stack pointer.Necessary whenever we init the
			!stack with its first entry.


c	Enter this record in the GRID and get the rest(if any) of the 
c	m records with the same nx,ny but different itheta,iphi
c	into the stack.
c	determine first what the new reference point of the GRID should
c	be. That is what nx,ny should grid(0,0) represent?
c	See note in subroutine-ARRAY_SCAN for explanation of the GRID scheme.

	ngxref=m.nx	!Never need earlier then current refs nx.
	ngyref=-(max_grid)/2	!at least attempt to start at a good place.
			!If grid dimensions are big enough this may
			!reduce some remapping calls.
c	PLace this current ref area stack index into GRID
	nx=m.nx-ngxref
	ny=m.ny-ngyref
	if(ny.lt.0.or.ny.gt.max_grid)then
		ngyref=m.ny+nymin	!Well we tried.
		ny=m.ny-ngyref
	endif
	grid(nx,ny)=itop

c	Get the rest of the m records for this area into stack.
	call fill_stack(m.nx,m.ny)
					!For protons this adds the rest of the
					!itheta,iphi records for this area to
					!the stack and itop is incrimented
					!accordingly
c	All done. Bye Bye!
	return
	end

	

	subroutine veritas_proc
c	This routine_ runs through the ntupple records looking for triggers.
c	To describe how we look for triggers in the array lets make the 
c	following definitions:
c	1:area record:This a record from the ntupple produced by KASAOMEGA.
c		It has the information about how a particular grid area
c		specified by nx,ny in the record pointing in a specific
c		direction specified by the itheta,iphi in the record will
c		affect each of the V_SIZE mirrors of the VERITAS
c		array if they were to be at this nx,ny,itheta,iphi position 
c		during a shower.  This information is listed in the area 
c		record as a pulse height. Also in the record is the relative
c		timing of the hit!
c	2:relative postion array:  This is an array that indicates the relative
c		nx,ny speration between the different mirrors of the VERITAS
c		array. In fact there are V_SIZE such arrays. Each one 
c		puts a different mirror of the array at relative location 0,0.
c	3:current reference area:  The search for an array trigger is made by
c		trying a particular area record(called the current reference
c		arera) and placing it at a particular postion in the 
c		VERITAS array(as long as its pulse height in this position 
c		indicates it would have fired in this positon in the array and
c		it has not already been tested in this position(see below)).
c		The rest of the ntupple is then searched to see
c		how many of the other VERITAS mirrors would also fire.  This
c		is all done by use of the relative postion arrays. Once the M 
c		file has been searched the total number of VERITAS mirrors 
c		that have fired are added up and that is the multiplicity of 
c		the trigger. The current reference area is tried at different 
c		areas in the array by using the V_SIZE different 
c		relative position arrays. Track
c		is kept of which area record have been tried at which positions
c		(so that we don't double count triggers) by setting their pulse
c		heights (in the stack) to -1 when they are used at a particular
c		mirror position.
c	Note here that the search through the ntupple event records is actually 
c	accomplished by first reading in a number of the events into
c	memory into a stack and it is the records in the stack that are
c	searched.

c	The individual trigger times of the veritas mirrors will have dynamic
c	timing corrections applied for tilted mounts.

c	Written: 1/3/91 G.H.S.V:1:0:1:0:0:0.3

c	Modified:

c	4/12/91 G.H.S. v:1:0:1:0:0:0.3
c		Speed up things by calculating nhits in ARRAY_SCAN.
c		Also set up a new variable MAX_MISS, which ARRAY_SCAN
c		tests on. If more then MAX_MISS areas are not found ARRAY_SCAN
c		quits. This speeds up things by a factor of 3.but see 2/2/92

c	2/1/92 G.H.S.v:1:0:1:0:0:0.6
c		Vast modifications to impliment new array search scheme that
c		doesn't require a mandtory counter and dtermines various
c		multiplicities.

c	2/2/92 G.H.S.v:1:0:1:0:0:0.6
c		For this search scheme we have to complete search for all 
c		v_size	mirrors since we need to see the -1 flag if we find one.
c		This means we can forget max_miss.

c	4/2/92 G.H.S.v:1:0:1:0:0:0.6
c		For implimentation of the GRID search scheme, every time
c		we get a new current ref position we have to check that its 
c		possible required positions are contained within GRID.
c		If not we will need to remap GRID so that they are so contained.

c	2/4/92 G.H.S.v:1:0:1:0:0:0.6
c		Add PE_THRESHOLD to dotdat file entry. And, add 
c		pe_threshold check before calculating dynamic times.

c	12/8/96 GHS V:1:0:2:1:1:1:1.1
c		Add code to determine PARALLAXWIDTH in VERITAS_PROC.

c	The Veritas M-file(so far) has the format:

c          m.nx
c	   m.ny		!Index to mount hit.(i*4)
c	   m.itheta		!Zenith angle index(in 1.0 deg steps)
c	   m.iphi		!Azimuthal index. in steps of 360/nsteps
c				   nsteps=(itheta-1)*5
c	   m.pheight(v_size)    !Needed for array trigger determination.

	include 'kas_array_trigger.inc'
	record /m_v/m_v
	record /m/m
	record /trigger/trigger
	real veritastimes(v_size)
	integer iveritas(v_size)
	logical out_of_range
	integer ntold_out/-1000000/

	real x,y,y_intercept(v_size),vslope(v_size),
	1 y2d_intercept(v_size),v2d_slope(v_size),
	1 xv_core(v_size*v_size),yv_core(v_size*v_size),
	1 xvmean(v_size),yvmean(v_size),vlength(v_size),vangle,
	1 dist(v_size),alph(v_size),sdevxy(v_size),
	1 aj,xj,yj,ak,xk,yk,xv_source(v_size*v_size),
	1 yv_source(v_size*v_size)

	integer ivp_index,iv2d_index
	integer center_required,ifirst
	real open_angle_min_p
	real open_angle_min_2d
	real dist_min_p
	real dist_min_2d
	parameter (open_angle_min_p=20.0)
	parameter (open_angle_min_2d=15.0)
	parameter (dist_min_2d=.1)
	logical center_even




	if(ifirst.ne.1)then
		ifirst=1
	print*,' Number of telescopes required for trigger:',
	1 hmult
	print*,' Parallax Minimum Opening angle(max is 180 - this angle):',
	1 open_angle_min_p
	print*,' 2d Minimum Opening angle(max is 180 - this angle):',
	1 open_angle_min_2d
	print*,' 2d Minimum dist:',
	1 dist_min_2d
	endif
c	We start with the records for 1 area in the stack already. Let's not 
c	worry about filling up the stack. Let it happen naturally.  We will,
c	however, have to keep track of where we are in the stack.
c	At this point itop=N where n points to the last itheta,iphi
c	record for this area. ibottom=1.


	itrigtop=ibottom	!Save the current ref area position in the stack
	veritas_tep=h_ntp.tep
	veritas_id=h_ntp.id
c	this is the start of the trigger scan loop.
c	get the current reference area.
100	call getmrec(m_v,itrigtop,out_of_range)
	if(out_of_range)then	!This probably means weve finished the shower.
				!New shower or end of file.
		if(v_beg_of_shower)then
				!Load first aread into M.				
!Notice: This is correct even for triangular array.('NS' or 'EW')
! Gives correct nx,ny. by rounding down. Ie if hntp.y/yseg=1.5 then
!	m.ny=1 as it should! GHS 10/4/98
!To insure round down for negative values, add then subtract 100000 to keep
!things positive during the intergeration.(different compiles interagize
!diferently.
						!Offset by 100000
			xp=(h_ntp.x/mhead.pe_head.xseg+100000.) 
			yp=(h_ntp.y/mhead.pe_head.yseg+100000.)
						!convert to integer(round down)
			m.nx=xp
			m.ny=yp
						!Remove offset
			m.nx=m.nx-100000
			m.ny=m.ny-100000

			m.itheta=h_ntp.theta
			m.iphi=h_ntp.phi
			veritas_tep=h_ntp.tep
			veritas_id=h_ntp.id
			do i=1,v_size
				m.pheight(i)=1.	!Set all pulse heights to 1
			enddo		!This array used to flag which area
					!has been used in which position in
					!the array. Checked agains pe_threshold
					!which we we set to 1 for now.
				!Add in image parameters. Same order as in
				!NTREAD_CW_HILLAS
			m.pheight(v_size+1)=h_ntp.xmean
			m.pheight(v_size+2)=h_ntp.ymean
			m.pheight(v_size+3)=h_ntp.dist
			m.pheight(v_size+4)=h_ntp.alph
			m.pheight(v_size+5)=h_ntp.sdevxy
			m.pheight(v_size+6)=h_ntp.size
			m.pheight(v_size+7)=h_ntp.xmpln
			m.pheight(v_size+8)=h_ntp.ympln
			m.pheight(v_size+9)=h_ntp.emaltmpln
			m.pheight(v_size+10)=h_ntp.x_offset
			m.pheight(v_size+11)=h_ntp.y_offset

c	Now we need to load the first event into the stack.
c	Set up for the begining of a shower.
			itop=0			!Init the stack!
			ibottom=0
c	Put it into the stack
			call putmrec(m)		!This incriments itop to 1.
			ibottom=1	!Init stack pointer.
					!Necessary whenever we init the
					!stack with its first entry.
						!Setup and fill grid and rest
						!of stack.
			ngxref=m.nx		!Never need earlier then 
						!current refs nx.
			ngyref=-(max_grid)/2	!at least attempt to start at 
						!a good place.
c	Zero the grid
			do i=0,max_grid
				do j=0,max_grid
					grid(i,j)=0
				enddo
			enddo

c	PLace this current ref area stack index into GRID
			nx=m.nx-ngxref
			ny=m.ny-ngyref
			if(ny.lt.0.or.ny.gt.max_grid)then
				ngyref=m.ny+nymin	!Well we tried.
				ny=m.ny-ngyref
			endif
			grid(nx,ny)=itop
c	Get the rest of the m records for this area into stack.
			call fill_stack(m.nx,m.ny)
					!For protons this adds the rest of the
					!itheta,iphi records for this area to
					!the stack and itop is incrimented
					!accordingly
			itrigtop=ibottom	!Save the current ref area 
						!position in the stack
					!load first record into m_v
			call getmrec(m_v,itrigtop,out_of_range)
		elseif(v_eof)then
			ntold=ntv
			call hrout(ntold,icycle,' ')
			call hldir(' ',' ')
			call hrend('VTUPLE')
			return
		else
		Print*,' kas_array_trigger:Fatal: Unexpected termination'
			stop
		endif
	endif

c	we are now ready to see if with this area record is part of array
c	triggers at various places in the array.

c	Check to see that the stack extends far enough to cover the possible
c	trigger configurations.
				!Position V_SIZE in the VERITAS array is the
				!one with the most negative nx,ny and
				!would thus give biggest forward reach
				!needed for the search.
!*************************************************************************
!EVEN/ODD
!*************************************************************************
	ivxmax=veritasloc_even(1,1,v_size)
	if(ivxmax.lt.veritasloc_odd(1,1,v_size))then
		ivxmax=veritasloc_odd(1,1,v_size)
	endif
	ivymax=veritasloc_even(2,1,v_size)
	if(ivymax.lt.veritasloc_odd(2,1,v_size))then
		ivymax=veritasloc_odd(2,1,v_size)
	endif
!*************************************************************************

	nxrequired=m_v.nx+ivxmax
	nyrequired=m_v.ny+ivymax

c	Fill out stack(if were not at the end of a shower)
	if((.not.v_beg_of_shower).and.(.not.v_eof))then
		call fill_stack(nxrequired,nyrequired)
	endif
c	Now trim the bottom of the stack.
c	to cover the array(not just the trigger).
!Since implimentation of our new scheme, we never look back before the current
!reference area in the stack. Any area before the current one has already been
!tried in all 7 array positions and this would be reflected by -1 in the
!current ref area's peheight array. So we can trim right up to the present
!current reference area.

	call trim_stack(m_v.nx,m_v.ny)   !array will give us its backward
		         		   !reach.

c	Check that GRID encompasses the reqired limits.  We know that the lower
c	nx is covered by definition but test for the other 3 directions.
	nyg=m_v.ny+nymin-ngyref	!Min ny
	if(nyg.ge.0)then
		nyg=m_v.ny+nymax-ngyref	!Max ny
		if(nyg.le.max_grid)then
					!Max nx index in Grid we will need
			nxg=m_v.nx+nxmax-ngxref
			if(nxg.le.max_grid)then
				goto 102	!Array is contined, go search.
			endif
		endif
	endif

!Get here if we need to remap GRID
	call remap_grid(m_v.nx,m_v.ny)

c	Collect the hits for the reference arera at different mirror positions.
c	Note that KASATRIG needs to order its mirror positions the same as
c	VERITAS_INI.

102	do i=1,v_size	                                                    
			!Check that the current reference area has not been 
			!already been checked in this position.
		if(m_v.pheight(i).eq.-1)then                                
			goto 101		!Try next position.
		endif                                                       
			!Look for the trigger. The -1 flags are set in
			!array_scan.

!***********************************************************************
! EVEN/ODD
!***********************************************************************
!	Figure out if we are have an even or an odd column at center.
!	This only good for NS!!!!
					!veritasloc_even or odd would give 
					!same result so just use even
		icenterx=m_v.nx-veritasloc_even(1,center_index,i)
		if(mod(icenterx,2).eq.0)then
			center_even=.true.
	 		call array_scan(m_v.nx,m_v.ny,m_v.itheta,m_v.iphi,
	1 veritasloc_even(1,1,i),v_size,trigger,itop,nhits)
		else
			center_even=.false.
	 		call array_scan(m_v.nx,m_v.ny,m_v.itheta,m_v.iphi,
	1 veritasloc_odd(1,1,i),v_size,trigger,itop,nhits)
		endif
!***********************************************************************


!See if we have a trigger.
		if(nhits.ge.hmult)then                                      

!***************************************************************************
!	Fill V_NTP record:
!***************************************************************************


!convert to triangular array stuff here.
!***********************************************************************
! EVEN/ODD
!***********************************************************************
		   if(center_even)then
		   	nxc=m_v.nx-veritasloc_even(1,i,center_index)
			nyc=m_v.ny-veritasloc_even(2,i,center_index)
		   else
		   	nxc=m_v.nx-veritasloc_odd(1,i,center_index)
			nyc=m_v.ny-veritasloc_odd(2,i,center_index)
		   endif
!***********************************************************************
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
!  FORCE NS
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
!	       	    	if(index(mhead.pe_head.petype,'NS').eq.0)then
!						!'EW' First
!				v_ntp.y=nyc*mhead.pe_head.yseg
!				if(mod(nyc,2).eq.0)then		!even
!					v_ntp.x=nxc*mhead.pe_head.xseg
!				else
!					v_ntp.x=
!	1 (nxc+.5)*mhead.pe_head.xseg
!				endif
!			else
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
				v_ntp.x=nxc*mhead.pe_head.xseg
				if(mod(nxc,2).eq.0)then		!even
					v_ntp.y=nyc*mhead.pe_head.yseg
				else
					v_ntp.y=
	1 (nyc+.5)*mhead.pe_head.yseg
				endif
!XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

			v_ntp.theta=m_v.itheta
			v_ntp.phi=m_v.iphi
			v_ntp.x_offset=m_v.x_offset
			v_ntp.y_offset=m_v.y_offset
			v_ntp.tep=veritas_tep
			v_ntp.file_id=veritas_id
			v_ntp.hits_center=nhits
	if(debug_trigger)then
		write(42,4342)v_ntp.x,v_ntp.y,m_v.itheta,m_v.iphi,nxc,nyc,
	1 m_v.ny,m_v.nx,i,icenterx
4342	format(' ',2(e14.7,','),8(i5,','))
	endif
!**************************************************************************
!	Determine paralaxx width and 2d pointing.
!**************************************************************************

!**************************************************************************
!				PARALLAX
!**************************************************************************
c	This method uses the direction of the center-of-mass of the image
c	in a telescope to indicate the direction the shower core is from that
c	telescope. We thus can determine for each telescope a line on the
c	ground that both the telescope and the shower core lay on. Our method
c	is to look at the intersection of each line from each triggering 
c	telescope with	the lines from all the other telescopes. It is expected
c	due to the quality difference of Gamma images and Hadronic images that
c	the distribution of intersection poins for gammas will be much more
c	compact than that of hadronic showers. We quantify this compactness
c	with a PARALLAXWIDTH parameter which is the root-mean-square width of
c	distance of all intersection points from the mean intersection point of
c	an array trigger.

c	First part of PARALLAX calculation is determination of mean point
c	of intersection of all triggers. Use XMEAN and YMEAN of image to 
c	give direction to the core of the shower from the center of the
c	mount. Use a line that goes through the center of the mount(on the
c	ground) in the direction indicated by the image.
c	CAUTION:
c		1: Assume mounts are all verticle(only sort of true for
c		   hadronic. Improve later!)
c		2: I think the X and Y in the focal plane is -X and -Y in the
c		   ground, but since we only use ratios its ok.
c	First make sure we have image information. DIST ne -1
c		DIST parameter is used as flag to indicate if the cleanup
c		left us enough pixels to determine image parameters.
c		DIST follows XMEAN and YMEAN after PHEIGHT
c	Determine line parameters for this trigger.
!**************************************************************************


!**************************************************************************
!				2 D
!**************************************************************************
!	The idea here is to use the fact that the axis of the image is
!	determined by 2 points. the first is the direction to the place on the
!	ground where the shower core hits. This is waht the parallaxwidth 
!	method ustilaizes. The other is the direction to the plac ein the sky
!	where the shower originated i.e. the source position. IN the
!	parallaxwidth mewthod we assumed that the origen was the source postion
!	This was necessary when we wanted to use the minimum light content 
!	showers. Showers with light content so low they couldn't have HIllas 
!	parameters determined for them. All that was needed was a centroid of 
!	the imafe(xmean,ymean). For this 2D analysis however we will need a 
!	bright enough image to determine hillas parameters since we use the 
!	actual axis of the elipse to point to the source. Again as in the 
!	PARALLAX method we use the intersections of the axes but in the imagae
!	plane, not on the ground. The resultant rms mean of the coordinates of
!	intersection is used as the source direction.  The mean width of this 
!	set of intersection points is also determined.
!
!	For the 2dcalculation we use the axis of the fitted image given
!	by the calls to w_hillas. This means we do need enough 'hit' pixels
!	to fit the elipse. WQe also make the following restrictions:
!		1.) Dist>.5 deg.  A standard cut that gives us a core distance
!				  far enough from the telescope to insure a
!				  good image. Also since our flag for an image
!				  that is to dim to generate Hillas parameters
!				  is to set D=-1 this cut insures only good 
!				  images.
!		2.)angle_2d>open_angle_min
!				  This is the opening angle between any 2 of
!			          the intersecting lines. If this angle is too
!				  small the source poisiton wilkl be very 
!				  poorly determined.
!**************************************************************************



!**************************************************************************
!				Energy resolution.
!**************************************************************************
!	For now we are just testing this technique. We just pass on the 
!	DIST,XMEAN,YMEAN,XMPLN,YMPLN,EMALTMPLN variable for each moount
!	that triggers. 0's otherwise. DIST will be -1 if no image pasted
!	cleanup. In that case XMEAN,YMEAN are coords in image plane of
!	center-of-mass of pixel adcs for pixels that triggered instead.
!**************************************************************************

!	Iterate over telescopes.
			ivp_index=0
			iv2d_index=0
			do j=1,v_size

				!Make sure this scope is triggered.
	if(trigger.m(j).pheight(j).ge.pe_threshold)then

!**************************************************************************
!			Energy resolution INIT
!**************************************************************************
		v_ntp.hit_trig(j)=trigger.m(j).pheight(j)
		v_ntp.xmean(j)=trigger.m(j).pheight(v_size+1)
		v_ntp.ymean(j)=trigger.m(j).pheight(v_size+2)
		v_ntp.dist(j)=trigger.m(j).pheight(v_size+3)
		v_ntp.size(j)=trigger.m(j).pheight(v_size+6)
		v_ntp.xmpln(j)=trigger.m(j).pheight(v_size+7)
		v_ntp.ympln(j)=trigger.m(j).pheight(v_size+8)
		v_ntp.emaltmpln(j)=trigger.m(j).pheight(v_size+9)
!**************************************************************************


!**************************************************************************
!				PARALLAX  INIT
!**************************************************************************
		v_ntp.hit_pat(j)=trigger.m(j).pheight(j)
		ivp_index=ivp_index+1   	
					!Index to line param array
					!Xmean and ymean follow directly
					!after pheights
		xvmean(ivp_index)=trigger.m(j).pheight(v_size+1)
		yvmean(ivp_index)=trigger.m(j).pheight(v_size+2)
					!Calculate our own DIST here since we
					!don't require enough of an image to
					!generate hillas parameters.
		vlength(ivp_index)=sqrt(xvmean(ivp_index)**2+
	1 yvmean(ivp_index)**2)
		if( (xvmean(ivp_index).eq.0) .and.
	1 (yvmean(ivp_index).eq.0) ) then   		!Reject if no slope.
			ivp_index=ivp_index-1
			v_ntp.hit_pat(j)=0
		else
			if(xvmean(ivp_index).ne.0)then
				vslope(ivp_index)=
	1 yvmean(ivp_index)/xvmean(ivp_index)
			else
				vslope(ivp_index)=1.e10*
	1 yvmean(ivp_index)/abs(yvmean(ivp_index))
			endif

				!Place array with center telescope at origen
			x=x_mplane(m_v.itheta,m_v.iphi,j)
			y=y_mplane(m_v.itheta,m_v.iphi,j)
			y_intercept(ivp_index)=y-vslope(ivp_index)*x
		endif
!********************* END Parallaxwidth init ********************************

!**************************************************************************
!				2D INIT
!**************************************************************************
		v_ntp.hit_pat_2d(j)=trigger.m(j).pheight(j)
		iv2d_index=iv2d_index+1   	
			!xmean,ymean,dist, alph and sdevxy follow after pheights
		x=trigger.m(j).pheight(v_size+1)
		y=trigger.m(j).pheight(v_size+2)
		dist(iv2d_index)=trigger.m(j).pheight(v_size+3)
		alph(iv2d_index)=trigger.m(j).pheight(v_size+4)
		sdevxy(iv2d_index)=trigger.m(j).pheight(v_size+5)
				  !Reject if no parameters or too close
		if(dist(iv2d_index).lt.dist_min_2d)then
			iv2d_index=iv2d_index-1
			v_ntp.hit_pat_2d(j)=0  !
		elseif(sdevxy(iv2d_index).eq.0.and.
	1 alph(iv2d_index).eq.0)then
			iv2d_index=iv2d_index-1
			v_ntp.hit_pat_2d(j)=0  !
		else
	 		!Determine slope the axis of the image in this telescope
			!Make sure we have a  non-infinite slope
			if(sdevxy(iv2d_index).ne.0)then
				v2d_slope(iv2d_index)=
	1 alph(iv2d_index)/sdevxy(iv2d_index)
			else
				v2d_slope(iv2d_index)=1.e10*
	1 alph(iv2d_index)/abs(alph(iv2d_index))	!Get sign
  			endif

				!determine y intercept of this line. It must
				!go through centroid of image(xmean,ymean)
  	  		y2d_intercept(iv2d_index)=y-v2d_slope(iv2d_index)*x
		endif
!********************* END 2D INIT ********************************************

	else
		v_ntp.hit_pat(j)=0
		v_ntp.hit_pat_2d(j)=0
		v_ntp.hit_trig(j)=0
		v_ntp.xmean(j)=0
		v_ntp.ymean(j)=0
		v_ntp.dist(j)=-1	!This may help in paw cuts.
		v_ntp.size(j)=0
		v_ntp.xmpln(j)=0
		v_ntp.ympln(j)=0
		v_ntp.emaltmpln(j)=0
	endif  			!Trigger this area test.
			enddo


!**************************************************************************
!			PARALLAX  Process
!**************************************************************************
c	Now determine all intercepts and collect xcore,ycore mean stuff.
			v_ntp.iv_core=0
			v_ntp.xcore=0
			v_ntp.ycore=0
	if(ivp_index.gt.1)then
		do j=1,ivp_index-1   !Don't look for last to
					 	    !intersect itself.
			do k=j+1,ivp_index
c	Check angle between lines. ONly use lines that are at least
c	open_angle_min(?)
c	apart.
				vangle=
	1 ((xvmean(j)*xvmean(k)+yvmean(j)*yvmean(k))/
	1 (vlength(j)*vlength(k)))

				if(abs(vangle).gt.1)then
					if(abs(vangle).gt.1.001)then
		print*,' vangle,j,xvmean(j),yvmean(j),vlength(j):',
	1 vangle,j,xvmean(j),yvmean(j),vlength(j)
		print*,' k,xvmean(k),yvmean(k),vlength(k):',
	1 vangle,k,xvmean(k),yvmean(k),vlength(k)
					endif
					vangle=1.0
				endif

				vangle=acosd(vangle)
				if(abs(vangle).gt.open_angle_min_p.and.
	1 abs(vangle).lt.(180-open_angle_min_p))then
						 	!Count intersections.
	v_ntp.iv_core=v_ntp.iv_core+1
	xv_core(v_ntp.iv_core)=(y_intercept(j)-y_intercept(k))/
	1 (vslope(k)-vslope(j))
	yv_core(v_ntp.iv_core)=vslope(k)*xv_core(v_ntp.iv_core)+
	1 y_intercept(k)
						 !Colect for mean calc.
	v_ntp.xcore=v_ntp.xcore+xv_core(v_ntp.iv_core)
	v_ntp.ycore=v_ntp.ycore+yv_core(v_ntp.iv_core)
				endif
			enddo
		enddo
c	Calculate mean core position and its width.
		if(v_ntp.iv_core.ge.2)then
			v_ntp.xcore=v_ntp.xcore/v_ntp.iv_core
			v_ntp.ycore=v_ntp.ycore/v_ntp.iv_core
			v_ntp.parallaxwidth=0
			do j=1,v_ntp.iv_core
				v_ntp.parallaxwidth=v_ntp.parallaxwidth	+
	1 (xv_core(j)-v_ntp.xcore)**2+(yv_core(j)-v_ntp.ycore)**2
			enddo
			v_ntp.parallaxwidth=
	1 sqrt(v_ntp.parallaxwidth/v_ntp.iv_core)

	  	else
			v_ntp.parallaxwidth=-1000.0
		endif 	!iv_core>2 test
	else
		v_ntp.parallaxwidth=-1000.
	endif           !ivp_index>1 test

!********************* END Parallax Process ********************************************

!**************************************************************************
!			2D  Process
!**************************************************************************
c Now determine all intercepts and collect xsource,ysource and  mean stuff.
			v_ntp.iv_source=0
			v_ntp.xsource=0
			v_ntp.ysource=0
					!We need 2 images to get a source
					!position.
	if(iv2d_index.gt.1)then
		do j=1,iv2d_index-1   !Don't look for last to
					 	    !intersect itself.
			do k=j+1,iv2d_index
c	Check angle between lines. ONly use lines that are at least
c	open_angle_min_2d(in deg). Use slope to get unit vectors for both
c	lines and then compute dot product.
						!Cos theta to x axis is
						!sqrt(1/a**2+1)
				aj=v2d_slope(j)
				xj=sqrt(1/(aj**2+1))
				yj=sqrt(aj**2/(aj**2+1))

				ak=v2d_slope(k)
				xk=sqrt(1/(ak**2+1))
				yk=sqrt(ak**2/(ak**2+1))

				vangle=abs(xj*xk+yj*yk)
               			if(abs(vangle).gt.1)then
					if(abs(vangle).gt.1.001)then
		print*,' vangle,j,xj,yj,dist(j):',
	1 vangle,j,xj,yj,dist(j)
		print*,' k,xk,yk,dist(k):',
	1 vangle,k,xk,yk,dist(k)
					endif
					vangle=1.0
				endif

				vangle=acosd(vangle)
				if(abs(vangle).gt.open_angle_min_2d.and.
	1 abs(vangle).lt.(180-open_angle_min_2d))then
				 	!Count intersections.
	v_ntp.iv_source=v_ntp.iv_source+1
	xv_source(v_ntp.iv_source)=(y2d_intercept(j)-y2d_intercept(k))/
	1 (v2d_slope(k)-v2d_slope(j))
	yv_source(v_ntp.iv_source)=v2d_slope(k)*xv_source(v_ntp.iv_source)+
	1 y2d_intercept(k)
						 !Colect for mean calc.
	v_ntp.xsource=v_ntp.xsource+xv_source(v_ntp.iv_source)
	v_ntp.ysource=v_ntp.ysource+yv_source(v_ntp.iv_source)
				endif
			enddo
		enddo
c	Calculate mean source position and its width.
		if(v_ntp.iv_source.ge.2)then
			v_ntp.xsource=v_ntp.xsource/v_ntp.iv_source
			v_ntp.ysource=v_ntp.ysource/v_ntp.iv_source
			v_ntp.sourcewidth=0
			do j=1,v_ntp.iv_source
				v_ntp.sourcewidth=v_ntp.sourcewidth	+
	1 (xv_source(j)-v_ntp.xsource)**2+(yv_source(j)-v_ntp.ysource)**2
			enddo
			v_ntp.sourcewidth=
	1 sqrt(v_ntp.sourcewidth/v_ntp.iv_source)

	  	else
			v_ntp.sourcewidth=-1000.0
		endif 	!iv_source>2 test
	else
		v_ntp.sourcewidth=-1000.	!No source determined flag.
	endif           !iv2d_index>1 test

!********************* END 2D  Process ********************************************


			!Write out v ntupple
			nt=ntv
			call ntwrite_cw_veritas(nt,ntold_out)
		endif                                     



!*************************************************
!For now forget about timing across array. Add later if we want it.
!**************************************************
!		!Form dynamic times.
!			do j=1,v_size
!		!Check pulseheight above threshold to see if this area found.
!		! I really do need m_v(J).pheight(J) here since the pulse
!		! height is set to -1 in the Jth position just after its
!		!laoded into m_v(J).pheight(j). This is confussing I know
!		!but its correct(I think).
!
!		call dynamic_tilt(m_v.itheta,m_v.iphi,
!	1 veritas_array(1,j),delta_t)
!					!In structure trigger stime folows
!					!pulse heights.
!!			veritastimes(j)=
!!	1 trigger.m(j).pheight(v_size+1)+delta_t
!!			iveritas(j)=trigger.m(j).pheight(j)	!Pheight
!!		else
!!			veritastimes(j)=0.0
!!			iveritas(j)=0
!		endif
!			enddo
!See if it all fits in the window.
!			call slide(veritastimes,iveritas,swindow,
!	1 v_size,ismult)
!			if(ismult.ge.hmult)then
!
!!We have an array trigger
!!Not YET!c	Now write this trigger out to the 'T' file.
!!	This is where we will write out the v_ntp ntupple record.
!			endif
!		endif	

101	enddo
430	itrigtop=itrigtop+1	!and search for another
	goto 100
	end


