MODULE KASSUM_HDF5_SUBS

	use structures
        type(m_head),pointer :: mhead
        integer :: mfile_in, mfile_out
        integer :: isize,npixels,isize_old
        character(len=80) ::input_mfile_name
        character(len=80) ::output_cmsum_file_name
        character(len=80) ::append_new_string
CONTAINS

    SUBROUTINE FILEOPEN
!******************************************************************************
!	Open the M file and read in its header.
!	Open the output summary file.
!******************************************************************************

!	Modified:
!               
!       08/05/03 GHS
!               Convert to using command line arguments for input. 


	IMPLICIT NONE
	
	integer :: ios,nchar,isymst,ierr,error

        allocate(mhead,stat=error)
	if(error/= 0)then
		print*,"Program could not allocte space for mhead."
	    	stop
	endif

!Open HDF5 input Mfile.

          call mhdf5_open(trim(input_mfile_name)//char(0),mfile_in,ierr)
          if(ierr<0)then
             print*,' MHDF5_open failure for file:',trim(input_mfile_name)
             stop  ' MHDF5 input File open failure'
          endif


          call mhdf5_make_types

          call mhdf5_mhead_read(mhead,mfile_in)

          if(index(mhead%options,'W490')/=0)then
             npixels=490
          elseif(index(mhead%options,'V499')/=0)then
             npixels=499
          else
             print*,' Unable to decode number of pixels in camera'
             stop 'Failure to decode number of pixels in camera'
          endif

!	open the output summary file.
!	First check to see if this is to be a new file.
	if(index(append_new_string,'APPEND')/=0)then
           call mhdf5_open(trim(output_cmsum_file_name)//char(0),mfile_out,&
                & ierr)
           if(ierr<0)then
             print*,' MHDF5_open failure for file mfile_out.hdf'
             stop  'File open failure'
          endif
          call mhdf5_mrec_size(mfile_out,isize) !Number of mrec events in file
       elseif(index(append_new_string,'NEW')/=0)then
                                        !Create the output HDF file and write 
                                        !out the header summary file.
          print*,' Creating new Ouput file:',trim(output_cmsum_file_name)
          call mhdf5_create(trim(output_cmsum_file_name)//char(0),mfile_out)
                              !Write out the mhead record into its own dataset.
          call mhdf5_mhead_write(mhead,mfile_out)
          isize=0             !Number of mrec events in file
       else
          stop 'append_new_string not defined'
       endif
       isize_old=isize
       return
    END SUBROUTINE FILEOPEN
!******************************************************************************

    SUBROUTINE PROCESS
!******************************************************************************
!	This routine_ procesess the compressed events for a 
!	detector.  All events are appended to the output file
!******************************************************************************

!	Modified:

	IMPLICIT NONE

        real,dimension(:),pointer :: disc,time
        real,dimension(490) :: disc_new
        real, dimension(60000) :: time_new
        real :: tep,x_offset,y_offset
        integer :: error,i,ierr,idfile,index,j,igev
        type(m_rec),pointer ::  mrec
        type(m_rec) :: mrec_new

        integer :: deb_size

        deallocate(mrec,stat=error)
        allocate(mrec,stat=error)
        if(error/= 0)then
           print*,"Program could not allocte space for mrec."
           stop
        endif
        
        deallocate(disc,stat=error)
        allocate(disc(npixels),stat=error)
        if(error/= 0)then
           print*,"Program could not allocte space for DISC."
           stop
        endif

        deallocate(time,stat=error)
        allocate(time(60000),stat=error)
        if(error/= 0)then
           print*,"Program could not allocte space for time."
           stop
        endif

         call mhdf5_mrec_size(mfile_out,isize) !Number of mrec events in file
          !print*,' isize:',isize

          i=0      !Index to first event in input file.
                 !isize is index to lastevent(+1) in output file.
                 !So effectivly isize is index to position for next new event.
        do
           call mhdf5_in(i,tep,idfile,x_offset,y_offset,mrec,disc,time, &
                & mfile_in,ierr,npixels)
           if(ierr<0)then
             exit                   !end of file found
           endif
           index=isize+i
                     !And write it out. Mrec event # start at 0.
           call mhdf5_out(index,tep,idfile,x_offset,y_offset,mrec,disc,time, &
                & mfile_out,npixels)
          i=i+1                      !Bump to next index.
        enddo

!	End of shower.
        call mhdf5_mrec_size(mfile_out,isize) !Number of mrec events in file
        igev=tep*1000.
        write(6,1000)npixels,isize_old,isize,i,igev,idfile,x_offset,y_offset
1000    format(i4,' pix.Start:',i9,' end:',i9,' net:',i6,' GeV:id:',&
             i5,i5,' x,y:',2f4.1) 
        call mhdf5_close(mfile_in)
        call mhdf5_close(mfile_out)

        call mhdf5_close_types

	return
    END SUBROUTINE PROCESS
END MODULE KASSUM_HDF5_SUBS

PROGRAM KASSUM_HDF5
!******************************************************************************
!	This program reads in the M files for the whipple
!	Camera and appends the records back out to a summary file.
!       All files are in HDF5 format.
!******************************************************************************

!	The basic structure is same for all cameras.
!	This program is run once for each individual M file.
!	Files:
!	"mfile_in.hdf"  =  'M' input file.
!	"mfile_sum.hdf" =  Output summary file.

!	G.H.Sembroski
!	Physics Dept
!	Purdue University
!	W.Lafayette, In 47907
!	E-Mail: sembroski@physics.purdue.edu

!	Modified:

!	25/10/00 GHS
!		Converted from kassum_compress.f90  to HDF5 format

!       08/05/03 GHS
!               Convert to using command line arguments for input. 

	use kassum_hdf5_subs
	IMPLICIT NONE
        integer :: i,num_cmd_args
        character(len=80) :: arg_opt
        integer,external :: iargc_

!******************************************************************************
!       Get the various command line arguments.
!       All command line arguments come in pairs: An option (-p) and 
!                                                 a string (kasoam.par)
!       Defaults are in wcamera_modules.f90
!******************************************************************************
        num_cmd_args=iargc_()
        if(num_cmd_args.gt.0)then
           do i=1,num_cmd_args,2
              call getarg_(i,arg_opt)
              arg_opt=trim(arg_opt)
              if(arg_opt=="-i")then
                 call getarg_(i+1,input_mfile_name)
                 input_mfile_name=trim(input_mfile_name)
                 !print*,' -i option gives input mfile filename:',&
              elseif(arg_opt=="-o")then
                 call getarg_(i+1,output_cmsum_file_name)
                 output_cmsum_file_name=trim(output_cmsum_file_name)
                 !print*,' -o option gives ouput binary m filename:',&
                 !                           &    trim(output_cmsum_file_name)
              elseif(arg_opt=="-a")then
                 call getarg_(i+1,append_new_string)
                 append_new_string=trim(append_new_string)
                 !print*,' -a option gives APPEND/NEW specification:',&
                 !                           &    trim(append_new_string)
              else
                 print*,' Illegal command line option #:',i,'Option:', &
                      trim(arg_opt)
                 stop 'KASSUM_HDF5: Illegal command line option'
              endif
           enddo
        else
           print*,' KASSUM_HDF5--No command line arguments.'
           print*,' KASSUM_HDF5--Using defaults for all.'
        endif
!******************************************************************************


	call fileopen		!Open the input 'M' file.

	call process

END PROGRAM KASSUM_HDF5
