
MODULE PES2HDF5_SUBS
	use structures
	type(m_head) :: mhead

CONTAINS

    SUBROUTINE FILEOPEN
!******************************************************************************
!	Open the PES file and read in its header.
!       Create the output HDF file, Read in the header record, Define a 
!       datatype for the header,define a dataset for the header, write out the
!       header to the HDF file, define the compound datatype for the pes 
!       records, define the pes dataset
!******************************************************************************

!	Modified:

	use structures
	IMPLICIT NONE

	integer :: pt,ios,error
 	real :: dni,area
        type(pe_head),pointer :: pehead

	character(len=12),dimension(9) ::  ptype=(/ &           !PMT/Grid type
	&' Visable    ',' Old Luny   ',' Solar Blind',' CEASAR     ', &
	&' ASGAT      ',' Part-on-gnd','Whipple     ',' VERITAS    ', &
	&'VERITAS-TRI '/)
	character(len=8),dimension(18) :: namtyp=(/ &	!Particle names.
	&'Gamma   ','Positron','Electron','Mu +    ','Mu -    ','PI 0    ', &
	&'PI +    ','PI -    ','K +     ','K -     ','K long  ','K short ', &
	&'Proton  ','Neutron ','eNeutrin','anti-eNu','muNeutri','ant-muNu'/)

      allocate(pehead,stat=error)
	if(error/= 0)then
		print*,"Program could not allocte space for pehead."
	    	stop
	endif

!	Input file. Structure from STRUCTURES
!	Read in the pe header lines and write the info out.
        OPEN(2,ACCESS='sequential',STATUS='old',FORM='unformatted',&
             & ACTION='READ',file='pesfile.dat',iostat=ios)
        if(ios>0)then
           print*,' File open failure for (pesfile.dat input file)'
           stop ' File open failure'
        endif
	read(2)pehead

 	dni=sqrt(1.-pehead%segment_head%dli**2-  &
		&   pehead%segment_head%dmi**2)
	area=pehead%xseg*pehead%yseg


	if(index(pehead%petype,'VI')/=0)then
		pt=1
	elseif(index(pehead%petype,'OL')/=0)then
		pt=2
	elseif(index(pehead%petype,'SB')/=0)then
		pt=3
	elseif(index(pehead%petype,'C')/=0)then
		pt=4
	elseif(index(pehead%petype,'AS')/=0)then
		pt=5
	elseif(index(pehead%petype,'POG')/=0)then
		pt=6
	elseif(index(pehead%petype,'W')/=0)then
		pt=7
	elseif(index(pehead%petype,'VA')/=0)then
		pt=8
	elseif(index(pehead%petype,'VT')/=0)then
		pt=9
	else
		write(6,1001)pehead%petype
1001	format(' PES2HDF5_FATAL--Illegal photon type:petype=',a)
		stop
	endif

	write(6,1002)
1002	format(' **********PES2HDF5 conversion program.*******',/, &
             & ' SHOWER SPECIFICATION:')

	write(6,1003)ptype(pt),						&
	& namtyp(pehead%segment_head%itype),                    &
	& pehead%segment_head%tep,                              &
	& pehead%segment_head%dli,                              &
	& pehead%segment_head%dmi,dni,                          &
	& pehead%segment_head%depth,                            &
	& pehead%hobs,                                          &
	& pehead%segment_head%magnet_field,                     &
	& pehead%segment_head%etr,                              &
	& pehead%xseg,pehead%yseg,                      &
	& pehead%segment_head%idfile,                           &
	& pehead%x_offset,pehead%y_offset,              &
	& pehead%segment_head%version,                          &
	& pehead%version
1003	format(      							&
	& '                Detector type: ',a,/,                        &
	& '                 Primary type: ',a8,/,                       &
	& '               Primary Energy: ',f10.5,' TeV',/              &
	& '        Primary  DLI DMI DNI : ', 3e14.7,/,                  &
	& '              Injection depth: ',f10.2,/,                    &
	& '            Detector altitude: ',f10.1,/,                    &
	& ' Magnetic Field specification: ',a,/,                        &
	& '                Thershold MeV: ' f8.2,/,                     &
	& '                    xseg,yseg: ',f8.2,' x',f8.2,/,           &
	& '                       IDFILE: ',i5,/,                       &
	& '   Shower Offset Impact Point: ','x=',f6.2,' y=',f6.2,/,     &
	& '         From KASCADE Version: ',a,/,                        &
	& '         From KASLITE Version: ',a)



!	open the output HDF file and write out the header summary file.

!These are C routines.
!       print*,' Creating HDF5 file'
        call peshdf5_create("pesfile.hdf")

!       Write out the pehead record into its own dataset.
!       print*,' Writing out pe_head'
        call peshdf5_pehead_write(pehead)

	return
    END SUBROUTINE FILEOPEN
!******************************************************************************

    SUBROUTINE PROCESS(verify)
!******************************************************************************
!	This routine_ procesess the pes files. 
!	All events are added to the output dataset file.
!******************************************************************************

!	Modified:

      use structures

      IMPLICIT NONE
      logical :: verify,v_fail

      integer :: icount,error,i,j,ierr,pt,ios
      type(pe_rec),pointer ::  pe,pe_old
      real :: dni,area
      type(pe_head),pointer :: pehead, pehead_old

	character(len=12),dimension(9) ::  ptype=(/ &           !PMT/Grid type
	&' Visable    ',' Old Luny   ',' Solar Blind',' CEASAR     ', &
	&' ASGAT      ',' Part-on-gnd','Whipple     ',' VERITAS    ', &
	&'VERITAS-TRI '/)
	character(len=8),dimension(18) :: namtyp=(/ &	!Particle names.
	&'Gamma   ','Positron','Electron','Mu +    ','Mu -    ','PI 0    ', &
	&'PI +    ','PI -    ','K +     ','K -     ','K long  ','K short ', &
	&'Proton  ','Neutron ','eNeutrin','anti-eNu','muNeutri','ant-muNu'/)

      allocate(pehead,stat=error)
	if(error/= 0)then
		print*,"Program could not allocte space for pehead."
	    	stop
	endif
      
      allocate(pe,stat=error)
	if(error/= 0)then
		print*,"Program could not allocte space for pe."
	    	stop
	endif


        if(.not.verify)then



!	read a pe from the input file until eof reached.
           do
!	   print*,' Read in pe record'
              read(2,end=533)pe
              icount=icount+1
!debug
!           if(icount<10)then
!              write(6,1000)icount,pe
1000 format(' i:',i6,2i5,e14.7,2f7.4,i6,2f6.2,i3,i4,f9.2)
!            endif

!And write it out. Pe event # start at 0.(i.e. index=icount-1)
!           print*,' Write out pe record'
              call peshdf5_out(icount-1,pe)
           enddo


        else

!DEBUG***************************************************************

!**************************************************************************
!Now verify the HDF5 file. Read header and all pes records 
!!     Check that this all worked.
!**************************************************************************
           allocate(pehead_old,stat=error)
           if(error/= 0)then
              print*,"Program could not allocte space for pehead_old."
              stop
           endif
      
           allocate(pe_old,stat=error)
           if(error/= 0)then
              print*,"Program could not allocte space for pe_old."
              stop
           endif
!	Input file. Structure from STRUCTURES
!	Read in the pe header lines and write the info out.
           OPEN(2,ACCESS='sequential',STATUS='old',FORM='unformatted',&
                & ACTION='READ',file='pesfile.dat',iostat=ios)
           if(ios>0)then
              print*,' File open failure for (pesfile.dat input file)'
              stop ' File open failure'
           endif
           read(2)pehead_old

           print*,' Open pesfile.hdf'
           call peshdf5_open("pesfile.hdf",ierr)
           if(ierr<0)then
              print*,' HDF5 open failure for file pesfile.dat'
              stop  'File open failure'
           endif
        
           print*,' Read in the header'
           call peshdf5_pehead_read(pehead)
           print*,' Header read in.'
 

           dni=sqrt(1.-pehead%segment_head%dli**2-  &
		&   pehead%segment_head%dmi**2)
           area=pehead%xseg*pehead%yseg

           if(index(pehead%petype,'VI')/=0)then
              pt=1
           elseif(index(pehead%petype,'OL')/=0)then
              pt=2
           elseif(index(pehead%petype,'SB')/=0)then
              pt=3
           elseif(index(pehead%petype,'C')/=0)then
              pt=4
           elseif(index(pehead%petype,'AS')/=0)then
              pt=5
           elseif(index(pehead%petype,'POG')/=0)then
              pt=6
           elseif(index(pehead%petype,'W')/=0)then
              pt=7
           elseif(index(pehead%petype,'VA')/=0)then
              pt=8
           elseif(index(pehead%petype,'VT')/=0)then
              pt=9
           else
              write(6,1001)pehead%petype
1001	format(' PES2HDF5_FATAL--Illegal photon type:petype=',a)
              stop
           endif

           write(6,1002)
1002	format(' SHOWER SPECIFICATION from HDF5 pes file:')

           write(6,1003)ptype(pt),			        &
                & namtyp(pehead%segment_head%itype),                    &
                & pehead%segment_head%tep,                              &
                & pehead%segment_head%dli,                              &
                & pehead%segment_head%dmi,dni,                          &
                & pehead%segment_head%depth,                            &
                & pehead%hobs,                                          &
                & pehead%segment_head%magnet_field,                     &
                & pehead%segment_head%etr,                              &
                & pehead%xseg,pehead%yseg,                      &
                & pehead%segment_head%idfile,                           &
                & pehead%x_offset,pehead%y_offset,              &
                & pehead%segment_head%version,                          &
                & pehead%version
1003	format(      							&
             & '                Detector type: ',a,/,                        &
             & '                 Primary type: ',a8,/,                       &
             & '               Primary Energy: ',f10.5,' TeV',/              &
             & '        Primary  DLI DMI DNI : ', 3e14.7,/,                  &
             & '              Injection depth: ',f10.2,/,                    &
             & '            Detector altitude: ',f10.1,/,                    &
             & ' Magnetic Field specification: ',a,/,                        &
             & '                Thershold MeV: ' f8.2,/,                     &
             & '                    xseg,yseg: ',f8.2,' x',f8.2,/,           &
             & '                       IDFILE: ',i5,/,                       &
             & '   Shower Offset Impact Point: ','x=',f6.2,' y=',f6.2,/,     &
             & '         From KASCADE Version: ',a,/,                        &
             & '         From KASLITE Version: ',a)

!	read a pe from the HDF file until eof reached.

           print*,' Read in and verify pe records'
           icount=0
           do
             
              read(2,end=533)pe_old

              call peshdf5_in(icount,pe,ierr)
!              print*,' ierr:',ierr
              if(ierr<0)then
                 print*,'No more events at i:',i
                 exit
              endif

              v_fail=.false.
              if(pe_old%nx/=pe%nx)then
                 v_fail=.true.
              elseif(pe_old%ny/=pe%ny)then
                 v_fail=.true.
              elseif(pe_old%time/=pe%time)then
                 v_fail=.true.
              elseif(pe_old%dlr/=pe%dlr)then
                 v_fail=.true.
              elseif(pe_old%dmr/=pe%dmr)then
                 v_fail=.true.
              elseif(pe_old%nnext/=pe%nnext)then
                 v_fail=.true.
              elseif(pe_old%xm/=pe%xm)then
                 v_fail=.true.
              elseif(pe_old%ym/=pe%ym)then
                 v_fail=.true.
              elseif(pe_old%spec/=pe%spec)then
                 v_fail=.true.
              elseif(pe_old%lambda/=pe%lambda)then
                 v_fail=.true.
              elseif(pe_old%em_alt/=pe%em_alt)then
                 v_fail=.true.
              endif
              if(v_fail)then
                 print*,'Verify failed at record #:',i
                 print*,' pe_old:',pe_old
                 print*,' pe:',pe
                 stop 'Verify failed'
              endif


!              if(i<10)then
!                 write(6,1000)i,pe
!              endif
              icount=icount+1
           enddo

!	End of shower.
!	write out the last pes record found and close it up
!          Print*,' Last pe record:'
!          write(6,1000)i,pe

        endif
!	End of shower.
!	write out the number of pes records found and close it up
533        print*,' End of shower found.',icount,' events sent to HDF file.'

!debug
!        Print*,' Last pe re4cord:'
!        write(6,1000)icount,pe

           close(unit=2)
           call peshdf5_close
           return
!END DEBUG***************************************************************

    END SUBROUTINE PROCESS
END MODULE PES2HDF5_SUBS

PROGRAM PES2HDF5
!******************************************************************************
!	This program reads in a binary PES file for the whipple
!	Camera and converts it into an hdf5 file.
!******************************************************************************

!	Files:
!	unit=2  'PES' input file.


!	G.H.Sembroski
!	Physics Dept
!	Purdue University
!	W.Lafayette, In 47907
!	E-Mail: 47474::SEMBROSKI

!	Modified:

!	19/9/00 GHS
!		Converted from kassum_compress.f90

!		In FILEOPEN
!		1:Use 'pesfile.dat' as assigned logical name of input binary
!                 pes file.
!               2.Use 'pesfile.hdf' as assigned logical name of output PES
!                 hdf5 file.
	use pes2hdf5_subs
	IMPLICIT NONE

	logical :: verify

        verify=.true.
!        verify=.false.

        if(.not.verify) then
           call fileopen	!Open the input 'PES' file.
        else
           print*,' Verifying BIN and HDF files'
        endif

                                !Create the output HDF file, Read in the header
                                !record, Define a datatype for the header, 
                                !define a dataset for the header, write out the
                                !header to the HDF file.
                                !define the compound datatype for the pes 
                                !records, define the pes dataset
	call process(verify)    !Read in PES file and write out to HDF5 pes
                                !dataset. Clean up when done.
END PROGRAM PES2HDF5
