!**********************************************************************
!	F77 code for KASTRIGGER
!	Includes CERN source code for RANLUX.
!**********************************************************************

	subroutine load_pst_patterns(pst_patterns,n)

!**********************************************************************
!	Read in from a binary file the pst pattern array. Done as a
!	f77 routine so we could use in Absoft fortran LInux the variuous
!	options (-N3 and -N51) that let us read binary files made on a 
!	VMS machine
!**********************************************************************
	integer*2 pst_patterns(n)
	integer*4 n,i
	open(41,file='pst_mul2.dat',status='OLD',form='UNFORMATTED')
	do i=1,n
        	read(41)pst_patterns(i)
		if(i.lt.100)then
		  write(6,4300)' i:pst_patterns:',i,pst_patterns(i)
4300	format(a,i5,o5)
		endif
	enddo
	close(unit=41)		
	return
	end
!**********************************************************************

        function gauss(x)
!	Modified:
!	2/4/92 G.H.S. V:1:0:1:0:0.4
!		Fix width of GAUSS. It was .6932. Make it 1.0 so nfluct has
!		correct width.

!	NOTE: This funciton only gives values out to 3/.6932=4.33 sigma and 
!	above 3 sigma its not perfect(but its not bad!).

c     This is petes for use by nfluct.
	real xdummy

!	Sum 6 random numbers(whose mean will be 3)

        sum=pran(xdummy)
        do i=1,5
                sum=sum+pran(xdummy)
	enddo
        gauss=(sum-3.)/.6932	!Put mean at 0. Correct width.
        return
        end
!**********************************************************************

       subroutine geom(dl,dm,dn,tix)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c      geom is a subroutine_ which determines
c      the new direction cosines of the particle
cu              dl dm dn dir cos of segm
cu       input:       tix       m.c.s. angle
cu       output:       dl dm dn new dir cos
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc.

c     10/23/86 G.H.S.       Always force final direction vector to be unit
c                    length. Adjust dn to make it so. Does this cause
c                    any bias??? May become non unit length by sucessive
c                    round off error.

c       Warning: This program has round off problems when tix < .001. If
c       this is an important region use geom8.

c	Modified:

c	15/12/93 GHS
c		Convert to real*8 internally to prevent round off problems
c		for small tix.

	external pran
	real*8 pi
	parameter  (pi=3.141592654)
	real*8 a(9),b(3),r9(3),f,rk,c,s,vlength
	real*8 dn_local,dl_local,dm_local,qq,p,u,tix_local
	real dl,dm,dn,tix
       qq(p,u)=sqrt(p*p+u*u)


	tix_local=tix		!Convert input variables to r*8 internaly
	dl_local=dl
	dm_local=dm
				!Calculate our own dn. Make it r*8 to prevent
				!round of problems for small tix
	dn_local=sqrt(1.d0-dl_local**2-dm_local**2)

       f=qq(dl_local,dm_local)       !f is sin of verticle angle.
       a(1)=dl_local*dn_local/f
       a(2)=-dm_local/f
       a(3)=dl_local
       a(4)=dm_local*dn_local/f
       a(5)=dl_local/f
       a(6)=dm_local
       a(7)=-f
       a(8)=0.
       a(9)=dn_local
c             | a(1) a(2) a(3) |  | b(1) |  |r9(1) |
c             | a(4) a(5) a(6) | x| b(2) |= |r9(2) |
c             | a(7) a(8) a(9) |  | b(3) |  |r9(3) |
c             vector b is new direction in frame of
c             track
c
c             vector a(3);a(6);a(9)  is track in lab
c             frame  -v
c
c             vector a(2);a(5);.. is vector product
c             vxz normalized (mod=1)

       rk=2.d0*pi*pran(xdummy)
       c=cos(rk)
       s=sin(rk)
       b(1)=c*sin(tix_local)
       b(2)=s*sin(tix_local)
       b(3)=cos(tix_local)
c
c standard routine for multiplying of matrices
c	Converted to r*8 15/12/93 GHS
       call mtxmlt(a,b,r9)
c
c new direction cosines
c
c      Force unit length to the vector.
c     10/23/86
       vlength=sqrt(r9(1)**2+r9(2)**2+r9(3)**2)
       dl=r9(1)/vlength
       dm=r9(2)/vlength
       dn=r9(3)/vlength
       if(abs(vlength-1.).gt.1.e-5)then
		write(6,1000)dl,dm,dn,r9(3),vlength
1000  format(' ***Geom:Found Unit vector correction .gt.1.e-5.
     1 dl,dm,dn,r9(3),vlength:',/,' ***',5e14.7)
	endif

       if(abs(dn).lt.1.e-8)then
               dn=1.e-8       !Cheap way to handle horizontal tracks.
               dl=(1.-1.e-8)*dl
               dm=(1.-1.e-8)*dm                       
       endif
       return
       end

!**********************************************************************

       subroutine geom8(dl,dm,dn,tix,tiy)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c      geom is a subroutine_ which determines
c      the new direction cosines of the particle
cu              dl dm dn dir cos of segm
cu       input:       tix    x scat angle
cu       input:       tiy    y scat angle
cu       output:       dl dm dn new dir cos
c        This version assumes input varaibles are r*8. Do this to avoid
c        roundoff problems for small tix (tix<.001)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc.

c     10/23/86 G.H.S.       Always force final direction vector to be unit
c                    length. Adjust dn to make it so. Does this cause
c                    any bias??? May become non unit length by sucessive
c                    round off error.


c	Modified:

c	15/12/93 GHS
c		Convert to real*8 internally to prevent round off problems
c		for small tix.
c       28/8/00 GHS
c               Convert to use of and x and y scattereing angles.

	external pran
	real*8 pi
	parameter  (pi=3.141592654)
	real*8 a(9),b(3),r9(3),f,c,s,vlength
!	real*8 rk
	real*8 dn_local,dl_local,dm_local,qq,p,u,tix_local
	real*8 dl,dm,dn,tix,tiy
       qq(p,u)=sqrt(p*p+u*u)


	dl_local=dl
	dm_local=dm
				!Calculate our own dn. Make it r*8 to prevent
				!round of problems for small tix
	dn_local=sqrt(1.d0-dl_local**2-dm_local**2)

       f=qq(dl_local,dm_local)       !f is sin of verticle angle.
       a(1)=dl_local*dn_local/f
       a(2)=-dm_local/f
       a(3)=dl_local
       a(4)=dm_local*dn_local/f
       a(5)=dl_local/f
       a(6)=dm_local
       a(7)=-f
       a(8)=0.
       a(9)=dn_local
c             | a(1) a(2) a(3) |  | b(1) |  |r9(1) |
c             | a(4) a(5) a(6) | x| b(2) |= |r9(2) |
c             | a(7) a(8) a(9) |  | b(3) |  |r9(3) |
c             vector b is new direction in frame of
c             track
c
c             vector a(3);a(6);a(9)  is track in lab
c             frame  -v
c
c             vector a(2);a(5);.. is vector product
c             vxz normalized (mod=1)

       tix_local=sqrt(tix**2+tiy**2)	!Get length of scatter.
	if(tix_local==0)then
	   return     !no change to vector.
	endif

!Use specific tix and tiy
!       rk=2.d0*pi*pran(xdummy)
!       c=cos(rk)
!       s=sin(rk)

       c=tix/tix_local
       s=tiy/tix_local
       b(1)=c*sin(tix_local)
       b(2)=s*sin(tix_local)
       b(3)=cos(tix_local)
c
c standard routine for multiplying of matrices
c	Converted to r*8 15/12/93 GHS
       call mtxmlt(a,b,r9)
c
c new direction cosines
c
c      Force unit length to the vector.
c     10/23/86
       vlength=sqrt(r9(1)**2+r9(2)**2+r9(3)**2)
       dl=r9(1)/vlength
       dm=r9(2)/vlength
       dn=r9(3)/vlength
       if(abs(vlength-1.).gt.1.e-5)then
		write(6,1000)dl,dm,dn,r9(3),vlength
1000  format(' ***Geom:Found Unit vector correction .gt.1.e-5.
     1 dl,dm,dn,r9(3),vlength:',/,' ***',5e14.7)
	endif

       if(abs(dn).lt.1.e-8)then
               dn=1.e-8       !Cheap way to handle horizontal tracks.
               dl=(1.-1.e-8)*dl
               dm=(1.-1.e-8)*dm                       
       endif
       return
       end

!**********************************************************************

	subroutine mass_number2charge_mass(ia,qz,xmass)
!******************************************************************************
!   Determine charge Z of most stable nuclei of nuclear number A
!   Determine mass from Z and A using semi-empirical mass formula
!******************************************************************************
!   Uses fromulae from "Physics of Nuclei and Particles, Vol 1", Marmier and 
!   Sheldon,1969, Acedemic Press, pg 36-38, formula: 2-35,2-38,2-39,2-40
!******************************************************************************
!   This is from the liquid drop model of stable nuclei.

!  Written by:
!  Glenn Sembroski
!  Physics Dept.
!  Purdue Univ.
!  W. Lafayette, IN USA 47907

! Modified:



	IMPLICIT NONE
	real a
	integer qz,ia
	real  xmass,pmass
	logical  first_argon

	real mp,mn,ap,av,as,ac,aa
	parameter (mp=938.256e-6)   !Mass of proton (TeV)
	parameter (mn=939.550e-6)   !Mass of neutron(TeV)
	parameter (ap=33.5e-6)     !Pairing coef.(TeV)
	parameter (av=14.1e-6)     !Volume coef (TeV)
	parameter (as=13.0e-6)     !Surface coef(TeV)
	parameter (ac=0.595e-6)    !Coulmb coef.(TeV)
	parameter (aa=19e-6)       !Assymetry coef(TeV)
	data first_argon /.true./


         a=ia
                         !Correct our formula for elements up to a=56(Fe)
                         !which is as high as NUC_LIB goes.
         if(ia==18)then	  !Force Oxygen isotope
	    qz=8
	 elseif(ia==24)then
            qz=12         !Force Magnesium
         elseif(ia==28)then
            qz=14         !Force silicon
         elseif(ia==32)then
            qz=16         !Force Sulpher
         elseif(ia==33)then
            qz=16         !Force Sulpher
         elseif(ia==35)then
            qz=17         !Force Chlorine
         elseif(ia==39)then
            qz=19         !Force Potassium
         elseif(ia==40)then
            qz=18         !Force Argon !Could have been calcium 40.
            if(first_argon)then
               print*,'Warning--Forcing Argon for all atomic masses of 40'
               first_argon=.false.
            endif
         elseif(a==56)then
            qz=26         !Force Iron.
         else
            qz=anint(a/(1.98+0.0155*(a**(2./3.)))) 	!Use nearest integer 
							!function
         endif

!First determine pairing mass term
         if(mod(qz,2)==0)then
            if(mod(ia,2)==0)then
               pmass=-ap*a**(-.75)   !even-even nuclei
            else
               pmass=0.              !even-odd nuclei
            endif
         else
            if(mod(ia,2)==0)then
               pmass=0.              !Odd-even nuclei
            else
               pmass=ap*a**(-.75)   !Odd-odd  nuclei
            endif
         endif

         xmass = qz*mp + (a-qz)*mn - av*a + as*(a**(2./3.)) +
	1 ac*(qz**2)/(a**(1./3.)) + aa*((a-2*qz)**2)/a + pmass
 	return
	end
!******************************************************************************


              subroutine mtxmlt(a,b,r)
c      Calculates r=Ab, where r,b are vectors and A is matrix, all 3-d.

c	Modified:

c	15/12/93 GHS
c		Make everything real*8

       real*8 a(9),b(3),r(3)
       do 100 i=1,3
              r(i)=0.
              ii=3*(i-1)
              do 101 j=1,3
                     ij=ii+j
                     r(i)=a(ij)*b(j)+r(i)
101           continue
100    continue
       return
       end
!****************************************************************************


	function nfluct(x)
!****************************************************************************
c       Puts a statistical fluctuation on number of pe's(photons)
!****************************************************************************
c       written by:  Pete Palfrey
c                    Purdue
c                    4/1/89

!	Modified:
!	11/5/98 GHS V:1:1:5.3
!		Fix bug in NFLUCT. For small values of x (x<12) check for
!		values of nfluct out to 18. before we were limited to 4*x.
!		This was especially bad for x<.25. This error became very 
!		noticeable when we went to segment step sizes of .02 radiation
!		lengths.

	external pran

c       See if fluctuation will be poisson.
        if(x.lt.12.) then        !Poisson
                sum=exp(-x)
                old=sum
                test=pran(dummy)
!	11/5/98 GHS V:1:1:5.3 Let imax always be 18.
!                imax=min(4.*x,18.)
                imax=18
                do i=1,imax
                        if(sum.gt.test) then
                                nfluct=i-1
                                return
                        end if
                        xx=i
                        old=old*x/xx
                        sum=sum+old
                end do
                nfluct=imax
c        Fluctuation is gaussian distributed.
        else
                sca=sqrt(x)*gauss(x) + x
                if(sca.gt.0.) then
                        nfluct=nint(sca)
                else
                        nfluct=0
                end if
        end if
        return
        end
!****************************************************************************


        FUNCTION REXP(X)
c      This function randomly picks from an exponatial distribution using
c      X as the scale factor.
c      Its used for interaction depth(Xis is gm/cm**2) and for decay length
c      (where x is in meters)
c      Rexp is distributed as exp(-s/X)
	external pran

        Q = pran(xdummy)

c       This produces the required random distribution.
	REXP = -X*ALOG(Q)
	RETURN
        END
!****************************************************************************


!****************************************************************************
!	RANLUX from CERN 98 distribution
!****************************************************************************

*
* $Id: w10m_f77.for,v 1.1.1.1 2004/01/07 18:31:06 cvsuser Exp $
*
* $Log: w10m_f77.for,v $
* Revision 1.1.1.1  2004/01/07 18:31:06  cvsuser
* GHS: Inital creation of KASCADE Simulation CVS repository.
* GHS: This is for KASCADE as of 07/01/04.
* GHS: Made by:
* GHS: Glenn Sembroski, Physics Dept. Purdue Univ, West Lafayette, In. 47907
* GHS: 765-494-5172
* GHS: sembroski@physics.purdue.edu
* GHS: This is KASCADE version: V:1:3:8:3:4:4
* GHS: Since this is inital installation no guarentee things will work.
*
* Revision 1.1.1.1  2001/11/26 20:10:06  sembroski
* Importing KASCADE Sources
*
* Revision 1.1.1.1  2001/09/04 14:27:53  sembroski
* Importing kascade sources
*
* Revision 1.2  1997/09/22 13:45:47  mclareni
* Correct error in initializing RANLUX by using RLUXIN with the output of
* RLUXUT from a previous run.
*
* Revision 1.1.1.1  1996/04/01 15:02:55  mclareni
* Mathlib gen
*
*
!#include "gen/pilot.h"
      SUBROUTINE RANLUX(RVEC,LENV)
C         Subtract-and-borrow random number generator proposed by
C         Marsaglia and Zaman, implemented by F. James with the name
C         RCARRY in 1991, and later improved by Martin Luescher
C         in 1993 to produce "Luxury Pseudorandom Numbers".
C     Fortran 77 coded by F. James, 1993
C
C   LUXURY LEVELS.
C   ------ ------      The available luxury levels are:
C
C  level 0  (p=24): equivalent to the original RCARRY of Marsaglia
C           and Zaman, very long period, but fails many tests.
C  level 1  (p=48): considerable improvement in quality over level 0,
C           now passes the gap test, but still fails spectral test.
C  level 2  (p=97): passes all known tests, but theoretically still
C           defective.
C  level 3  (p=223): DEFAULT VALUE.  Any theoretically possible
C           correlations have very small chance of being observed.
C  level 4  (p=389): highest possible luxury, all 24 bits chaotic.
C
C!!! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C!!!  Calling sequences for RANLUX:                                  ++
C!!!      CALL RANLUX (RVEC, LEN)   returns a vector RVEC of LEN     ++
C!!!                   32-bit random floating point numbers between  ++
C!!!                   zero (not included) and one (also not incl.). ++
C!!!      CALL RLUXGO(LUX,INT,K1,K2) initializes the generator from  ++
C!!!               one 32-bit integer INT and sets Luxury Level LUX  ++
C!!!               which is integer between zero and MAXLEV, or if   ++
C!!!               LUX .GT. 24, it sets p=LUX directly.  K1 and K2   ++
C!!!               should be set to zero unless restarting at a break++ 
C!!!               point given by output of RLUXAT (see RLUXAT).     ++
C!!!      CALL RLUXAT(LUX,INT,K1,K2) gets the values of four integers++
C!!!               which can be used to restart the RANLUX generator ++
C!!!               at the current point by calling RLUXGO.  K1 and K2++
C!!!               specify how many numbers were generated since the ++
C!!!               initialization with LUX and INT.  The restarting  ++
C!!!               skips over  K1+K2*E9   numbers, so it can be long.++
C!!!   A more efficient but less convenient way of restarting is by: ++
C!!!      CALL RLUXIN(ISVEC)    restarts the generator from vector   ++
C!!!                   ISVEC of 25 32-bit integers (see RLUXUT)      ++
C!!!      CALL RLUXUT(ISVEC)    outputs the current values of the 25 ++
C!!!                 32-bit integer seeds, to be used for restarting ++
C!!!      ISVEC must be dimensioned 25 in the calling program        ++
C!!! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      DIMENSION RVEC(LENV)
      DIMENSION SEEDS(24), ISEEDS(24), ISDEXT(25)
      PARAMETER (MAXLEV=4, LXDFLT=3)
      DIMENSION NDSKIP(0:MAXLEV)
      DIMENSION NEXT(24)
      PARAMETER (TWOP12=4096., IGIGA=1000000000,JSDFLT=314159265)
      PARAMETER (ITWO24=2**24, ICONS=2147483563)
      SAVE NOTYET, I24, J24, CARRY, SEEDS, TWOM24, TWOM12, LUXLEV
      SAVE NSKIP, NDSKIP, IN24, NEXT, KOUNT, MKOUNT, INSEED
!      SAVE precarry
      INTEGER LUXLEV
      LOGICAL NOTYET
      DATA NOTYET, LUXLEV, IN24, KOUNT, MKOUNT /.TRUE., LXDFLT, 0,0,0/
      DATA I24,J24,CARRY/24,10,0./
C                               default
C  Luxury Level   0     1     2   *3*    4
      DATA NDSKIP/0,   24,   73,  199,  365 /
Corresponds to p=24    48    97   223   389
C     time factor 1     2     3     6    10   on slow workstation
C                 1    1.5    2     3     5   on fast mainframe
C
C  NOTYET is .TRUE. if no initialization has been performed yet.
C              Default Initialization by Multiplicative Congruential
      IF (NOTYET) THEN
         NOTYET = .FALSE.
         JSEED = JSDFLT  
         INSEED = JSEED
         WRITE(6,'(A,I12)') ' RANLUX DEFAULT INITIALIZATION: ',JSEED
         LUXLEV = LXDFLT
         NSKIP = NDSKIP(LUXLEV)
         LP = NSKIP + 24
         IN24 = 0
         KOUNT = 0
         MKOUNT = 0
         WRITE(6,'(A,I2,A,I4)')  ' RANLUX DEFAULT LUXURY LEVEL =  ',
     +        LUXLEV,'      p =',LP
            TWOM24 = 1.
         DO 25 I= 1, 24
            TWOM24 = TWOM24 * 0.5
         K = JSEED/53668
         JSEED = 40014*(JSEED-K*53668) -K*12211
         IF (JSEED .LT. 0)  JSEED = JSEED+ICONS
         ISEEDS(I) = MOD(JSEED,ITWO24)
   25    CONTINUE
         TWOM12 = TWOM24 * 4096.
         DO 50 I= 1,24
         SEEDS(I) = REAL(ISEEDS(I))*TWOM24
         NEXT(I) = I-1
   50    CONTINUE
         NEXT(1) = 24
         I24 = 24
         J24 = 10
         CARRY = 0.
         IF (SEEDS(24) .EQ. 0.) CARRY = TWOM24
      ENDIF
C
C          The Generator proper: "Subtract-with-borrow",
C          as proposed by Marsaglia and Zaman,
C          Florida State University, March, 1989
C
      DO 100 IVEC= 1, LENV
      UNI = SEEDS(J24) - SEEDS(I24) - CARRY 



!      precarry=carry
      IF (UNI .LT. 0.)  THEN
         UNI = UNI + 1.0
         CARRY = TWOM24
      ELSE
         CARRY = 0.
      ENDIF
!debug
! Looking for when uni becomes NaN. This is a dumb way to find this out
! but it should work. I think
!      call check_seeds(uni,j24,i24,precarry)
!enddebug

      SEEDS(I24) = UNI
      I24 = NEXT(I24)
      J24 = NEXT(J24)
      RVEC(IVEC) = UNI
C  small numbers (with less than 12 "significant" bits) are "padded".
      IF (UNI .LT. TWOM12)  THEN
         RVEC(IVEC) = RVEC(IVEC) + TWOM24*SEEDS(J24)
C        and zero is forbidden in case someone takes a logarithm
         IF (RVEC(IVEC) .EQ. 0.)  RVEC(IVEC) = TWOM24*TWOM24
      ENDIF
C        Skipping to luxury.  As proposed by Martin Luscher.
      IN24 = IN24 + 1
      IF (IN24 .EQ. 24)  THEN
         IN24 = 0
         KOUNT = KOUNT + NSKIP
         DO 90 ISK= 1, NSKIP
         UNI = SEEDS(J24) - SEEDS(I24) - CARRY
!         precarry=carry
	 IF (UNI .LT. 0.)  THEN
            UNI = UNI + 1.0
            CARRY = TWOM24
         ELSE
            CARRY = 0.
         ENDIF
!debug
! Looking for when uni becomes NaN. This is a dumb way to find this out
! but it should work. I think
!	 call check_seeds(uni,j24,i24,precarry)
!enddebug
         SEEDS(I24) = UNI
         I24 = NEXT(I24)
         J24 = NEXT(J24)
   90    CONTINUE
      ENDIF
  100 CONTINUE
      KOUNT = KOUNT + LENV
      IF (KOUNT .GE. IGIGA)  THEN
         MKOUNT = MKOUNT + 1
         KOUNT = KOUNT - IGIGA
      ENDIF
      RETURN
C
C           Entry to input and float integer seeds from previous run
      ENTRY RLUXIN(ISDEXT)
         NOTYET = .FALSE.
         TWOM24 = 1.
         DO 195 I= 1, 24
         NEXT(I) = I-1
  195    TWOM24 = TWOM24 * 0.5
         NEXT(1) = 24
         TWOM12 = TWOM24 * 4096.
      WRITE(6,'(A)') ' FULL INITIALIZATION OF RANLUX WITH 25 INTEGERS:'
      WRITE(6,'(5X,5I12)') ISDEXT
      DO 200 I= 1, 24
	 if(isdext(i)<0)then
	    print*,'WARNING-RANLUX-possible bad input seed file'
	    print*,'WARNING-RANLUX-Negative seed found'
	    print*,'WARNING-RANLUX-All seeds but last
	1 should be positive'
	    print*,' I,ISDEXT(I):',i,isdext(i)
	endif
      SEEDS(I) = REAL(ISDEXT(I))*TWOM24
  200 CONTINUE
      CARRY = 0.
      IF (ISDEXT(25) .LT. 0)  CARRY = TWOM24
      ISD = IABS(ISDEXT(25))
      I24 = MOD(ISD,100)
      ISD = ISD/100
      J24 = MOD(ISD,100)
      ISD = ISD/100
      IN24 = MOD(ISD,100)
      ISD = ISD/100
      LUXLEV = ISD
        IF (LUXLEV .LE. MAXLEV) THEN
          NSKIP = NDSKIP(LUXLEV)
          WRITE (6,'(A,I2)') ' RANLUX LUXURY LEVEL SET BY RLUXIN TO: ',
     +                         LUXLEV
        ELSE  IF (LUXLEV .GE. 24) THEN
          NSKIP = LUXLEV - 24
          WRITE (6,'(A,I5)') ' RANLUX P-VALUE SET BY RLUXIN TO:',LUXLEV
        ELSE
          NSKIP = NDSKIP(MAXLEV)
          WRITE (6,'(A,I5)') ' RANLUX ILLEGAL LUXURY RLUXIN: ',LUXLEV
          LUXLEV = MAXLEV
        ENDIF
      INSEED = -1
      RETURN
C
C                    Entry to ouput seeds as integers
      ENTRY RLUXUT(ISDEXT)
      DO 300 I= 1, 24
         ISDEXT(I) = INT(SEEDS(I)*TWOP12*TWOP12)
	 if(isdext(i)<0)then
	    print*,'WARNING-RANLUX-possible error converting 
	1 seeds to integer for saving'
	    print*,'WARNING-RANLUX-Negative seed found'
	    print*,'WARNING-RANLUX-All seeds but last
	1 should be positive'
	    print*,' I,ISDEXT(I),SEEDS(I):',i,isdext(i),seeds(i)
	endif

 300	CONTINUE
      ISDEXT(25) = I24 + 100*J24 + 10000*IN24 + 1000000*LUXLEV
      IF (CARRY .GT. 0.)  ISDEXT(25) = -ISDEXT(25)
      RETURN
C
C                    Entry to output the "convenient" restart point
      ENTRY RLUXAT(LOUT,INOUT,K1,K2)
      LOUT = LUXLEV
      INOUT = INSEED
      K1 = KOUNT
      K2 = MKOUNT
      RETURN
C
C                    Entry to initialize from one or three integers
      ENTRY RLUXGO(LUX,INS,K1,K2)
         IF (LUX .LT. 0) THEN
            LUXLEV = LXDFLT
         ELSE IF (LUX .LE. MAXLEV) THEN
            LUXLEV = LUX
         ELSE IF (LUX .LT. 24 .OR. LUX .GT. 2000) THEN
            LUXLEV = MAXLEV
            WRITE (6,'(A,I7)') ' RANLUX ILLEGAL LUXURY RLUXGO: ',LUX
         ELSE
            LUXLEV = LUX
            DO 310 ILX= 0, MAXLEV
              IF (LUX .EQ. NDSKIP(ILX)+24)  LUXLEV = ILX
  310       CONTINUE
         ENDIF
      IF (LUXLEV .LE. MAXLEV)  THEN
         NSKIP = NDSKIP(LUXLEV)
         WRITE(6,'(A,I2,A,I4)') ' RANLUX LUXURY LEVEL SET BY RLUXGO :',
     +        LUXLEV,'     P=', NSKIP+24
      ELSE
          NSKIP = LUXLEV - 24
          WRITE (6,'(A,I5)') ' RANLUX P-VALUE SET BY RLUXGO TO:',LUXLEV
      ENDIF
      IN24 = 0
      IF (INS .LT. 0)  WRITE (6,'(A)')   
     +   ' Illegal initialization by RLUXGO, negative input seed'
      IF (INS .GT. 0)  THEN
        JSEED = INS
        WRITE(6,'(A,3I12)') ' RANLUX INITIALIZED BY RLUXGO FROM SEEDS',
     +      JSEED, K1,K2
      ELSE
        JSEED = JSDFLT
        WRITE(6,'(A)')' RANLUX INITIALIZED BY RLUXGO FROM DEFAULT SEED'
      ENDIF
      INSEED = JSEED
      NOTYET = .FALSE.
      TWOM24 = 1.
         DO 325 I= 1, 24
           TWOM24 = TWOM24 * 0.5
         K = JSEED/53668
         JSEED = 40014*(JSEED-K*53668) -K*12211
         IF (JSEED .LT. 0)  JSEED = JSEED+ICONS
         ISEEDS(I) = MOD(JSEED,ITWO24)
  325    CONTINUE
      TWOM12 = TWOM24 * 4096.
         DO 350 I= 1,24
         SEEDS(I) = REAL(ISEEDS(I))*TWOM24
         NEXT(I) = I-1
  350    CONTINUE
      NEXT(1) = 24
      I24 = 24
      J24 = 10
      CARRY = 0.
      IF (SEEDS(24) .EQ. 0.) CARRY = TWOM24
C        If restarting at a break point, skip K1 + IGIGA*K2
C        Note that this is the number of numbers delivered to
C        the user PLUS the number skipped (if luxury .GT. 0).
      KOUNT = K1
      MKOUNT = K2
      IF (K1+K2 .NE. 0)  THEN
        DO 500 IOUTER= 1, K2+1
          INNER = IGIGA
          IF (IOUTER .EQ. K2+1)  INNER = K1
          DO 450 ISK= 1, INNER
            UNI = SEEDS(J24) - SEEDS(I24) - CARRY 
!            precarry=carry
	    IF (UNI .LT. 0.)  THEN
               UNI = UNI + 1.0
               CARRY = TWOM24
            ELSE
               CARRY = 0.
            ENDIF
!debug
! Looking for when uni becomes NaN. This is a dumb way to find this out
! but it should work. I think
!	    call check_seeds(uni,j24,i24,precarry)
!enddebug
            SEEDS(I24) = UNI
            I24 = NEXT(I24)
            J24 = NEXT(J24)
  450     CONTINUE
  500   CONTINUE
C         Get the right value of IN24 by direct calculation
        IN24 = MOD(KOUNT, NSKIP+24)
        IF (MKOUNT .GT. 0)  THEN
           IZIP = MOD(IGIGA, NSKIP+24)
           IZIP2 = MKOUNT*IZIP + IN24
           IN24 = MOD(IZIP2, NSKIP+24)
        ENDIF
C       Now IN24 had better be between zero and 23 inclusive
        IF (IN24 .GT. 23) THEN
           WRITE (6,'(A/A,3I11,A,I5)')  
     +    '  Error in RESTARTING with RLUXGO:','  The values', INS,
     +     K1, K2, ' cannot occur at luxury level', LUXLEV
           IN24 = 0
        ENDIF
      ENDIF
      RETURN
      END

	subroutine check_seeds(unij24,i24,carry)
!  Check for when a seed will go NaN
! Looking for when uni becomes NaN. This is a dumb way to find this out
! but it should work. I think
	real uni,carry
	integer j24,i24
	integer iuni,iseedl(25)
	PARAMETER (TWOP12=4096)
	iuni=INT(uni*TWOP12*TWOP12)
        if(iuni.lt.0)then
	   print*,' Seed for RANLUX found as NaN.'
	   print*,' Write out set of seeds that led to this.'
	   print*,' j24,i24,carry,uni,iuni:',j24,i24,carry,uni,iuni
	   call RLUXUT(ISeedl)
	   write(3,1000) iseedl
1000	   format((5i15))
	   close (3)
	   WRITE(6,1001) iseedl
1001  	format('   Random bad seed vector written into file assigned to
	1 for003: ',/,5(' ',5(5i15,/)),/)
	   stop 'BAD SEEDS about to be generated'
	endif
	end
